   close all
   clear all

%------------------------------------------------------
% FDLIB
%
% Hydrostatic shape of a two-dimensional
% sessile drop resting on a horizontal plane
% or pendant drop hanging underneath a horizontal plane,
% for a specified area and contact angle
%------------------------------------------------------

  Jsp=-1; % pendant
  Jsp= 1; % sessile

  a = 1.0;   % equivalent drop radius

  gac =1.0;  % acceleration of gravity
  rhod=1.0;  % density of the drop
  rhoa=0.0;  % density of the ambient fluid

  npts = 4*32;     %  number of interfacial markers
  epsilon=0.01; % for the shooting method

  alpha=0.25*pi;   % contact angle
  alpha=0.75*pi;   % contact angle

  maxiter=16;
  tol=0.000000001;

%---
% prepare
%---

  area = pi*a^2;
  Drho = rhod-rhoa;

%----------------------------------
% Assume drop shape
% is a truncated circle
% and compute the circle radius "acr"
% as a function of  the drop
% area and contact angle
%----------------------------------

  rcrc = sqrt(area/(alpha-0.5*sin(2.0*alpha)));
  B = 1.0/rcrc;

  figure(1)
  hold on
  xlabel('y/a','fontsize',14)
  ylabel('x/a','fontsize',14)
  set(gca,'fontsize',14)
  axis equal

  if(Jsp==1)
   axis([-2.50 2.5 -0.5 2.0])
  else
   axis([-2.50 2.5 -2.5 0.5])
  end
  box on

  if(Jsp==1)
   gamma = 16.0; % surface tension
   nloop = 16;
  else
   gamma = 16.0; % surface tension
   nloop = 7;
  end

%====
for iloop=1:nloop
%====

  capls = gamma/(abs(Drho)*gac);
  capl  = sqrt(capls);

%---
% compute the drop shape
%---

  [B,x,y] = drop_2d  ...
 ...
   (Jsp,gac,gamma,rhod,rhoa,area ...
   ,alpha,npts,epsilon,maxiter,tol ...
   ,B ...
   );

 d = -x(npts+1);
 b =  y(npts+1); % semilenth of the base

 drus = 2.0*capl*sin(0.5*alpha);

 % these two should be equal
 [1-cos(alpha)  0.5*d^2/capls+B*d]
 [area-2*b*d    2*capls*(B*b-sin(alpha))]
 pause
 

%---
%  shift to reset the origin at the wall
%---

 x = x+d;

%---
% plot
%---

 figure(1)
 plot( y,Jsp*x,'-k')
 plot(-y,Jsp*x,'-k')

%---
% circular interface
%---

 Ncrc=64;
 dtheta=alpha/Ncrc;

 xc=rcrc*cos(alpha);

 for i=1:Ncrc+1
  theta=pi-alpha+(i-1)*dtheta;
  xcrc(i)=xc+rcrc*cos(theta);
  ycrc(i)=   rcrc*sin(theta);
 end

%---
% wall
%---

 xwall(1)=-2.5;
 ywall(1)= 0;
 xwall(2)= 2.5;
 ywall(2)= 0;

%---

 if(Jsp==1)
  patch([xwall xwall(2) xwall(1)] ,[ywall ywall(2)-0.2 ywall(1)-0.2],'y')
  plot( ycrc,-xcrc,':k')
  plot(-ycrc,-xcrc,':k')
 else
  patch([xwall xwall(2) xwall(1)] ,[ywall ywall(2)+0.2 ywall(1)+0.2],'y')
  plot( ycrc,xcrc,':k')
  plot(-ycrc,xcrc,':k')
 end
 plot(xwall,ywall,'k')

 gamma = 0.75*gamma;

%===
 end % of iloop 
%===

%---
% done
%---
