      subroutine drop_2d_ode
     +
     +   (npts
     +   ,capls
     +   ,Isp
     +   ,dpsi
     +   ,shp
     +   ,x,y
     +   ,area
     +   )

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c--------------------------------------------
c Integrate ODEs by the modified Euler method
c with uniform step size
c
c Legend:
c ------
c
c xp:  x prime
c yp:  y prime
c
c capls: square of the capillary number
c shp:   shooting parameter
c area:  drop area
c--------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension x(800),y(800)

c---
c prepare
c---

      dpsih = 0.5D0*dpsi

c---
c Begin integrating from the top
c---

      psi  = 0.0D0

      x(1) = 0.0D0
      y(1) = 0.0D0

      Do i=1,npts

       If(i.eq.1) then
        xp = 0.0D0        ! top of the drop
        yp = 1.0D0/shp
       Else
        Q  = Isp*x(i)/capls-shp
        xp = Dsin(psi)/Q
        yp =-Dcos(psi)/Q
       End If

       xpsv  = xp    ! save
       ypsv  = yp

       i1 = i+1

       x(i1) = x(i)+xp*dpsi
       y(i1) = y(i)+yp*dpsi
       psi   = psi+dpsi

       Q  = Isp*x(i1)/capls-shp   ! second velocity evaluation
       xp = Dsin(psi)/Q
       yp =-Dcos(psi)/Q

       x(i1) = x(i) + (xpsv+xp)*dpsih
       y(i1) = y(i) + (ypsv+yp)*dpsih

      End Do

c--------------------------------------
c Compute area of integrated half-shape
c by the trapezoidal rule
c--------------------------------------

      area = 0.0D0

      Do i=1,npts
       area = area+(y(i)+y(i+1))*Dabs(x(i+1)-x(i))
      End Do

      area = 0.5D0*area      ! to account for trapezoidal weights

c---
c double the area to get the full shape
c---

      area = 2.0D0*area

c-----
c Done
c-----

      Return
      End
