function F = drop_2di1_ode (solution)

%=========================================
% FDLIB
%
% Copyright by C. Pozrikidis, 1999
% All rights reserved
%
% This program is to be used only under the
% stipulations of the licensing agreement
%=========================================

global area capls Isp npts beta alpha1 alpha2 x y 

%--------------------------------------------
% two-dimensional drop on an inclined plane.
%
% Integrate ODEs by RK4 with uniform
% step size for the slope angle psi
%
% SYMBOLS:
% -------
%
% dpsi: increments in psi
% C:  shooting parameter
%--------------------------------------------

%------------------
% dispense variables
%------------------

C = solution(1);
alpha2 = solution(2);

%--------
% prepare
%--------

 csb = cos(beta);
 snb = sin(beta);
 dpsi = (alpha1+alpha2)/npts;
 dpsih = 0.5*dpsi;

%----------------
% first contact point
%----------------

 psi  = -alpha1;
 x(1) = 0.0;
 y(1) = 0.0;

%----------
% integrate
%----------

 for i=1:npts
   Q  = Isp*(csb*x(i)+snb*y(i))/capls-C;
   xp = sin(psi)/Q;
   yp =-cos(psi)/Q;
   xp1 = xp;
   yp1 = yp;

   psi    = psi    +dpsih;
   x(i+1) = x(i)+xp*dpsih;
   y(i+1) = y(i)+yp*dpsih;

   Q  = Isp*(csb*x(i+1)+snb*y(i+1))/capls-C;
   xp = sin(psi)/Q;
   yp =-cos(psi)/Q;
   xp2 = xp;
   yp2 = yp;

   x(i+1) = x(i)+xp*dpsih;
   y(i+1) = y(i)+yp*dpsih;

   Q  = Isp*(csb*x(i+1)+snb*y(i+1))/capls-C;
   xp = sin(psi)/Q;
   yp =-cos(psi)/Q;
   xp3 = xp;
   yp3 = yp;

   psi    = psi  +dpsih;
   x(i+1) = x(i)+xp*dpsi;
   y(i+1) = y(i)+yp*dpsi;

   Q  = Isp*(csb*x(i+1)+snb*y(i+1))/capls-C;
   xp = sin(psi)/Q;
   yp =-cos(psi)/Q;
   xp4 = xp;
   yp4 = yp;

   x(i+1) = x(i) + (xp1+2*xp2+2*xp3+xp4)*dpsi/6.0;
   y(i+1) = y(i) + (yp1+2*yp2+2*yp3+yp4)*dpsi/6.0;

 end

 F(1) = x(npts+1);

%----------------------------------------
% compute the are of the integrated shape
% by the trapezoidal rule
%----------------------------------------

 ar = 0.0;

 for i=1:npts
   ar = ar+(x(i+1)+x(i))*(y(i+1)-y(i));
 end

 ar = 0.5*ar; % to account for trapezoidal weights

 F(2) = ar-area;

%-----
% done
%-----

 return
