function [x,s,B] = drop_ax ...
 ...
  (Jsp,gac,gamma,rhod,rhoa,volume ...
  ,alpha,npts,epsilon,maxiter,tol ...
  ,B ...
  )
 
%============================================
% FDLIB
%
% Hydrostatic shape of an axisymmetric
% sessile drop resting on a horizontal plate,
% or pendant drop hanging underneath
% a horizontal plate,
% for a specified volume and contact angle.
%
% shp: shooting parameter: B
%--------------------------------------------

 Iflag=0;  % for success

%--------
% prepare
%--------

 drho  = rhod-rhoa;            % density difference
 capls = gamma/(gac*abs(drho));  % square of the capillary number

 Isp = 1;                        % Isp is an orientation index
 if(drho<0) 
   Isp = -Isp;
 end
 if(Jsp==-1)
   Isp = -Isp;
 end

%-----------
% initial B
%-----------

  shp(1) = B;

%---
% compute the initial solution of the odes
% to start the secant method
%---

  dpsi = alpha/npts;

  Ic=1;   % counter

   [x,s,vlm] = drop_ax_ode ...
   ...
   (npts ...
   ,capls ...
   ,Isp ...
   ,dpsi ...
   ,shp(Ic) ...
   );

   error(Ic) = vlm - volume;

%-------------------------
% second start-up solution 
%-------------------------

   Ic=2;
   shp(2) = shp(1)+epsilon;

   [x,s,vlm] = drop_ax_ode ...
   ...
   (npts ...
   ,capls ...
   ,Isp ...
   ,dpsi ...
   ,shp(Ic) ...
   );

    error(Ic) = vlm - volume;

%---------------------------------------
% iterate on shp using the secant method
% until convergence
%---------------------------------------

   for iter=1:maxiter

    Ic = Ic+1;

%---
% secant updating
%---
     
   Icb = Ic-2;
   Ica = Ic-1;

   dedc = (error(Ica)-error(Icb))/(shp(Ica)-shp(Icb));
   shp(Ic) = shp(Ica)-error(Ica)/dedc;

   [x,s,vlm] = drop_ax_ode ...
   ...
   (npts ...
   ,capls ...
   ,Isp ...
   ,dpsi ...
   ,shp(Ic) ...
   );

   error(Ic) = vlm - volume;

   if(abs(error(Ic))<tol)
    break
   end

%====
   end
%====

   if(iter==maxiter)
    disp('drop_ax: ODE solver failed')
    Iflag=1;
    return
   end

%---
%  constant B
%---

 B = shp(Ic);

%---
% done
%---
 
 return
