function [x,s,Ntot,vlm,slope] = drop_ax1_ode ...
   ...
   (npts1 ...
   ,capls ...
   ,Isp ...
   ,psi_max ...
   ,B ...
   ,ratio ...
   ,Next ...
   ,npts2 ...
   )

%======================================
% FDLIB
%
% Copyright by C. Pozrikidis, 1999
% All rights reserved
%
% This program is to be used only under the
% stipulations of the licensing agreement
%======================================

%--------------------------------------------
% Integrate ODEs using RK4
%
% SYMBOLS:
% -------
%
% npts1: number of steps using the psi parametrization
% dpsi:   increments in psi
% npts2: number of steps using the x parametrization
%        
% B:  shooting parameter
%--------------------------------------------

%==============
% first section
%==============

%---------
% set the step size dpsi
% so that it increases geometrically
% by the factor "ratio"
%---------

% for i=1:npts1
%  dpsi(i) = psi_max/npts1;
% end

 if(ratio==1)
   alp  = 1.0;
   factor = 1.0/npts1;
 else
   texp = 1/(npts1-1);
   alp = ratio^texp;
   factor = (1.0-alp)/(1.0-alp^npts1);
 end

 dpsi(1) = psi_max*factor; 

 for i=2:npts1
  dpsi(i) = dpsi(i-1)*alp;
 end

%----------------
% top of the drop
%----------------

 psi  = 0.0;
 x(1) = 0.0;
 s(1) = 0.0;

 for i=1:npts1

  dpsih(i) = 0.5D0*dpsi(i);

   if(i==1) 
     xp = 0.0;
     sp = 2.0/B;
   else
     Q  = sin(psi)/s(i)+Isp*x(i)/capls-B;
     xp = sin(psi)/Q;
     sp =-cos(psi)/Q;
   end
   xp1 = xp;
   sp1 = sp;

   psi    = psi    +dpsih(i);
   x(i+1) = x(i)+xp*dpsih(i);
   s(i+1) = s(i)+sp*dpsih(i);

   Q  = sin(psi)/s(i+1)+Isp*x(i+1)/capls-B;
   xp = sin(psi)/Q;
   sp =-cos(psi)/Q;
   xp2 = xp;
   sp2 = sp;

   x(i+1) = x(i)+xp*dpsih(i);
   s(i+1) = s(i)+sp*dpsih(i);

   Q  = sin(psi)/s(i+1)+Isp*x(i+1)/capls-B;
   xp = sin(psi)/Q;
   sp =-cos(psi)/Q;
   xp3 = xp;
   sp3 = sp;

   psi    = psi    +dpsih(i);
   x(i+1) = x(i)+xp*dpsi(i);
   s(i+1) = s(i)+sp*dpsi(i);

   Q  = sin(psi)/s(i+1)+Isp*x(i+1)/capls-B;
   xp = sin(psi)/Q;
   sp =-cos(psi)/Q;
   xp4 = xp;
   sp4 = sp;

   x(i+1) = x(i) + (xp1+2*xp2+2*xp3+xp4)*dpsi(i)/6.0;
   s(i+1) = s(i) + (sp1+2*sp2+2*sp3+sp4)*dpsi(i)/6.0;

   end

 Ntot=npts1;

%==============
% continue the integration
% with a uniform step "dx"
% using the x parametrization
% up to x = Next*x(npts1+1)
%==============

 dx = Next*x(npts1+1)/npts2;

%---
% initial slope
%---

 Q  = sin(psi)/s(npts1+1)+Isp*x(npts1+1)/capls-B;
 xp = sin(psi)/Q;
 sp =-cos(psi)/Q;
 q(npts1+1) = xp/sp; % dx/dsigma

%---
% integrate
%---

 for i=npts1+1:npts1+npts2+2

  dsdx = q(i);
  tmp = 1+q(i)*q(i);
  dqdx = tmp*( 1/s(i)+sqrt(tmp)*(Isp*x(i)/capls-B) );
  dsdx1 = dsdx;
  dqdx1 = dqdx;

  x(i+1) = x(i)+0.5*dx;
  s(i+1) = s(i)+0.5*dx*dsdx;
  q(i+1) = q(i)+0.5*dx*dqdx;

  dsdx = q(i+1);
  tmp = 1+q(i+1)*q(i+1);
  dqdx = tmp*( 1/s(i+1)+sqrt(tmp)*(Isp*x(i+1)/capls-B) );
  dsdx2 = dsdx;
  dqdx2 = dqdx;

  x(i+1) = x(i)+0.5*dx;
  s(i+1) = s(i)+0.5*dx*dsdx;
  q(i+1) = q(i)+0.5*dx*dqdx;

  dsdx = q(i+1);
  tmp = 1+q(i+1)*q(i+1);
  dqdx = tmp*( 1/s(i+1)+sqrt(tmp)*(Isp*x(i+1)/capls-B) );
  dsdx3 = dsdx;
  dqdx3 = dqdx;

  x(i+1) = x(i)+dx;
  s(i+1) = s(i)+dx*dsdx;
  q(i+1) = q(i)+dx*dqdx;

  dsdx = q(i+1);
  tmp = 1+q(i+1)*q(i+1);
  dqdx = tmp*( 1/s(i+1)+sqrt(tmp)*(Isp*x(i+1)/capls-B) );
  dsdx4 = dsdx;
  dqdx4 = dqdx;

  s(i+1) = s(i)+dx*(dsdx1+2.0*dsdx2+2.0*dsdx3+dsdx4)/6.0;
  q(i+1) = q(i)+dx*(dqdx1+2.0*dqdx2+2.0*dqdx3+dqdx4)/6.0;

  Ntot=Ntot+1;
  if(s(i+1)<0)
    break;
  end

 end

 slope=q(i+1);

%-------------------------------------------
% compute the volume of the integrated shape
% by the trapezoidal rule
%-------------------------------------------

 vlm = 0.0;

 for i=1:Ntot
   vlm = vlm+(s(i+1)*s(i+1)+s(i)*s(i))*abs(x(i+1)-x(i));
 end

 vlm = 0.5*vlm; % to account for trapezoidal weights
 vlm = pi*vlm;

%-----
% done
%-----

 return
