function [Iflag,x,x1,hmen] = men_2d ...
 ...
  (b,gac,gamma,rhop,rhoa ...
  ,alpha,npts,epsilon,maxiter,tol ...
  ,hin ...
  )

%-------------------------------------------
% shape of a 2D meniscus between
% two vertical parallel plates computed
% by a shooting method for h
%-------------------------------------------

 Iflag = 0;  % flag for success

%-----
% prepare
%----

 drho  = rhop-rhoa ;             % density difference
 capls = gamma/(gac*abs(drho));  % square of the capillary length

%-----------------------------------------
% initial guess for h
% computed by assuming a circular interface
%-----------------------------------------

h(1) = hin;

if(abs(alpha-0.5*pi)<0.0000001)
  cota = 0.0;
else
  cota = 1.0/tan(alpha);
end

%---
% compute the first solution of the odes
% to start-up the secant method
%---

  Ic=1;   % counter

  [x,x1,x2] = men_2d_ode(npts,capls,b,h(Ic));
  error(Ic) = x2(npts+1)-cota;

%-------------------------
% second start-up solution 
%-------------------------

  Ic=2;

  h(2) = h(1)+epsilon;
  [x,x1,x2] = men_2d_ode(npts,capls,b,h(Ic));
  error(Ic) = x2(npts+1)-cota;

%---------------------------------------
% iterate hshp using the secant method
% until convergence
%---------------------------------------

 for iter=1:maxiter

  Ic = Ic+1;

%---
% secant updating
%---
     
  Icb = Ic-2;
  Ica = Ic-1;

  dedh = (error(Ica)-error(Icb))/(h(Ica)-h(Icb));
  h(Ic) = h(Ica)-error(Ica)/dedh;

  [x,x1,x2] = men_2d_ode (npts,capls,b,h(Ic));

  error(Ic) = x2(npts+1)-cota;

  if(abs(error(Ic))<tol)
    break
  end

%---
   end
%---

   if(iter==maxiter)
    disp('men_2d: ODE solver failed')
    Iflag=1;
    return
   end

   hmen=h(Ic);

%---
% done
%---
 
 return
