      subroutine men_2d_ode
     +
     +  (npts
     +  ,capls
     +  ,b
     +  ,h
     +  ,x,f,g
     +  )

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licencing agreement.
c----------------------------------------

c--------------------------------------------
c Integrate the following ODEs by the explicit Euler method
c with uniform step size dx
c
c SYMBOLS:
c -------
c
c df/dx = g        (definition)
c dg/dx = (f+h) (1.0+g**2)**1.5 / capls
c
c Integration interval: (0, b)
c
c Initial condition f(0) = 0, g(0) = 0
c--------------------------------------

      Implicit double precision (a-h,o-z)

      Dimension x(800),f(800),g(800)

c--------
c prepare
c--------

      dx  = b/(npts+1.0-1.0)     ! uniform x step
      dxh = 0.5*dx

c----------
c mid-plane
c----------

      x(1) = 0.0
      f(1) = 0.0
      g(1) = 0.0

c----------------------------
c integrate by second-order RK
c----------------------------

      Do i=1,npts

       If(i.eq.1) then
        fp = 0.0         ! value at mid-plane
        gp = h/capls     ! value at mid-plane
       Else
        fp = g(i)
        gp = (f(i)+h)*sqrt( (1.0+g(i)**2)**3 )/capls
       End If

       fsv = f(i)    ! save
       gsv = g(i)    ! save

       fpsv = fp
       gpsv = gp

       i1 = i+1

       x(i1) = x(i) +    dx
       f(i1) = f(i) + fp*dx
       g(i1) = g(i) + gp*dx

       fp = g(i1)        ! second velocity evaluation
       gp = (f(i1)+h)*sqrt( (1.0+g(i1)**2)**3 )/capls

       f(i1) = fsv + (fpsv+fp)*dxh
       g(i1) = gsv + (gpsv+gp)*dxh

      End Do

c-----
c Done
c-----

      Return
      End
