function obj = fcircles(xcontact)

global d a1 a2 A
global Nxi Net
global xi1 xi2
global xi cshxi snhxi
global eta cseta sneta
global Niter tol
global capls
global x y z J Ikeep
global ycnt1 ycnt2
global forcex1 forcey1 forcez1 torqux1 torquy1 torquz1
global forcex2 forcey2 forcez2 torqux2 torquy2 torquz2
global cang1 cang2
global cangle1 cangle2

%========================================
% meniscus in the exterior of two circles
%========================================

%---
% grid
%---

Dxi = (xi2-xi1)/Nxi;
Deta = 2.0*pi/Net;
Dxi2 = 2.0*Dxi;
Deta2 = 2.0*Deta;

%=============
% solve the capillary equation
%=============

% first circle:

 Ic = 0;

 for j=1:Net
  Ic=Ic+1;
  x(1,j) = xcontact(Ic);
 end
 x(1,Net+1) = x(1,1);
 x(1,Net+2) = x(1,2);

% second circle:

 for j=1:Net
  Ic=Ic+1;
  x(Nxi+1,j) = xcontact(Ic);
 end
 x(Nxi+1,Net+1) = x(Nxi+1,1);
 x(Nxi+1,Net+2) = x(Nxi+1,2);

%----
% prepare for iterations
%----

 nonlinear = 0;
 omega = 0.5;
 Iflag = 0;

%========
for iterate=1:Niter
%========

%---
% compute dx/dy and dx/dz
%---

for i=2:Nxi
 for j=2:Net+1
  den = cshxi(i)+cseta(j);
  MAT(1,1) =  A*(cshxi(i)*cseta(j)+1.0)/den^2;
  MAT(1,2) = -A*snhxi(i)*sneta(j)/den^2;
  MAT(2,1) = -MAT(1,2);
  MAT(2,2) =  MAT(1,1);
% det(MAT)
% J(i,j)^2
% inv(MAT)
% MAT'/J(i,j)^2
% pause
  RHS(1) = (x(i+1,j)-x(i-1,j))/Dxi2;
  RHS(2) = (x(i,j+1)-x(i,j-1))/Deta2;
  SOL = RHS/MAT';
  dxdy(i,j) = SOL(1);
  dxdz(i,j) = SOL(2);
 end
 % wrap:
 dxdy(i,1) = dxdy(i,Net+1);
 dxdz(i,1) = dxdz(i,Net+1);
 dxdy(i,Net+2) = dxdy(i,2);
 dxdz(i,Net+2) = dxdz(i,2);
end

% boundary values
% around the cylinders
% needed for the second derivatives

for j=1:Net+2
% dxdy(1,j) = 2.0*dxdy(2,j) -dxdy(3,j);
% dxdz(1,j) = 2.0*dxdz(2,j) -dxdz(3,j);
  dxdy(1,j) = 3.0*dxdy(2,j)-3.0*dxdy(3,j)+dxdy(4,j);
  dxdz(1,j) = 3.0*dxdz(2,j)-3.0*dxdz(3,j)+dxdz(4,j);
% dxdy(Nxi+1,j) = 2.0*dxdy(Nxi,j)-dxdy(Nxi-1,j);
% dxdz(Nxi+1,j) = 2.0*dxdz(Nxi,j)-dxdz(Nxi-1,j);
  dxdy(Nxi+1,j) = 3.0*dxdy(Nxi,j)-3.0*dxdy(Nxi-1,j)+dxdy(Nxi-2,j);
  dxdz(Nxi+1,j) = 3.0*dxdz(Nxi,j)-3.0*dxdz(Nxi-1,j)+dxdz(Nxi-2,j);
end

%---
% now compute the second derivatives
%---

for i=2:Nxi
 for j=2:Net+1
  den = cshxi(i)+cseta(j);
  MAT(1,1) =  A*(cshxi(i)*cseta(j)+1.0)/den^2;
  MAT(1,2) = -A*snhxi(i)*sneta(j)/den^2;
  MAT(2,1) = -MAT(1,2);
  MAT(2,2) =  MAT(1,1);
  RHS(1) = (dxdy(i+1,j)-dxdy(i-1,j))/Dxi2;
  RHS(2) = (dxdy(i,j+1)-dxdy(i,j-1))/Deta2;
  SOL = RHS/MAT';
  dxdyy(i,j) = SOL(1);
  dxdyz(i,j) = SOL(2);
  RHS(1) = (dxdz(i+1,j)-dxdz(i-1,j))/Dxi2;
  RHS(2) = (dxdz(i,j+1)-dxdz(i,j-1))/Deta2;
  SOL = RHS/MAT';
  dxdzy(i,j) = SOL(1);
  dxdzz(i,j) = SOL(2);
 end
 % wrap:
 dxdyy(i,1) = dxdyy(i,Net+1);
 dxdyz(i,1) = dxdyz(i,Net+1);
 dxdzy(i,1) = dxdzy(i,Net+1);
 dxdzz(i,1) = dxdzz(i,Net+1);
 dxdyy(i,Net+2) = dxdyy(i,2);
 dxdyz(i,Net+2) = dxdyz(i,2);
 dxdzy(i,Net+2) = dxdzy(i,2);
 dxdzz(i,Net+2) = dxdzz(i,2);
end

%---
% point Gauss-Seidel:
%---

err = 0.0;

xneo = x;

for i=2:Nxi
  for j=2:Net+1

  if(Ikeep(i,j) == 1)
   JS = J(i,j)*J(i,j);
   fL = x(i-1,j); 
   fR = x(i+1,j);
   fB = x(i,j-1);
   fT = x(i,j+1);
   if(Ikeep(i-1,j)==0) fL = 0; end
   if(Ikeep(i+1,j)==0) fR = 0; end
   if(Ikeep(i,j-1)==0) fB = 0; end
   if(Ikeep(i,j+1)==0) fT = 0; end
   xnew = ((fR+fL)/Dxi^2+(fT+fB)/Deta^2)/JS;
%  nonlinear = 0;                       % linearize
%~~~
   if(nonlinear==1)
    xnew = xnew +     dxdz(i,j)^2        *dxdyy(i,j);
%   xnew = xnew - 2.0*dxdy(i,j)*dxdz(i,j)*dxdyz(i,j);
    xnew = xnew -     dxdy(i,j)*dxdz(i,j)*(dxdyz(i,j)+dxdzy(i,j));
    xnew = xnew +     dxdy(i,j)^2        *dxdzz(i,j);
%   xnew = xnew +     dxdz(i,j)^2;
%   xnew = xnew - 2.0*dxdy(i,j)*dxdz(i,j);
%   xnew = xnew +     dxdy(i,j)^2;
   end
%~~~
   tmp = 1.0+dxdy(i,j)^2+dxdz(i,j)^2;
   G = 2.0*(1.0/Dxi^2+1.0/Deta^2)/JS + tmp^(3/2)/capls;
%  G = 2.0*(1.0/Dxi^2+1.0/Deta^2)/JS + 1.0/capls;  % linearize
   xnew = xnew/G;
   diff = abs(x(i,j)-xnew);
   if(diff>err) err=diff; end
   xneo(i,j) = x(i,j)+omega*(xnew-x(i,j));
  end

 end
end

%[iterate, nonlinear, err]
%err

 x = xneo;

%---
% wrap
%---

for i=1:Nxi+1
  x(i,1)     = x(i,Net+1);
  x(i,Net+2) = x(i,2);
end

%---
% stopping check
%---

if(err<10.0*tol)
  nonlinear = 1;
  omega = 0.2;
end

if(err<tol)
  Iflag = 1;
  break;
end

%---
end  % of iterations
%---

if(Iflag == 0)
 disp('iterations did not coverge')
 pause
end

%============
% compute dxdxi, dxdeta,
% the surface normal and tension vector
% around the contact lines
%============

x0 = 0.0;
y0 = ycnt1;
z0 = 0.0;

% first contact line

for j=1:Net+1
 dxdxi1(j) = (-x(3,j)+4.0*x(2,j)-3.0*x(1,j))/Dxi2;
 den = cshxi(1)+cseta(j);
 den2 = den*den;
 dydxi1(j) =  A*(cshxi(1)*cseta(j)+1.0)/den2;
 dzdxi1(j) = -A*snhxi(1)*sneta(j)/den2;
 if(j==1)
  dxdeta1(j) = (x(1,2)-x(1,Net))/Deta2;
 else
  dxdeta1(j) = (x(1,j+1)-x(1,j-1))/Deta2;
 end
 dydeta1(j) =-dzdxi1(j);
 dzdeta1(j) = dydxi1(j);
 vnx1(j) = dydxi1(j)*dzdeta1(j)-dzdxi1(j)*dydeta1(j);
 vny1(j) = dzdxi1(j)*dxdeta1(j)-dxdxi1(j)*dzdeta1(j);
 vnz1(j) = dxdxi1(j)*dydeta1(j)-dydxi1(j)*dxdeta1(j);
 vnm1(j) = sqrt(vnx1(j)^2+ vny1(j)^2+vnz1(j)^2);
 vnx1(j) = vnx1(j)/vnm1(j);
 vny1(j) = vny1(j)/vnm1(j);
 vnz1(j) = vnz1(j)/vnm1(j);

  Iskip =1;
  if(Iskip==0)
   den = cshxi(1)+cseta(j);
   MAT(1,1) =  A*(cshxi(1)*cseta(j)+1.0)/den^2;
   MAT(1,2) = -A*snhxi(1)*sneta(j)/den^2;
   MAT(2,1) = -MAT(1,2);
   MAT(2,2) =  MAT(1,1);
   RHS(1) = dxdxi1(j);
   RHS(2) = dxdeta1(j);
   SOL = RHS/MAT';
   dxdy1(j) = SOL(1);
   dxdz1(j) = SOL(2);
   vnm1(j) = sqrt(1+SOL(1)^2+SOL(2)^2);
   vnx1(i) = 1.0/vnm1(j);
   vny1(i) = -dxdy1(j)/vnm1(j);
   vnz1(i) = -dxdz1(j)/vnm1(j);
  end

 tngx1(j) = dydeta1(j)*vnz1(j)-dzdeta1(j)*vny1(j);
 tngy1(j) = dzdeta1(j)*vnx1(j)-dxdeta1(j)*vnz1(j);
 tngz1(j) = dxdeta1(j)*vny1(j)-dydeta1(j)*vnx1(j);
 crsx1(j) = (y(1,j)-y0)*tngz1(j)-(z(1,j)-z0)*tngy1(j);
 crsy1(j) = (z(1,j)-z0)*tngx1(j)-(x(1,j)-x0)*tngz1(j);
 crsz1(j) = (x(1,j)-x0)*tngy1(j)-(y(1,j)-y0)*tngx1(j);
end

x0 = 0.0;
y0 = ycnt2;
z0 = 0.0;

% second contact line

for j=1:Net+1
 dxdxi2(j) = -(-x(Nxi-1,j)+4.0*x(Nxi,j)-3.0*x(Nxi+1,j))/Dxi2;
 den = cshxi(Nxi+1)+cseta(j);
 den2 = den*den;
 dydxi2(j) =  A*(cshxi(Nxi+1)*cseta(j)+1.0)/den2;
 dzdxi2(j) = -A* snhxi(Nxi+1)*sneta(j)/den2;
 if(j==1)
  dxdeta2(j) = (x(Nxi+1,2)-x(Nxi+1,Net))/Deta2;
 else
  dxdeta2(j) = (x(Nxi+1,j+1)-x(Nxi+1,j-1))/Deta2;
 end
 dydeta2(j) =-dzdxi2(j);
 dzdeta2(j) = dydxi2(j);
 vnx2(j) = dydxi2(j)*dzdeta2(j)-dzdxi2(j)*dydeta2(j);
 vny2(j) = dzdxi2(j)*dxdeta2(j)-dxdxi2(j)*dzdeta2(j);
 vnz2(j) = dxdxi2(j)*dydeta2(j)-dydxi2(j)*dxdeta2(j);
 vnm2(j) = sqrt(vnx2(j)^2+ vny2(j)^2+vnz2(j)^2);
 vnx2(j) = vnx2(j)/vnm2(j);
 vny2(j) = vny2(j)/vnm2(j);
 vnz2(j) = vnz2(j)/vnm2(j);

 tngx2(j) = dydeta2(j)*vnz2(j)-dzdeta2(j)*vny2(j);
 tngy2(j) = dzdeta2(j)*vnx2(j)-dxdeta2(j)*vnz2(j);
 tngz2(j) = dxdeta2(j)*vny2(j)-dydeta2(j)*vnx2(j);
 tngx2(j) = -tngx2(j);
 tngy2(j) = -tngy2(j);
 tngz2(j) = -tngz2(j);

 crsx2(j) = (y(Nxi+1,j)-y0)*tngz2(j)-(z(Nxi+1,j)-z0)*tngy2(j);
 crsy2(j) = (z(Nxi+1,j)-z0)*tngx2(j)-(x(Nxi+1,j)-x0)*tngz2(j);
 crsz2(j) = (x(Nxi+1,j)-x0)*tngy2(j)-(y(Nxi+1,j)-y0)*tngx2(j);
end

%---
% first cylinder normal vector
%---

for j=1:Net+1
 vnpx1(j) = 0.0;
 vnpy1(j) = y(1,j)-ycnt1;
 vnpz1(j) = z(1,j);
 vnpm1(j) = sqrt(vnpx1(j)^2+vnpy1(j)^2+vnpz1(j)^2);
 vnpx1(j) = vnpx1(j)/vnpm1(j);
 vnpy1(j) = vnpy1(j)/vnpm1(j);
 vnpz1(j) = vnpz1(j)/vnpm1(j);
end

%---
% second cylinder normal vector
%---

for j=1:Net+1
 vnpx2(j) = 0.0;
 vnpy2(j) = y(Nxi+1,j)-ycnt2;
 vnpz2(j) = z(Nxi+1,j);
 vnpm2(j) = sqrt(vnpx2(j)^2+vnpy2(j)^2+vnpz2(j)^2);
 vnpx2(j) = vnpx2(j)/vnpm2(j);
 vnpy2(j) = vnpy2(j)/vnpm2(j);
 vnpz2(j) = vnpz2(j)/vnpm2(j);
end

%---
% contact angles
%---

for j=1:Net+1
  prj = vnx1(j)*vnpx1(j) ...
       +vny1(j)*vnpy1(j) ...
       +vnz1(j)*vnpz1(j);
  cangle1(j) = acos(prj);
end

for j=1:Net+1
  prj = vnx2(j)*vnpx2(j) ...
       +vny2(j)*vnpy2(j) ...
       +vnz2(j)*vnpz2(j);
  cangle2(j) = acos(prj);
end

%---
% objective function
%---

for j=1:Net
  obj(j)     = cangle1(j)-cang1;
  obj(j+Net) = cangle2(j)-cang2;
end

%---
% force and torque
%---

forcex1 = 0.0;forcey1 = 0.0;
forcez1 = 0.0;
torqux1 = 0.0;torquy1 = 0.0;
torquz1 = 0.0;

forcex2 = 0.0;forcey2 = 0.0;
forcez2 = 0.0;
torqux2 = 0.0;torquy2 = 0.0;
torquz2 = 0.0;

for j=1:Net

  forcex1 = forcex1+tngx1(j);
  forcey1 = forcey1+tngy1(j);
  forcez1 = forcez1+tngz1(j);
  torqux1 = torqux1+crsx1(j);
  torquy1 = torquy1+crsy1(j);
  torquz1 = torquz1+crsz1(j);

  forcex2 = forcex2+tngx2(j);
  forcey2 = forcey2+tngy2(j);
  forcez2 = forcez2+tngz2(j);
  torqux2 = torqux2+crsx2(j);
  torquy2 = torquy2+crsy2(j);
  torquz2 = torquz2+crsz2(j);

end

forcex1 = forcex1*Deta;
forcey1 = forcey1*Deta;
forcez1 = forcez1*Deta;

torqux1 = torqux1*Deta;
torquy1 = torquy1*Deta;
torquz1 = torquz1*Deta;

forcex2 = forcex2*Deta;
forcey2 = forcey2*Deta;
forcez2 = forcez2*Deta;
torqux2 = torqux2*Deta;
torquy2 = torquy2*Deta;
torquz2 = torquz2*Deta;

%[forcex1, forcey1, forcez1, torqux1, torquy1, torquz1]
%[forcex2, forcey2, forcez2, torqux2, torquy2, torquz2]

%---
% done
%---

return
