      program spf

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c-----------------------------------------------
c Similarity solutions for stagnation-point flows
c
c The problem is reduced
c to solving a boundary-value problem involving 
c ODEs
c
c-----
c
c 2D Oblique stagnation-point flow
c
c see Pozrikidis (1997, pp. 205-208)
c
c    Six ODEs
c
c    x1 = F, x2 = M, x3 = N
c    x4 = G, x5 = K, x6 = L
c
c----
c
c Axisymmetric orthogonal stagnation-point flow
c (Homann flow)
c
c see Pozrikidis (1997, pp. 208-211)
c
c   Three ODEs
c
c---
c
c swirling von Karman flow
c
c see Pozrikidis (1997, pp. 211-214)
c
c Five ODEs
c
c-------
c
c SYMBOLS:
c -------
c
c IRK    Runge-Kutta selector (1 or 2)
c Nstep  number of steps per batch
c N      Number of equations
c
c----------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension x(20),u(20),xs(20),us(20)
      Dimension xinit(20),sol(20,0:1000)

      common/fs/rm

c----------
c constants
c----------

      Null = 0

c---
c input
c---

      write (6,*) 
      write (6,*) 'Enter'
      write (6,*) 
      write (6,*) '1 for 2D oblique stagnation-point flow'
      write (6,*) '2 for Homann flow'
      write (6,*) '3 for swirling von Karman flow'
      write (6,*) '0 to quit'
      write (6,*) '---------'
      read  (5,*) Menu

      If(menu.eq.0) Go to 99

      write (6,*) 
      write (6,*) ' Enter 1 to select the RK1'
      write (6,*) '       2 to select the RK2'
      write (6,*) '--------------------------'
      write (6,*) 
      read  (5,*) IRK

      write (6,*) 
      write (6,*) ' Enter the integration step'
      write (6,*) '---------------------------'
      write (6,*) 
      read  (5,*) Dt

c-------------------
c set number of ODEs
c and initialize
c-------------------

      if(menu.eq.1) N=6
      if(menu.eq.2) N=3
      if(menu.eq.3) N=5

      Do j=1,N
        xinit(j) = 0.0
        x(j)     = xinit(j)
      End Do

c-------------------
c initial conditions
c-------------------

c-----------------------
      if(menu.eq.1) then
c-----------------------

c        write (6,*) 
c        write (6,*) " Enter the initial values for x(3) and x(6)"
c        write (6,*) " ------------------------------------------"
c        read  (5,*) xinit(3),xinit(6)

         xinit(3) = 1.2326      ! these are the correct values
         xinit(6) = 1.406544

         x(3) = xinit(3)
         x(6) = xinit(6)

c----------------------------
      elseif(menu.eq.2) then
c----------------------------

c        write (6,*)
c        write (6,*) " Please enter the initial value for xin(3)"
c        write (6,*) " -----------------------------------------"
c        read  (5,*) xinit(3)

         xinit(3) = 1.3120      ! this is the correct value

         x(3) = xinit(3)

c----------------------------
      elseif(menu.eq.3) then
c----------------------------

c        write (6,*)
c        write (6,*) " Enter the initial values for xin(3) and xin(5)"
c        write (6,*) " ----------------------------------------------"
c        read  (5,*) xinit(3),xinit(5)

         xinit(3) = -1.02046    ! these are the correct values
         xinit(4) =  1.0
         xinit(5) = -0.6159 

         x(3) = xinit(3)
         x(4) = xinit(4)
         x(5) = xinit(5)

c-----------
      End If
c-----------

      close (1)

c---
c open output files
c---

      open (2,file="spf.out")

  98  Continue 

      write (6,*) 
      write (6,*) ' Enter the number of steps before inquiring'
      write (6,*) '-------------------------------------------'
      write (6,*) 
      read  (5,*) Nstep

c---------------
c integrate odes
c---------------

      Kstep = 0    ! batch step counter
      Istep = 0    ! batch step counter

      time  = 0.0  ! phase-space time

c-------------------------
c print initial conditions
c-------------------------

      write (2,100) Kstep,time,(x(j),j=1,n)

c---
c print values corresponding
c to the objective functions
c---

      if(menu.eq.1) then

        write (6,*)
        write (6,*) " Objective functions should tend to 1.0, 1.0"
        write (6,*)
        write (6,100) Kstep,time,x(2),x(6)

      elseif(menu.eq.2) then

        write (6,*)
        write (6,*) " Objective function should tend to 1.0"
        write (6,*)
        write (6,100) Kstep,time,x(2)

      elseif(menu.eq.3) then

        write (6,*)
        write (6,*) " Objective functions should tend to 0, 0"
        write (6,*)
        write (6,100) Kstep,time,x(2),x(4)

      End If

      Do j=1,N
       sol(j,Kstep) = x(j)
      End Do

 92   Continue

      call odes (Menu,N,x,u)

c----------------------
       if(IRK.eq.1) then    ! first order RK
c----------------------

         Do j=1,N
           x(j) = x(j) + u(j)*dt
           sol(j,Kstep) = x(j)
         End Do

c----------------------------
       elseif(IRK.eq.2) then    ! second order RK
c----------------------------

          Do j=1,N
            xs(j) = x(j)
            us(j) = u(j)
            x (j) = x(j) + u(j)*dt
          End Do

          call odes (Menu,N,x,u)

          Do j=1,N
            x(j) = xs(j) + 0.5*(us(j)+u(j))*Dt
            sol(j,Kstep) = x(j)
          End Do

c--------------
         End If 
c--------------

c---
c reset indices and time
c---

      Istep = Istep + 1
      Kstep = Kstep + 1

      time = time + Dt

      write (2,100) Kstep,Time,(x(j),j=1,n)

c-----------------------
c print objective values
c-----------------------

      If(menu.eq.1) then
        write (6,100) Kstep,Time,x(2),x(6)
      Else If(menu.eq.2) then
        write (6,100) Kstep,Time,x(2)
      Else If(menu.eq.3) then
        write (6,100) Kstep,Time,x(2),x(4)
      End If

c-----------------
c stopping inquiry
c-----------------

      If(Istep.eq.Nstep) then
        write (6,*)
        write (6,*)  " Enter the new batch of steps"
        write (6,*)  " 0 to quit"
        write (6,*)  " ---------"
        read  (5,*)  Nstep

        If(Nstep.eq.0) Go to 93

        Istep = 0    ! step counter
        Go to 92

      End If

      Go to 92

c------------------------
c End of integrating ODEs
c------------------------

   93 Continue

      write (2,100) Null

      write (6,*) 
      write (6,*) ' Shoot with different initial conditions ?'
      write (6,*) 
      write (6,*) ' Enter 1 for yes, 0 for No'
      write (6,*) ' -------------------------'
      read  (5,*) Irepeat

      If(Irepeat.eq.0) Go to 99

      write (6,*)
      write (6,*) " Old initial values were:"

      Do i=1,N
        write (6,100) i,xinit(i)
      End Do

c--------------------
c zero initial values
c--------------------

      Do j=1,N
       xinit(j) = 0.0
       x    (j) = 0.0
      End Do

      if(menu.eq.1) then

         write (6,*) 
         write (6,*) " Enter new initial values for x(3) and x(6)"
         write (6,*) " ------------------------------------------"
         read  (5,*) xinit(3),xinit(6)

         x(3) = xinit(3)
         x(6) = xinit(6)

      elseif(Menu.eq.2) then

         write (6,*)
         write (6,*) " Please enter new initial value for xin(3)"
         write (6,*) " -----------------------------------------"
         read  (5,*) xinit(3)

         x(3) = xinit(3)

      elseif(Menu.eq.3) then

         write (6,*)
         write (6,*) " Enter new initial values for xin(3) and xin(5)"
         write (6,*) " ----------------------------------------------"
         read  (5,*) xinit(3),xinit(5)

         x(3) = xinit(3)
         x(5) = xinit(5)

      End If

      Go to 98

c-----
c Done
c-----

   99  Continue

       Close (2)

  100  Format (1x,i3,10(1x,f9.5))

       Stop 
       End
