      subroutine sf_orifice_3D
     +
     +   (x0,y0,z0
     +   ,a
     +   ,x,y,z
     +   ,ux,uy,uz
     +   )

c---------------------------------------
c Stokes flow:
c
c Velocity for simple shear flow over a
c circular orifice on a plane wall
c
c The z axis is perpendicular to the wall
c
c SYMBOLS:
c -------
c
c (x0,y0,z0): center of the orifice
c (x,y,z):    velocity point
c--------------------------------------

      Implicit Double Precision (a-h,o-z)

c---
c constants
c---

      pi  = 3.14159 265358
      pi3 = 3.0*pi

c---
c shift the origin
c---

      xx = x-x0
      yy = y-y0
      zz = z-z0

      rhos = xx*xx+yy*yy
      rho  = sqrt(rhos)
      thet = acos(xx/rho)
      if(yy.lt.0) thet = -thet
      thet2 = 2.0*thet

      as   = a*a
      zzs  = zz*zz
      rs   = rhos+zzs
      rlms = 0.5*(rs-as+sqrt((as-rs)**2+4.0*zzs*as))
      arlm = sqrt(rlms)
      rlm  = arlm * zz/abs(zz)

      zet  = zz/rlm     ! dimensionless
      zets = zet*zet    ! dimensionless
      xis  = rlms/as    ! dimensionless

      cs  = cos(thet)
      cs2 = cos(thet2)
      sn2 = sin(thet2)

      fc1 = a/pi3 *xis*zet*(1.0-zets)
     +                  /((xis+1.0)*(xis+zets))

      fc2 = a/pi *zet*(1.0-arlm/a * atan(a/arlm)
     +                    - zets/(3.0*(xis+zets)) 
     +                )

      fc3 = 2.0*rlm/pi3 * zets/(xis+zets)
     +                  * sqrt((1.0-zets)/(1.0+xis))

      ux = cs2*fc1+fc2
      uy = sn2*fc1
      uz = cs *fc3

c---
c add simple shear flow above the wall
c---

      if(zz.gt.0) ux = ux+zz

c---
c done
c---

      return
      end
