      subroutine velocity (menu,x,y,u,v)

c===========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement
c===========================================

c----------------------------------------
c Computes the velocity components u and v
c at the point x,y
c----------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension xblas(800),fblas(800),fpblas(800)
      Dimension xspf(800),fspf(800),gspf(800)
      Dimension x_hom(800),f_hom(800)

c--------------
c common blocks
c--------------

      common/atlantic1/strength,RL,sig0,rlam,Vx,Vy,circ,Re,chi,cc
      common/atlantic2/wall,wall1,wall2,wall3,width
      common/atlantic3/xc,yc,zc,a,b,aob,boa,xs,ys,zs,R,xsi,ysi,zsi
      common/atlantic4/alpha,beta,gama,rm,g0,g1,blob_rad,Dcn
      common/atlantic5/xpv,ypv,xr,sr,ypv1,ypv2
      common/atlantic6/xblas,fblas,fpblas,xspf,fspf,gspf,xhom,fhom
      common/atlantic7/Nblas,Nspf,Nhom
      common/atlantic8/vsr
      common/atlantic9/rlr,rli,sc
      common/pacific1/max1,max2,max3
      common/pacific2/a11,a12,a13,a21,a22,a23,a31,a32,a33
      common/velvet/Iframe,Iflow,Isym,Ieddies
      common/numdiff/eps,eps2

c----
c constants
c---

      pi  = 3.14159 265358
      pi2 = 2.0*pi
      pi4 = 4.0*pi
      pi8 = 8.0*pi

c----
c initialize
c---

      x_orfc = 0.0   ! center of the orifice
      y_orfc = 0.0
      z_orfc = 0.0

c=================
c  POTENTIAL FLOWS
c=================

c------------------------------
c flow due to a 2D point source
c located at the point (xs, ys)
c------------------------------

      if(menu.eq.101) then

      rs = (x-xs)**2+(y-ys)**2

      u = (x-xs)/rs/pi2
      v = (y-ys)/rs/pi2

c----------------------------------------------
c flow due to a periodic array of point sources
c separated by distance equal to RL
c
c one of the point sources is located at the 
c point (xs, ys)
c----------------------------------------------

      elseif(menu.eq.102) then

      wn = pi2/RL          ! wave number
      A  = wn*(y-ys)
      B  = wn*(x-xs)
      C  = cosh(A)-cos(B)

      cf = strength/(2.0*RL)

      u = cf *  sin(B)/C
      v = cf * sinh(A)/C

c-------------------------------------------------
c flow due to a 2D point source above a plane wall 
c-------------------------------------------------

      elseif(menu.eq.103) then

      strength = 1.0

      Dists = (x-xs)**2+(y-ys)**2      !  point source
      u     = (x-xs)/Dists
      v     = (y-ys)/Dists

      Dists = (x-xsi)**2+(y-ysi)**2    ! image point source
      u = u + (x-xsi)/Dists
      v = v + (y-ysi)/Dists

      cf = strength/pi2
      u = cf*u
      v = cf*v

c-------------------------------------------
c flow due to a 2D point source in a channel
c confined between two parallel walls
c-------------------------------------------

      elseif(menu.eq.104) then

      strength = 1.0

      h  = wall2-wall1
      h2 = 2.0*h
      wn = pi2/h2          ! wave number

      dx = x-xs
      dy = y-ys

      A = wn*dx
      B = wn*dy
      C = cosh(A)
      D = C-cos(B)

      dyi = y-2.0*wall1+ys     ! image system

      Bi  = wn*dyi
      Di  = C-cos(Bi)

      cf = strength /(2.0*h2)

      u = cf * (sinh(A)/D+sinh(A )/Di)
      v = cf * (sin (B)/D+sin (Bi)/Di)

c-----------------------------------
c flow due to a 2D point source in a
c semi-infinite strip
c-----------------------------------

      elseif(menu.eq.105) then

      strength = 1.0

      h  = wall2-wall1
      h2 = 2.0*h
      wn = pi2/h2          ! wave number

      cf = strength /(2.0*h2)

      dx  = x-xs
      dxi = x-2.0*wall1+xs
      A   = wn*dx
      Ai  = wn*dxi

      dy = y-ys
      B  = wn*dy
      C  = cosh(B)
      D  = C-cos(A)
      Di = C-cos(Ai)

      u = cf * (sin(A)/D + sin(Ai)/Di)
      v = cf * sinh(B) * (1.0/D+1.0/Di)

      dy = y-2.0*wall3+ys
      B  = wn*dy
      C  = cosh(B)
      D  = C-cos(A)
      Di = C-cos(Ai)

      u = u + cf * (sin(A)/D + sin(Ai)/Di)
      v = v + cf * sinh(B) * (1.0/D+1.0/Di)

c-----------------------------------------------------
c flow due to a 2D point source of unit strength
c in front of a cylinder
c with zero circulation around the cylinder
c
c see Pozrikidis (1997, p. 363)
c-----------------------------------------------------

      elseif(menu.eq.106) then

      strength = 1.0

      Dist2 = (x-xs)**2+(y-ys)**2      !  point source
      u     = (x-xs)/Dist2
      v     = (y-ys)/Dist2

      Dist2 = (x-xsi)**2+(y-ysi)**2    ! image point source
      u = u + (x-xsi)/Dist2
      v = v + (y-ysi)/Dist2

      Dist2 = (x-xc)**2+(y-yc)**2      ! point sink at the
      u = u - (x-xc)/Dist2             ! center of the cylinder
      v = v - (y-yc)/Dist2

      cf = strength/pi2
      u = cf*u
      v = cf*v

c------------------------------------
c periodic point sources above a wall
c------------------------------------

      elseif(menu.eq.107) then

      wn = pi2/RL          ! wave number

      A = wn*(y-ys)
      B = wn*(x-xs)
      C = cosh(A)-cos(B)

      Ai = wn*(y-2.0*wall+ys)
      Bi = B
      Ci = cosh(Ai)-cos(Bi)

      cf = strength/(2.0*RL)

      u = cf * ( sin(B)/C+ sin(Bi)/Ci)
      v = cf * (sinh(A)/C+sinh(Ai)/Ci)

c--------------------
c 2D potential dipole
c--------------------

      elseif(menu.eq.108) then

      rs = x*x+y*y
      rq = rs*rs
      x2 = 2.0*x
      u  = -1.0/rs + x2*x/rq
      v  =           x2*y/rq

c--------------------------------
c 2D point source in uniform flow
c--------------------------------

      elseif(menu.eq.109) then

      Ustr = 1.0

      rs = (x-xs)**2+(y-ys)**2
      u  = 1.0 + beta*(x-xs)/rs
      v  =       beta*(y-ys)/rs

      u  = u*Ustr
      v  = v*Ustr

c---------------------------------------
c 2D source panel with constant strength
c---------------------------------------

      elseif(menu.eq.110) then

      atd = atan2(y,x-b)-atan2(y,x+b)
      tmp = log( ((x-b)**2+y**2)/((x+b)**2+y**2) )

      u =-sig0*tmp/pi4
      v = sig0*atd/pi2

c------------------------------
c flow due to a 3D point source
c located at the point (xs, ys)
c------------------------------

      elseif(menu.eq.170) then

      cf = 1.0/pi4

      r = sqrt((x-xs)**2+(y-ys)**2)
      r3 = r*r*r

      u = cf *(x-xs)/r3
      v = cf *(y-ys)/r3

c-------------------------------------------------
c flow due to a 3D point source above a plane wall 
c located at y = wall
c-------------------------------------------------

      elseif(menu.eq.171) then

      strength = 1.0

      Dists = (x-xs)**2+(y-ys)**2      !  point source
      Dist3 = sqrt(Dists**3)
      u     = (x-xs)/Dist3
      v     = (y-ys)/Dist3

      Dists = (x-xsi)**2+(y-ysi)**2    ! image point source
      Dist3 = sqrt(Dists**3)
      u = u + (x-xsi)/Dist3
      v = v + (y-ysi)/Dist3

      cf = strength/pi4

      u = cf*u
      v = cf*v

c---------------------------------------------------
c flow due to a 3D point source in front of a sphere
c---------------------------------------------------

      elseif(menu.eq.172) then

      z  = 0.0
      zs = 0.0
      zc = 0.0

      Distc = sqrt((x-xs)**2+(y-ys)**2+(z-zs)**2)**3
      u = (x-xs)/Distc
      v = (y-ys)/Distc

      fc = a/R 

      Distc = sqrt((x-xsi)**2+(y-ysi)**2+(z-zsi)**2)**3
      u = u + fc*(x-xsi)/Distc
      v = v + fc*(y-ysi)/Distc

      NN = 40    ! for numerical integration
     +           ! by the trapozoidal rule

      sumu = 0.0
      sumv = 0.0

      Do i=1,NN     ! integration by the trapezoidal rule
        ff = (i-0.5D0)/NN
        xx = xc + (xsi-xc)*ff
        yy = yc + (ysi-yc)*ff
        zz = zc + (zsi-zc)*ff
        Dist  = sqrt( (x-xx)**2+(y-yy)**2+(z-zz)**2 )
        Distc = Dist**3
        sumu  = sumu + (x-xx)/Distc
        sumv  = sumv + (y-yy)/Distc
      End Do

      u = u - fc*sumu/NN
      v = v - fc*sumv/NN

c-------------------------------------------
c flow due to a 2p array of 3d point sources
c-------------------------------------------

      elseif(menu.eq.173) then

      z  = 0.0

      x0 = 0.0
      y0 = 0.0
      z0 = 0.0

      Iopt   = 2
      method = 2   ! (Ewald)

       call lgf_3d_2p_ewald
     +
     +  (a11,a12
     +  ,a21,a22
     +  ,b11,b12
     +  ,b21,b22
     +  ,ew,area
     +  )

      call lgf_3d_2p
     +
     +    (Iopt
     +    ,method
     +    ,x,y,z
     +    ,x0,y0,z0
     +    ,a11,a12,a21,a22
     +    ,b11,b12,b21,b22
     +    ,ew,area
     +    ,max1,max2,max3
     +    ,G
     +    ,Gx,Gy,Gz
     +    )

      u = -Gx
      v = -Gy

c--------------------------------
c 3D point source in uniform flow
c--------------------------------

      elseif(menu.eq.174) then

      Ustr = 1.0

      r  = sqrt((x-xs)**2+(y-ys)**2)
      rc = r*r*r
      u  = 1.0 + beta*(x-xs)/rc
      v  =       beta*(y-ys)/rc

      u  = u*Ustr
      v  = v*Ustr

c--------------------------
c flow past a Rankine ovoid
c--------------------------

      elseif(menu.eq.175) then

      Ustr = 1.0
      ys   = 0.0
      u    = 1.0
      v    = 0.0

      strg = Ustr*xs*xs/rlam

      r  = sqrt((x-xs)**2+(y-ys)**2)
      rc = r*r*r
      u  = u + strg*(x-xs)/rc
      v  = v + strg*(y-ys)/rc

      r  = sqrt((x+xs)**2+(y+ys)**2)
      rc = r*r*r
      u  = u - strg*(x+xs)/rc
      v  = v - strg*(y+ys)/rc

      u  = u*Ustr
      v  = v*Ustr

c--------------------
c 3D potential dipole
c--------------------

      elseif(menu.eq.176) then

      r  = sqrt(x*x+y*y)
      r3 = r*r*r
      r5 = r*r*r3
      x3 = 3.0*x
      u  = -1.0/r3 + x3*x/r5
      v  =           x3*y/r5

c----------------------
c ring of point sources
c----------------------

      elseif(menu.eq.121) then

      call ps_ring (x,y,xs,ys,u,v)

      u = -u
      v = -v

c------------------------------------
c ring of point sources before a wall
c------------------------------------

      elseif(menu.eq.122) then

      call ps_ring_w (wall,x,y,xs,ys,u,v)

      u = -u
      v = -v

c-------------------
c flow near a corner
c-------------------

      elseif(menu.eq.129) then

      x1 = x - eps
      x2 = x + eps
      y1 = y - eps
      y2 = y + eps

      call corner(x1,y,rm,phi1)
      call corner(x2,y,rm,phi2)
      call corner(x,y1,rm,phi3)
      call corner(x,y2,rm,phi4)

      u = (phi2-phi1)/eps2
      v = (phi4-phi3)/eps2

c-----------------------------------------
c flow past a stationary circular cylinder
c-----------------------------------------

      elseif(menu.eq.130) then

      x1 = x - eps
      x2 = x + eps
      y1 = y - eps
      y2 = y + eps

      call cylinder (x,y1,a,circ,psi1)
      call cylinder (x,y2,a,circ,psi2)
      call cylinder (x1,y,a,circ,psi3)
      call cylinder (x2,y,a,circ,psi4)

      u =  (psi2-psi1)/eps2
      v = -(psi4-psi3)/eps2

c---------------------------------------
c flow due to a moving circular cylinder
c---------------------------------------

      elseif(menu.eq.131) then

      y1 = y - eps
      y2 = y + eps
      x1 = x - eps
      x2 = x + eps

      call cylinder_moving (x,y1,a,circ,psi1)
      call cylinder_moving (x,y2,a,circ,psi2)
      call cylinder_moving (x1,y,a,circ,psi3)
      call cylinder_moving (x2,y,a,circ,psi4)

      u =  (psi2-psi1)/eps2
      v = -(psi4-psi3)/eps2

c-----------------------
c flow past a flat plate
c-----------------------

      elseif(menu.eq.132) then

      d  =  x*x-y*y-1.0
      e  =  2.0*x*y
      r2 =  d*d + e*e
      r  =  sqrt(r2)
      arg = d/r
      arg = acos(arg)

      IF(e.lt.0) arg =  pi2-arg
      IF(y.lt.0) arg = -pi2+arg

      arg = -0.5*arg
      r   =  sqrt(r)

      AA  =  cos(arg)/r 
      BB  =  sin(arg)/r 
      CC  =   Vy*y
      DD  = - Vy*(x+circ)
      u   = Vx + AA*CC-BB*DD
      v   =      AA*DD+BB*CC
      v   = -v

c--------------------------------
c flow past a stationary ellipse
c--------------------------------

      elseif(menu.eq.133) then

      x1 = x - eps
      x2 = x + eps
      y1 = y - eps
      y2 = y + eps

      call ellipse (x,y1,a,b,Vx,Vy,circ,psi1)
      call ellipse (x,y2,a,b,Vx,Vy,circ,psi2)
      call ellipse (x1,y,a,b,Vx,Vy,circ,psi3)
      call ellipse (x2,y,a,b,Vx,Vy,circ,psi4)

      u =  (psi2-psi1)/eps2
      v = -(psi4-psi3)/eps2

c------------------------------
c flow past a joukowski airfoil 
c------------------------------

      elseif(menu.eq.134) then

      x1 = x - eps
      x2 = x + eps
      y1 = y - eps
      y2 = y + eps

      call joukowski (xc,yc,Vx,Vy,circ,x,y1,psi1)
      call joukowski (xc,yc,Vx,Vy,circ,x,y2,psi2)
      call joukowski (xc,yc,Vx,Vy,circ,x1,y,psi3)
      call joukowski (xc,yc,Vx,Vy,circ,x2,y,psi4)

      u =  (psi2-psi1)/eps2
      v = -(psi4-psi3)/eps2

c-----------------------------
c flow through an open channel over a bump
c by conformal mapping
c-----------------------------

      elseif(menu.eq.135
     +   .or.menu.eq.136
     +      ) then

      u = 1.0
      v = 0.0

c-----------------------------
c  flow down and up a strip
c  by conformal mapping
c-----------------------------

      elseif(menu.eq.137) then

      x1 = x - eps
      x2 = x + eps
      y1 = y - eps
      y2 = y + eps

      call strip (x,y1,psi1)
      call strip (x,y2,psi2)
      call strip (x1,y,psi3)
      call strip (x2,y,psi4)

      u =  (psi2-psi1)/eps2
      v = -(psi4-psi3)/eps2

c------------------------------
c flow past a stationary sphere
c------------------------------

      elseif(menu.eq.138) then

      ac = a*a*a

      xs = x*x
      ys = y*y
      rs = xs+ys
      r  = sqrt(rs)
      rc = r*r*r

      u = 1.0+0.5*ac/rc * (1.0-3.0*xs /rs)
      v =     0.5*ac/rc * (   -3.0*x*y/rs)

c------------------------------
c flow due to a moving sphere
c------------------------------

      elseif(menu.eq.139) then

      ac = a*a*a

      xs = x*x
      ys = y*y
      rs = xs+ys
      r  = sqrt(rs)
      rc = r*r*r

      u = -0.5*ac/rc * (1.0-3.0*xs /rs)
      v = -0.5*ac/rc * (   -3.0*x*y/rs)

c---------------------------------
c orthogonal stagnation-point flow
c past a sphere
c---------------------------------

      elseif(menu.eq.140) then

      x1 = x - eps
      x2 = x + eps
      y1 = y - eps
      y2 = y + eps

      call ospfs (x,y1,a,psi1)
      call ospfs (x,y2,a,psi2)
      call ospfs (x1,y,a,psi3)
      call ospfs (x2,y,a,psi4)

      eps2y = eps2*y

      u =  (psi2-psi1)/eps2y
      v = -(psi4-psi3)/eps2y

c-----------------------------------
c axial flow past a prolate spheroid
c-----------------------------------

      elseif(menu.eq.151) then

      a    = 1.0    ! major axis
      b    = boa*a
      e    = sqrt(1.0-boa*boa)
      c    = a*e
      es   = e*e
      tmp  = log((1.0+e)/(1.0-e))
      delx = 2.0*e/(1-es) -  tmp

      x1 = x - eps
      x2 = x + eps
      y1 = y - eps
      y2 = y + eps

      call sphrd_pr_axial (x1,y,c,delx,phi3)
      call sphrd_pr_axial (x2,y,c,delx,phi4)
      call sphrd_pr_axial (x,y1,c,delx,phi1)
      call sphrd_pr_axial (x,y2,c,delx,phi2)

      u = (phi4-phi3)/eps2
      v = (phi2-phi1)/eps2

c-----------------------------------
c transverse flow past a prolate spheroid
c-----------------------------------

      elseif(menu.eq.152) then

      a    = 1.0    ! major axis
      b    = boa*a
      e    = sqrt(1.0-boa*boa)
      c    = a*e
      es   = e*e
      tmp  = log((1.0+e)/(1.0-e))
      dely = e *(2.0*es-1.0)/(1.0-es) + 0.5*tmp

      x1 = x - eps
      x2 = x + eps
      y1 = y - eps
      y2 = y + eps

      call sphrd_pr_trans (x1,y,c,dely,phi3)
      call sphrd_pr_trans (x2,y,c,dely,phi4)
      call sphrd_pr_trans (x,y1,c,dely,phi1)
      call sphrd_pr_trans (x,y2,c,dely,phi2)

      u = (phi4-phi3)/eps2
      v = (phi2-phi1)/eps2

c-----------------------------------
c axial flow past an oblate spheroid
c-----------------------------------

      elseif(menu.eq.153) then

      b     = 1.0
      a     = aob*b
      e     = sqrt(1.0-aob*aob)
      c     = b*e
      es    = e*e
      zet0  = a/c
      zet0s = zet0*zet0
      delx  = aob - asin(e)/e

      x1 = x - eps
      x2 = x + eps
      y1 = y - eps
      y2 = y + eps

      call sphrd_ob_axial (x1,y,c,delx,phi3)
      call sphrd_ob_axial (x2,y,c,delx,phi4)
      call sphrd_ob_axial (x,y1,c,delx,phi1)
      call sphrd_ob_axial (x,y2,c,delx,phi2)

      u = (phi4-phi3)/eps2
      v = (phi2-phi1)/eps2

c-----------------------------------
c transverse flow past an oblate spheroid
c-----------------------------------

      elseif(menu.eq.154) then

      b    = 1.0
      a    = aob*b
      e    = sqrt(1.0-aob*aob)
      c    = b*e
      es   = e*e
      zet0 = a/c
      zet0s= zet0*zet0
      dely = (zet0s+2.0)/(zet0s+1.0)/zet0-atan(1.0/zet0)

      x1 = x - eps
      x2 = x + eps
      y1 = y - eps
      y2 = y + eps

      call sphrd_ob_trans (x1,y,c,dely,phi3)
      call sphrd_ob_trans (x2,y,c,dely,phi4)
      call sphrd_ob_trans (x,y1,c,dely,phi1)
      call sphrd_ob_trans (x,y2,c,dely,phi2)

      u = (phi4-phi3)/eps2
      v = (phi2-phi1)/eps2


c-----------------------------
c fLow past a prolate spheroid
c-----------------------------

      elseif(menu.eq.141) then

      x1 = x - eps
      x2 = x + eps
      y1 = y - eps
      y2 = y + eps

      call sphrd_pr (x1,y,c,delx,dely,phi3)
      call sphrd_pr (x2,y,c,delx,dely,phi4)
      call sphrd_pr (x,y1,c,delx,dely,phi1)
      call sphrd_pr (x,y2,c,delx,dely,phi2)

      u = (phi4-phi3)/eps2
      v = (phi2-phi1)/eps2

c-----------------------------
c fLow past an oblate spheroid
c-----------------------------

      elseif(menu.eq.142) then

      x1 = x - eps
      x2 = x + eps
      y1 = y - eps
      y2 = y + eps

      call sphrd_ob (x1,y,bb,c,Iflow,delx,dely,phi3)
      call sphrd_ob (x2,y,bb,c,Iflow,delx,dely,phi4)
      call sphrd_ob (x,y1,bb,c,Iflow,delx,dely,phi1)
      call sphrd_ob (x,y2,bb,c,Iflow,delx,dely,phi2)

      u = (phi4-phi3)/eps2
      v = (phi2-phi1)/eps2

c----------------------------------------------------------
c VORTEX FLOWS - VORTEX FLOWS - VORTEX FLOWS - VORTEX FLOWS
c----------------------------------------------------------

c---------------------------------
c Periodic array of Stuart generalized point vortices
c separated by distance 2.0
c---------------------------------

      elseif(menu.eq.201) then

      wn = pi2/RL    ! wave number

      A  = wn*y
      B  = wn*x
      Den = cosh(A)-beta*cos(B)
      u   = -      sinh(A)/Den
      v   =   beta*sin (B)/Den

c-------------------------
c a pair of point vortices
c-------------------------

      elseif(menu.eq.202) then

      ypv1 = 1.0
      ypv2 =-1.0
      r2 = (y-ypv1)**2+x**2
      d2 = (y-ypv2)**2+x**2
      u = -(y-ypv1)/r2 + (y-ypv2)/d2 - 0.50
      v =        x/r2 - x/d2

c-----------------------------------------
c point vortex outside a circular cylinder
c-----------------------------------------

      elseif(menu.eq.203) then

      as = a*a
      Dists = (xpv-xc)**2+(ypv-yc)**2
      xpvi = xc + as*(xpv-xc)/Dists   ! image
      ypvi = yc + as*(ypv-yc)/Dists   ! image
      Dists = (x-xpv)**2+(y-ypv)**2
      u = -(y-ypv)/Dists
      v =  (x-xpv)/Dists
      Dists = (x-xpvi)**2+(y-ypvi)**2
      u = u + (y-ypvi)/Dists
      v = v - (x-xpvi)/Dists

c----------------------------------------
c point vortex inside a circular cylinder
c----------------------------------------

      elseif(menu.eq.204) then

      as    = a*a
      Dists = (xpv-xc)**2+(ypv-yc)**2
      xpvi  = xc + as*(xpv-xc)/Dists   ! image
      ypvi  = yc + as*(ypv-yc)/Dists   ! image

      Dists = (x-xpv)**2+(y-ypv)**2
      u = -(y-ypv)/Dists
      v =  (x-xpv)/Dists
      Dists = (x-xpvi)**2+(y-ypvi)**2
      u = u + (y-ypvi)/Dists
      v = v - (x-xpvi)/Dists

c----------------------------------------
c point vortex between two parallel walls 
c----------------------------------------

      elseif(menu.eq.205) then

c     write (6,*) wall1, wall2,xpv,ypv
c     pause

      h  = wall2-wall1
      h2 = 2.0*h

      wn = pi2/h2     ! wave number

      A  = wn*(x-xpv)
      B  = wn*(y-ypv)
      Bi = wn*(y-2.0*wall1+ypv)
      C  = cosh(A)
      D  = C-cos(B)
      Di = C-cos(Bi)

      cf = 1.0/(2.0*h2)

      u  = - cf * (sin(B)/D-sin(Bi)/Di)
      v  =   cf * sinh(A)*(1.0/D-1.0/Di)

c--------------------------------------
c point vortex in a semi-infinite strip
c--------------------------------------

      elseif(menu.eq.206) then

      h  = wall2-wall1
      h2 = 2.0*h
      wn = pi2/h2          ! wave number

      cf = 1.0/(2.0*h2)

      dx  = x-xpv
      dxi = x-2.0*wall1+xpv

      A   = wn*dx
      Ai  = wn*dxi

      dy = y-ypv
      B  = wn*dy
      C  = cosh(B)
      D  = C-cos(A)
      Di = C-cos(Ai)

      u = - cf * sinh(B) * (1.0/D-1.0/Di)
      v =   cf * (sin(A)/D-sin(Ai)/Di)

      dy = y-2.0*wall3+ypv
      B  = wn*dy
      C  = cosh(B)
      D  = C-cos(A)
      Di = C-cos(Ai)

      u = u + cf * sinh(B)*(1.0/D-1.0/Di)
      v = v - cf * (sin(A)/D-sin(Ai)/Di)

c---------------------------------
c Periodic array of vortex blobs 
c with Gaussian distribution
c---------------------------------

      elseif(menu.eq.208) then

      wn  = pi2/RL   ! wave numbed
      A   = wn*y
      B   = wn*x
      Den = 2.0*RL*(cosh(A)-cos(B))
      u   = - sinh(A)/Den
      v   =   sin (B)/Den

      if(blob_rad.gt.0) then

       sumx = 0.0
       sumy = 0.0

       Do m=-10,10
        xh = x-m*RL
        yh = y
        rs = xh**2+yh**2
        r  = sqrt(rs)
        rr = r/blob_rad
        g  = 1.0-exp(-rr**2)
        sumx = sumx + xh/rs *(g-1.0)
        sumy = sumy + yh/rs *(g-1.0)
       End Do

       u = u - sumy/pi2
       v = v + sumx/pi2

      End If
      
      u = - u    ! change the sign for plotting purposes
      v = - v

c-----------------------
c Periodic vortex street 
c-----------------------

      elseif(menu.eq.209) then

      shift = 0.0
      if(Isym.eq.1) shift = 0.25*a

      wn  = pi2/a    ! wave number

      tmp1   =  wn*(y-0.5*b)
      tmp2   =  wn*(x-shift)
      den = 2.0*a*(cosh(tmp1)-cos(tmp2))
      u   = - sinh(tmp1)/Den
      v   =    sin(tmp2)/Den

      tmp1  =  wn*(y+0.5*b)
      tmp2  =  wn*(x+shift)
      den = 2.0*a*(cosh(tmp1)-cos(tmp2))
      u   = u + sinh(tmp1)/Den
      v   = v -  sin(tmp2)/Den

c-----------------
c line vortex ring
c-----------------

      elseif(menu.eq.210) then

      call lvr (x,y,xr,sr,u,v)

c------------------------------------
c line vortex ring in front of a wall
c------------------------------------

      elseif(menu.eq.211) then

      call lvr (x,y,xr,sr,u,v)
      call lvr (x,y,xri,sri,ui,vi)

      u = u - ui
      v = v - vi

c---------------------------------
c line vortex ring before a sphere
c---------------------------------

      elseif(menu.eq.212.or.menu.eq.213) then

      as  = a*a
      dds = (xr-xc)**2+sr*sr
      xri = xc+as*(xr-xc)/dds
      sri =    as* sr    /dds
      wri = -a/sqrt(dds)

      call lvr (x,y,xr,sr,u,v)
      call lvr (x,y,xri,sri,ui,vi)

      u = u + wri*ui
      v = v + wri*vi

c---------------
c  Hill's vortex
c---------------

      elseif(menu.eq.220) then

      x1 = x - eps
      x2 = x + eps
      y1 = y - eps
      y2 = y + eps

      call hill (x,y1,a,psi1)
      call hill (x,y2,a,psi2)
      call hill (x1,y,a,psi3)
      call hill (x2,y,a,psi4)

      tmp = eps2*y

      u =  (psi2-psi1)/tmp
      v = -(psi4-psi3)/tmp

      if(Iframe.eq.1)  then
        u = -u + 2/15.0
        v = - v
      end if

c------------------------------------
c vortex panel with constant strength
c------------------------------------

      elseif(menu.eq.230) then

      atd = atan2(y,x-b)-atan2(y,x+b)
      tmp = log( ((x-b)**2+y**2)/((x+b)**2+y**2) )

      u = -g0*atd/pi2
      v = -g0*tmp/pi4

c-----------------------------------
c  vortex panel with linear strength
c-----------------------------------

      elseif(menu.eq.231) then

      atd = atan2(y,x-b)-atan2(y,x+b)
      tmp = log( ((x-b)**2+y*y)/((x+b)**2+y*y) )

      u  = -g0*atd/pi2
      v  = -g0*tmp/pi4
      xh = x+b

      u = u - g1*( y*tmp+2.0*xh*atd      )/pi4
      v = v - g1*(xh*tmp-2.0*y *atd+4.0*b)/pi4

c============
c STOKES FLOW 
c============

c----------------------
c stagnation-point flow
c----------------------

      elseif(menu.eq.301) then

      x1 = x - eps
      x2 = x + eps
      y1 = y - eps
      y2 = y + eps

      call stokes_spf (x,y1,alpha,psi1)
      call stokes_spf (x,y2,alpha,psi2)
      call stokes_spf (x1,y,alpha,psi3)
      call stokes_spf (x2,y,alpha,psi4)

      u =  (psi2-psi1)/eps2
      v = -(psi4-psi3)/eps2

c--------
c Scraper 
c--------

      elseif(menu.eq.302) then

      x1 = x - eps
      x2 = x + eps
      y1 = y - eps
      y2 = y + eps

      call stokes_scraper (x,y1,alpha,psi1)
      call stokes_scraper (x,y2,alpha,psi2)
      call stokes_scraper (x1,y,alpha,psi3)
      call stokes_scraper (x2,y,alpha,psi4)

      u =  (psi2-psi1)/eps2
      v = -(psi4-psi3)/eps2

c-------
c finite belt on a wall
c-------

      elseif(menu.eq.303) then

      x1 = x - eps
      x2 = x + eps
      y1 = y - eps
      y2 = y + eps

      call stokes_belt (x,y1,width,psi1)
      call stokes_belt (x,y2,width,psi2)
      call stokes_belt (x1,y,width,psi3)
      call stokes_belt (x2,y,width,psi4)

      u =  (psi2-psi1)/eps2
      v = -(psi4-psi3)/eps2

c-----
c tape 
c-----

      elseif(menu.eq.304) then

      y1 = y - eps
      y2 = y + eps
      x1 = x - eps
      x2 = x + eps

      call stokes_tape (x,y1,alpha,psi1)
      call stokes_tape (x,y2,alpha,psi2)
      call stokes_tape (x1,y,alpha,psi3)
      call stokes_tape (x2,y,alpha,psi4)

      u =  (psi2-psi1)/eps2
      v = -(psi4-psi3)/eps2

c-----------------
c Flow in a corner 
c-----------------

      elseif(menu.eq.305) then

      y1 = y - eps
      y2 = y + eps
      x1 = x - eps
      x2 = x + eps

      call moffatt (x,y1,alpha,rlr,rli,Ieddies,Isym,psi1)
      call moffatt (x,y2,alpha,rlr,rli,Ieddies,Isym,psi2)
      call moffatt (x1,y,alpha,rlr,rli,Ieddies,Isym,psi3)
      call moffatt (x2,y,alpha,rlr,rli,Ieddies,Isym,psi4)

      u =   (psi2-psi1)/eps2
      v = - (psi4-psi3)/eps2

c--------------------------
c flow due to constant shear stress 
c inside a corner
c--------------------------

      elseif(menu.eq.306) then

      y1 = y - eps
      y2 = y + eps
      x1 = x - eps
      x2 = x + eps

      call sf_crn_sh (x,y1,alpha,psi1)
      call sf_crn_sh (x,y2,alpha,psi2)
      call sf_crn_sh (x1,y,alpha,psi3)
      call sf_crn_sh (x2,y,alpha,psi4)

      u =  (psi2-psi1)/eps2
      v = -(psi4-psi3)/eps2

c--------------------
c  translating sphere
c--------------------

      elseif(menu.eq.307) then

      y1 = y - eps
      y2 = y + eps
      x1 = x - eps
      x2 = x + eps

      call stokes_sphere (a,x,y1,psi1)
      call stokes_sphere (a,x,y2,psi2)
      call stokes_sphere (a,x1,y,psi3)
      call stokes_sphere (a,x2,y,psi4)

      eps2y = eps2*y

      u =  (psi2-psi1)/eps2y
      v = -(psi4-psi3)/eps2y

      if(Iframe.eq.0) then
       u = 1.0-u
       v =    -v
      end if

c--------------------
c  translating drop 
c--------------------

      elseif(menu.eq.308) then

      call stokes_drop (x,y,a,vsr,u,v)

      if(Iframe.eq.0) then
       u = 1.0-u
       v =    -v
      end if

c--------------
c  3D Stokeslet
c--------------

      elseif(menu.eq.309) then

      xs = x*x
      ys = y*y
      r  = sqrt(xs+ys)
      rc = r*r*r
      u  = 1/r + xs /rc
      v  =       x*y/rc

c--------------
c  2D Stokeslet
c--------------

      else if(menu.eq.310) then

      xs = x*x
      ys = y*y
      rs = xs+ys
      u  = - 0.50*log(rs) + xs /rs
      v  =                  x*y/rs

c-----------------------------------------
c doubly-periodic array of 2D point forces
c-----------------------------------------

      else if(menu.eq.311) then

      Iselect = 1  ! stress is not required

      x0 = 0.0     ! coordinates of one point force
      y0 = 0.0

       call ewald_2d
     +
     +  (a11,a12,a21,a22
     +  ,b11,b12,b21,b22
     +  ,ew,area
     +  )

       call qqq_2d (b11,b12,b21,b22,max2,ew)

      call sgf_2d_2p
     +
     +   (Iselect
     +   ,x,y
     +   ,x0,y0
     +   ,a11,a12,a21,a22
     +   ,b11,b12,b21,b22
     +   ,ew,area
     +   ,max1,max2
     +   ,gxx,gxy
     +   ,gyx,gyy
     +   ,px,py
     +   ,txxx,txxy,tyxx,tyxy
     +   ,txyx,txyy,tyyx,tyyy
     +   )

      u = gxx
      v = gxy

c---------------------------------
c  Triple array of 3D point forces
c---------------------------------

      elseif(menu.eq.312) then

      Iselect = 1

      x0 = 0.0     ! coordinates of the point force
      y0 = 0.0
      z0 = 0.0

      z = 0.0      ! streamline pattern in the xy plane

       call sgf_3d_3p_ewald
     +
     +   (a11,a12,a13
     +   ,a21,a22,a23
     +   ,a31,a32,a33
     +   ,b11,b12,b13
     +   ,b21,b22,b23
     +   ,b31,b32,b33
     +   ,ew,vlm
     +   )

      call sgf_3d_3p_qqq
     +
     +  (b11,b12,b13
     +  ,b21,b22,b23
     +  ,b31,b32,b33
     +  ,max2
     +  ,ew
     +  )

      call sgf_3d_3p
     +
     +  (Iselect
     +  ,x,y,z
     +  ,x0,y0,z0
     +  ,a11,a12,a13
     +  ,a21,a22,a23
     +  ,a31,a32,a33
     +  ,b11,b12,b13
     +  ,b21,b22,b23
     +  ,b31,b32,b33
     +  ,ew,vlm
     +  ,max1,max2
     +  ,Gxx,Gxy,Gxz
     +  ,Gyx,Gyy,Gyz
     +  ,Gzx,Gzy,Gzz
     +  ,px,py,pz
     +  ,Txxx,Txxy,Txxz,Tyxy,Tyxz,Tzxz
     +  ,Txyx,Txyy,Txyz,Tyyy,Tyyz,Tzyz
     +  ,Txzx,Txzy,Txzz,Tyzy,Tyzz,Tzzz
     +  )

      u = gxx
      v = gxy

c--------------------------------------
c  3D point force in front of a sphere
c-------------------------------------

      elseif(menu.eq.313) then

      z = 0.0       ! consider the xy plane

      call sgf_3d_sph
     +
     +  (x,y,z
     +  ,xs,ys,zs
     +  ,xc,yc,zc
     +  ,a
     +  ,gxx,gxy,gxz
     +  ,gyx,gyy,gyz
     +  ,gzx,gzy,gzz
     +  )

      if(Iflow.eq.1) then
        u = gxx
        v = gyx
      else
        u = gxy
        v = gyy
      end if

c------------------------------------
c  3D point force in front of a wall
c------------------------------------

      elseif(menu.eq.314) then

      x0 = 1.0  ! coordinates of the point force
      y0 = 0.0
      z0 = 0.0

      z = 0.0  ! streamlines in the xy plane

      Iselect = 1

      call sgf_3d_w
     +
     +  (Iselect
     +  ,x ,y ,z
     +  ,x0,y0,z0
     +  ,wall
     +  ,Gxx,Gxy,Gxz
     +  ,Gyx,Gyy,Gyz
     +  ,Gzx,Gzy,Gzz
     +  ,px,py,pz
     +  ,Txxx,Txxy,Txxz,Tyxy,Tyxz,Tzxz
     +  ,Txyx,Txyy,Txyz,Tyyy,Tyyz,Tzyz
     +  ,Txzx,Txzy,Txzz,Tyzy,Tyzz,Tzzz
     +  )

      if(Iflow.eq.1) then
        u = gxx
        v = gyx
      else
        u = gxy
        v = gyy
      End If

c------------------------------
c flow due to a double periodic
c  array of 3D point forces
c------------------------------

      elseif(menu.eq.315) then

      Iselect = 1

      xx = x 

      if(Iflow.eq.1) then ! streamlines in the xz plane
       yy = 0.0 
       zz = y 
      elseif(Iflow.eq.2) then ! streamlines in the xy plane
       yy = y
       zz = 0.0
      elseif(Iflow.eq.3) then ! streamlines in the xz plane
       yy = 0.0 
       zz = y 
      end if

      x0 = 0.0   ! coordinates of a point force
      y0 = 0.0
      z0 = 0.0

      method = 2        ! Fast summation method
      eps    = 0.001    ! for numerical differentiation

       call sgf_3d_2p_ewald
     +
     + (a11,a12
     + ,a21,a22
     + ,b11,b12
     + ,b21,b22
     + ,ew,area
     + )

      call sgf_3d_2p
     +
     +   (method
     +   ,eps
     +   ,xx,yy,zz
     +   ,x0,y0,z0
     +   ,a11,a12,a21,a22
     +   ,b11,b12,b21,b22
     +   ,ew,area
     +   ,max1,max2
     +   ,Gxx,Gxy,Gxz
     +   ,Gyx,Gyy,Gyz
     +   ,Gzx,Gzy,Gzz
     +   )

      if(Iflow.eq.1) then ! streamlines in the xz plane
       Gxx = Gxx-(zz-z0)*pi4/area
       u = Gxx
       v = Gxz
      elseif(Iflow.eq.2) then ! streamlines in the xy plane
       u = Gxx
       v = Gxy
      elseif(Iflow.eq.3) then ! streamlines in the xz plane
       u = Gxx
       v = Gxz
      end if

c----------------------------
c 1P array of 2D point forces
c----------------------------

      elseif(menu.eq.316) then

      Iselect = 1
      Ising   = 0

      x0 = 0.0   ! coordinates of a point force
      y0 = 0.0

      call sgf_2d_1p
     +
     +  (Iselect
     +  ,x,y
     +  ,X0,Y0
     +  ,RL
     +  ,Ising
     +  ,Gxx,Gxy
     +  ,Gyx,Gyy
     +  ,px,py
     +  ,Txxx,Txxy,Tyxx,Tyxy
     +  ,Txyx,Txyy,Tyyx,Tyyy
     +  )

      if(Iflow.eq.1) then
        Gxx = Gxx-(y-y0)*pi2/RL
        u = Gxx
        v = Gyx
      elseif(Iflow.eq.2) then
        u = Gxy
        v = Gyy
      elseif(Iflow.eq.3) then
        u = Gxx
        v = Gyx
      end If

c-----------------------------------------
c 1P array of 2D point forces in a channel
c-----------------------------------------

      elseif(menu.eq.317) then

      Iselect = 1
      Ising   = 0

      x0 = xs
      y0 = ys

      x0 = 0.0    ! coordinates of a point force
      if(Iflow.eq.1) then
       y0 = 0.25
      Else
       y0 = 0.5
      End If

      IQPD = 0
      IQPD = 1

      NP = 10
      h = 1.0

      call sgf_2d_1p_ww
     +
     +  (Iselect
     +  ,IQPD
     +  ,X,Y
     +  ,X0,Y0
     +  ,RL
     +  ,NP
     +  ,h
     +  ,Gxx,Gxy
     +  ,Gyx,Gyy
     +  ,px,py
     +  ,Txxx,Txxy,Tyxx,Tyxy
     +  ,Txyx,Txyy,Tyyx,Tyyy
     +  )

      if(Iflow.eq.1) then
        u = gxy
        v = gyy
      else
        u = gxx
        v = gyx
      end if

c-----------------------------------
c  2D point force in front of a wall
c-----------------------------------

      elseif(menu.eq.318) then

      x0 = 0.0  ! coordinates of the point force
      y0 = 1.0

      Iselect = 1

      call sgf_2d_w
     +
     +  (Iselect
     +  ,x,y 
     +  ,x0,y0
     +  ,wall
     +  ,Gxx,Gxy
     +  ,Gyx,Gyy
     +  ,Px,Py
     +  ,Txxx,Txxy,Tyxx,Tyxy
     +  ,Txyx,Txyy,Tyyx,Tyyy
     +  )

      If(Iflow.eq.1) then
        u = gxy
        v = gyy
      Else
        u = gxx
        v = gyx
      End If

c-----------------------------------
c  an array of 2D point forces in front of a wall
c-----------------------------------

      elseif(menu.eq.319) then

      x0 = xs  ! coordinates of the point force
      y0 = ys 

      Iselect = 1
      Ising = 0

      call sgf_2d_1p_w
     +
     +   (Iselect
     +   ,x,y 
     +   ,x0,y0
     +   ,wall
     +   ,RL
     +   ,Ising
     +   ,Gxx,Gxy
     +   ,Gyx,Gyy
     +   ,px,py
     +   ,Txxx,Txxy,Tyxx,Tyxy
     +   ,Txyx,Txyy,Tyyx,Tyyy
     +   )

      if(Iflow.eq.1) then
        u = gxy
        v = gyy
      else
        u = gxx
        v = gyx
      end if

c------------------------------------------
c  linear Stokes flow past a spherical drop
c------------------------------------------

      elseif(menu.eq.320) then

      as = a*a

      xs = x*x
      ys = y*y
      rs = xs+ys
      r  = sqrt(rs)
      rp = r**5

      if(r.gt.a) then                    ! exterior flow
       b1 = (1.0+chi)*(-0.5+ 5.0/20.0)
       b2 = (1.0+chi)*(-2.5+15.0/20.0)
       u = y    +(b1*(1.0-5.0*xs/rs)+b2*xs/as)*y/rp
       v = chi*x+(b1*(1.0-5.0*ys/rs)+b2*ys/as)*x/rp
      else
       del = (1.0-chi)/(1.0+chi)
       zet = 10.0*del
       a0  = 5.0/4.0*(1.0+chi)/10.0
       a1  = -3.0+2.0*zet/5.0
       a2  = -3.0-2.0*zet/5.0
       u   = a0*(a1+xs/as+5.0*ys/as)*y
       v   = a0*(a2+xs/as+5.0*ys/as)*x
      end if

c-----------------------------------------
c  Stokes stagnation-point flow in a fluid
c-----------------------------------------

      elseif(menu.eq.330) then

      u = x*(    x*x+3.0*y*y)
      v =-y*(3.0*x*x+    y*y)

c-------------------------------------
c  Stokes flow past a slender cylinder
c-------------------------------------

      elseif(menu.eq.340) then
c---

c---
c singularity representation
c---

      N = 90 
      Dx0 = 2.0*a/N

c---
      if(Iflow.eq.1) then      ! axial flow
c---
        u = 0.0
        v = 0.0
        Do i=1,N
          x0 = -a+(i-0.5)*Dx0
          dx = x-x0
          dy = y
          r2 = dx*dx+dy*dy
          r  = sqrt(r2)
          r3 = r2*r
          u  = u +1.0/r + dx*dx/r3
          v  = v        + dx*dy/r3
        End Do
        cf = Dx0*alpha/pi8
        u  = u*cf
        v  = v*cf
c---
      else                   ! Transverse flow
c---
        xx = 0.0
        yy = x
        zz = y
        us = 0.0
        vs = 0.0
        ud = 0.0
        vd = 0.0

        Do i=1,N
          x0 = -a+(i-0.5)*Dx0
          dx = xx-x0
          dy = yy
          dz = zz
          r2 = dx*dx+dy*dy+dz*dz
          r  = sqrt(r2)
          r3 = r2*r
          rp = r**5
          us = us + 1.0/r+ dy*dy/r3
          vs = vs          dy*dz/r3
          ud = ud - 1.0/r3 + 3.0*dy*dy/rp
          vd = vd          + 3.0*dy*dz/rp
        End Do

        cfs = Dx0*beta/pi8
        cfd = Dx0*gama/pi4
        u = us*cfs + ud*cfd
        v = vs*cfs + vd*cfd
        u = - u + 1.0
        v = - v
c---
      end if
c---

c------------------------------
c  Shear flow over a plane wall 
c  with a circular orifice
c  of zero thickness
c
c  Analytical expressions derived
c  by A.M.J. Davis
c------------------------------

      elseif(menu.eq.350) then

c---
c In Davis's notation, xx replaces x
c and zz replaces  y
c
c Shear flow is along the xx axis
c aa is the radius of the orifice
c---

      xx = x
      yy = 0.0   ! streamlines in the xx-zz plane
      zz = y

      call sf_orifice_3D
     +
     +  (x_orfc
     +  ,y_orfc
     +  ,z_orfc
     +  ,a
     +  ,xx,yy,zz
     +  ,ux,uy,uz
     +  )

      u = ux
      v = uz

c-------------------------------
c  Stokes flow over a plane wall 
c  over a discoidal patch 
c  of zero shear stress
c-------------------------------

      elseif(menu.eq.351) then

c---
c flow over a circular orifice
c---

      xx = x
      yy = 0.0   ! streamlines in the xx-zz plane
      zz = y

      call sf_orifice_3D
     +
     +   (x_orfc
     +   ,y_orfc
     +   ,z_orfc
     +   ,a
     +   ,xx,yy,zz
     +   ,ux,uy,uz
     +   )

c---
c image flow
c---

      zz = 2.0*z_orfc-zz

      call sf_orifice_3D
     +
     +   (x_orfc,y_orfc,z_orfc
     +   ,a
     +   ,xx,yy,zz
     +   ,ux1,uy1,uz1
     +   )

      ux = ux+ux1
      uy = uy+uy1
      uz = uz-uz1

      u = ux
      v = uz

c-------------------------------------------
c  Stokes flow over a plane wall with a slit 
c-------------------------------------------

      elseif(menu.eq.352) then

      y1 = y - eps
      y2 = y + eps
      x1 = x - eps
      x2 = x + eps

      call sf_orifice_2D(x,y1,a,psi1)
      call sf_orifice_2D(x,y2,a,psi2)
      call sf_orifice_2D(x1,y,a,psi3)
      call sf_orifice_2D(x2,y,a,psi4)

      u =  (psi2-psi1)/eps2
      v = -(psi4-psi3)/eps2

c-------------------------------
c Stokes flow over a plane wall
c over a zero shear stress patch 
c-------------------------------

      elseif(menu.eq.353) then

      y1 = y - eps
      y2 = y + eps
      x1 = x - eps
      x2 = x + eps

      call sf_patch_2D (x,y1,a,psi1)
      call sf_patch_2D (x,y2,a,psi2)
      call sf_patch_2D (x1,y,a,psi3)
      call sf_patch_2D (x2,y,a,psi4)

      u =  (psi2-psi1)/eps2
      v = -(psi4-psi3)/eps2

c----------------------
c  ring of point forces
c----------------------

      elseif(menu.eq.360) then

      X0 = xr
      Y0 = sr

      Iopt = 1

      call sgf_ax_fs
     +
     +  (Iopt
     +  ,X0,Y0
     +  ,X,Y
     +  ,GXX,GXY
     +  ,GYX,GYY
     +  ,QXXX,QXXY,QXYX,QXYY
     +  ,QYXX,QYXY,QYYX,QYYY
     +  ,PXX,PXY,PYX,PYY
     +  ,Iaxis
     +  )

      if(Iflow.eq.1) then
        u = gxx
        v = gyx
      else
        u = gxy
        v = gyy
      end if

c------------------------------------
c  ring of point forces before a wall
c------------------------------------

      elseif(menu.eq.361) then

      Iopt = 1

      X0 = xr
      Y0 = sr

      call sgf_ax_w
     +
     +  (Iopt
     +  ,X0,Y0
     +  ,X,Y
     +  ,wall
     +  ,GXX,GXY
     +  ,GYX,GYY
     +  ,QXXX,QXXY,QXYX,QXYY
     +  ,QYXX,QYXY,QYYX,QYYY
     +  ,PXX,PXY,PYX,PYY
     +  ,Iaxis
     +  )

      if(Iflow.eq.1) then
        u = gxx
        v = gyx
      else
        u = gxy
        v = gyy
      end if

c-----------------------
c  periodic ring of point forces
c-----------------------

      elseif(menu.eq.362) then

      Iopt = 1

      X0 = xr
      Y0 = sr

      Nsum = 10
      Np   = 4
      RL   = 1.0  ! period

      call sgf_ax_1p
     +
     + (Iopt
     + ,X0,Y0
     + ,X,Y
     + ,RL
     + ,Nsum,Np
     + ,GXX,GXY
     + ,GYX,GYY
     + ,QXXX,QXXY,QXYX,QXYY
     + ,QYXX,QYXY,QYYX,QYYY
     + ,PXX,PXY,PYX,PYY
     + ,Iaxis
     + )

      if(Iflow.eq.1) then
        u = gxx + cc
        v = gyx
      else
        u = gxy
        v = gyy
      end if

c---------------------------------
c  ring of point forces in a tube
c---------------------------------

      else if(menu.eq.363) then

      Nsum = 10
      Np   = 3 
      sc   = 1.0
      RL   = 1.0  ! period

      X0 = xr
      Y0 = sr
    
      call sgf_ax_1p_ct
     +
     +  (x,y
     +  ,x0,y0
     +  ,sc
     +  ,RL
     +  ,Nsum,Np
     +  ,gxx,gxy
     +  ,gyx,gyy
     +  )

      if(Iflow.eq.1) then
        u = gxx
        v = gyx
      else
        u = gxy
        v = gyy
      end if

c------------------------------------
c  3D point source in front of a wall
c------------------------------------

      elseif(menu.eq.370) then

      x0 = 1.0  ! coordinates of the point source
      y0 = 0.0
      z0 = 0.0

      z = 0.0  ! streamlines in the xy plane

      Iselect = 2

      call sgf_3d_w
     +
     +  (Iselect
     +  ,x0,y0,z0
     +  ,x,y,z
     +  ,wall
     +  ,Gxx,Gxy,Gxz
     +  ,Gyx,Gyy,Gyz
     +  ,Gzx,Gzy,Gzz
     +  ,Px,Py,Pz
     +  ,Txxx,Txxy,Txxz,Tyxy,Tyxz,Tzxz
     +  ,Txyx,Txyy,Txyz,Tyyy,Tyyz,Tzyz
     +  ,Txzx,Txzy,Txzz,Tyzy,Tyzz,Tzzz
     +  )

      u=-Px
      v=-Py

c------------------------------------
c  2D point source in front of a wall
c------------------------------------

      elseif(menu.eq.371) then

      x0 = 0.0  ! coordinates of the point source
      y0 = 1.0

      Iselect = 2

      call sgf_2d_w
     +
     +  (Iselect
     +  ,x0,y0
     +  ,x,y
     +  ,wall
     +  ,Gxx,Gxy
     +  ,Gyx,Gyy
     +  ,Px,Py
     +  ,Txxx,Txxy,Tyxx,Tyxy
     +  ,Txyx,Txyy,Tyyx,Tyyy
     +  )

      u =-Px
      v =-Py

c--------------------------
c  2D periodic point source
c--------------------------

      elseif(menu.eq.372) then

      x0 = 0.0  ! coordinates of the point source
      y0 = 0.0

      Iselect = 2
      Idesing = 0

      call sgf_2d_1p
     +
     +   (Iselect
     +   ,x0,y0
     +   ,x,y
     +   ,RL
     +   ,Idesing
     +   ,Gxx,Gxy
     +   ,Gyx,Gyy
     +   ,Px,Py
     +   ,Txxx,Txxy,Tyxx,Tyxy
     +   ,Txyx,Txyy,Tyyx,Tyyy
     +   )

      u =-Px
      v =-Py

c---------------------------------------
c  2D periodic point source above a wall
c---------------------------------------

      elseif(menu.eq.373) then

      x0 = 0.0  ! coordinates of the point source
      y0 = 0.1

      Iselect = 2
      Idesing = 0

      call sgf_2d_1p_w
     +
     +   (Iselect
     +   ,x0,y0
     +   ,x,y
     +   ,wall
     +   ,RL
     +   ,Idesing
     +   ,Gxx,Gxy
     +   ,Gyx,Gyy
     +   ,Px,Py
     +   ,Txxx,Txxy,Tyxx,Tyxy
     +   ,Txyx,Txyy,Tyyx,Tyyy
     +   )

      u =-Px
      v =-Py

c===================
c MISCELANEOUS FLOWS
c===================

c--------------
c  Kovasznay flow
c--------------

      elseif(menu.eq.401) then

      tmp  = Re/pi2
      cons = sqrt(tmp*tmp+4)-tmp
      rk   = pi2
      tmp  = -cons*0.5*rk*x
      prf  = exp(tmp)
      rky  = rk*y
      u    =  1.0-cos(rky)*prf
      v    = -0.5*cons*sin(rky)*prf

c--------------------
c  Taylor cellular flow 
c--------------------

      elseif(menu.eq.402) then

      pi2x = pi2*x
      pi2y = pi2*y
      u    = - cos(pi2x)*sin(pi2y)
      v    =   sin(pi2x)*cos(pi2y)

c-------------------------
c  Oseen flow past a sphere
c-------------------------

      elseif(menu.eq.403) then

      x1 = x - eps
      x2 = x + eps
      y1 = y - eps
      y2 = y + eps

      call oseen (x,y1,Re,psi1)
      call oseen (x,y2,Re,psi2)
      call oseen (x1,y,Re,psi3)
      call oseen (x2,y,Re,psi4)

      tmp = eps2*y

      u =  (psi2-psi1)/tmp
      v = -(psi4-psi3)/tmp

c--------------------------------------
c flow due to a point source of momentum
c--------------------------------------

      elseif(menu.eq.404) then

      x1 = x - eps
      x2 = x + eps
      y1 = y - eps
      y2 = y + eps

      call landau (x,y1,Dcn,psi1)
      call landau (x,y2,Dcn,psi2)
      call landau (x1,y,Dcn,psi3)
      call landau (x2,y,Dcn,psi4)

      tmp = eps2*y

      u =  (psi2-psi1)/tmp
      v = -(psi4-psi3)/tmp

c------------------------
c  Blasius boundary layer
c------------------------

      elseif(menu.eq.405) then

      eta = y/sqrt(x)

      Do i=1,Nblas        ! interpolate through the simil sln
        diff = eta-xblas(i)
        if(diff.lt.0) Go to 252
      End Do

  252 Continue

      i = i-1
      fc = (eta-xblas(i))/(xblas(i+1)-xblas(i))
      f  =  fblas(i)+fc*( fblas(i+1)- fblas(i))
      fp = fpblas(i)+fc*(fpblas(i+1)-fpblas(i))
      u  = fp
      v  = 0.5/sqrt(x) * (eta*fp-f)

c----------------------------------
c  2D OBLIQUE STAGNATION-POINT FLOW
c----------------------------------

      elseif(menu.eq.406) then

      sn   = sin(alpha)
      srsn = sqrt(sn)
      ct   = 1.0/tan(alpha)

      x1 = x - eps
      x2 = x + eps
      y1 = y - eps
      y2 = y + eps

      call spf (x,y1,srsn,ct,Nspf,xspf,fspf,gspf,psi1)
      call spf (x,y2,srsn,ct,Nspf,xspf,fspf,gspf,psi2)
      call spf (x1,y,srsn,ct,Nspf,xspf,fspf,gspf,psi3)
      call spf (x2,y,srsn,ct,Nspf,xspf,fspf,gspf,psi4)

      u =  (psi2-psi1)/eps2
      v = -(psi4-psi3)/eps2

c-----------------------------------------------
c  AXISYMMETRIC ORTHOGONAL STAGNATION-POINT FLOW
c-----------------------------------------------

      elseif(menu.eq.407) then

      x1 = x - eps
      x2 = x + eps
      y1 = y - eps
      y2 = y + eps

      call homann (x,y1,Nhom,xhom,fhom,psi1)
      call homann (x,y2,Nhom,xhom,fhom,psi2)
      call homann (x1,y,Nhom,xhom,fhom,psi3)
      call homann (x2,y,Nhom,xhom,fhom,psi4)

      u =  (psi2-psi1)/eps2
      v = -(psi4-psi3)/eps2

c---------------
c 2D LINEAR FLOW 
c---------------

      elseif(menu.eq.408) then
 
      u = y
      v = chi*x

c===========
      End If
c===========

 100  format (1x,i3,6(1x,f10.5))
 101  format (1x,i3,6(1x,i5.3))
 102  format (6(1x,f10.5))

      return
      end
