      subroutine tube_ann
     +
     +  (a1
     +  ,a2
     +  ,visc
     +  ,den
     +  ,gac
     +  ,V1
     +  ,V2
     +  ,chi
     +  ,beta
     +  ,s
     +  ,flow_rate
     +  ,umean
     +  ,u
     +  )

c======================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c======================================

c==================================================
c Steady unidirectional flow through an annular tube
c confined between two concentric cylinders
c
c SYMBOLS:
c -------
c
c a1:   inner cylinder radius
c a2:   outer cylinder radius
c
c visc: fluid viscosity
c den:  fluid density
c
c V1:   inner cylinder velocity
c V2:   outer cylinder velocity
c
c chi:  negative of the pressure gradient
c
c beta: inclination angle
c gac:  acceleration of gravity
c
c=================================================

      Implicit Double Precision (a-h,o-z)

c----------
c constants
c----------

      pi = 3.14159 265358 D0

c--------
c prepare
c--------

      a1s = a1**2
      a2s = a2**2

      area = pi*(a2s-a1s)

      fc = log(a2/a1)

      chim = chi + den*gac*Dsin(beta)  ! modified pressure gradient

c--------------------
c velocity evaluation
c--------------------

      tmp = Dlog(a2/s)/fc

      u = V2 + (V1-V2)*tmp
     +       + 0.25D0*chim/visc *(a2s-s**2-(a2s-a1s)*tmp)

c-------------------------
c flow rate, mean velocity
c-------------------------

      flow_rate =       pi*(U2*a2s-U1*a1s)
     +           -0.5D0*pi*(U2-U1)*(a2s-a1s)/fc
     +    + pi*chim/(8.0D0*visc) * (a2s-a1s)*(a2s+a1s-(a2s-a1s)/fc)
     +             

      umean  = flow_rate/area

c-----
c Done
c-----

      Return
      End
