      program tube_ann_dr

c===========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c===========================================

c===========================================
c Steady unidirectional pressure-driven
c or gravity-driven flow through an annular tube
c confined between two concentric cylinders
c
c SYMBOLS:
c -------
c
c a1:   inner cylinder radius
c a2:   outer cylinder radius
c visc: fluid viscosity
c den:  fluid density
c V1:   inner cylinder velocity
c V2:   outer cylinder velocity
c chi:   negative of the pressure gradient
c beta: inclination angle
c gac:  acceleration of gravity
c
c===========================================

      Implicit Double Precision (a-h,o-z)

      Parameter (Kpts=64)    ! divisions to draw the cylinder

c---
c constants
c---

      pi = 3.14159 265358 D0
      pi2 = 2.0*pi

      Null = 0
      None = 1
      Ntwo = 2

      zero = 0.0

c---
c constants
c---

      open  (1,file="tube_ann.dat",status="unknown")
       read (1,*) nprof
       read (1,*) a1   ! inner tube radius
       read (1,*) a2   ! outer tube radius
       read (1,*) visc
       read (1,*) den
       read (1,*) gac
       read (1,*) V1
       read (1,*) V2
       read (1,*) chi
       read (1,*) beta
      close (1)

c---
c prepare
c---

      open (2,file="tube_ann.out",status="unknown")

      beta = beta*pi

      nprof1 = nprof+1
      Kpts1  = Kpts+1

c---
c Draw the cylinders
c---

      Dth = pi2/(Kpts1-1.0)

      write (2,900) Kpts1
      Do i=1,Kpts1
       thet = (i-1.0)*Dth
       write (2,900) i,a1*cos(thet),a1*sin(thet)
      End Do

      write (2,900) Kpts1
      Do i=1,Kpts1
       thet = (i-1.0)*Dth
       write (2,900) i,a2*cos(thet),a2*sin(thet)
      End Do

c---
c Draw a vertical line
c---

      write (2,900) Ntwo
      write (2,900) None,zero,a1
      write (2,900) Ntwo,zero,a2

c---
c profile
c---

      write (6,*)
      write (6,*)  " velocity profile"
      write (6,*)

      write (2,900) nprof1+2

      write (2,900) Null,zero,a1

      step = (a2-a1)/(nprof1-1.0)

      write (6,*)
      write (6,*) " u, sigma"
      write (6,*)

      sigma = a1

      Do i=1,nprof1

       call tube_ann
     +
     +    (a1
     +    ,a2
     +    ,visc
     +    ,den
     +    ,gac
     +    ,V1
     +    ,V2
     +    ,chi
     +    ,beta
     +    ,sigma
     +    ,flow_rate
     +    ,umean
     +    ,u
     +    )

       write (2,900) i,u,sigma
       write (6,900) i,u,sigma

       sigma = sigma + step

      End Do

      write (2,900) Null,zero,a2
      write (2,900) Null

      close (2)

c---
c Done with a profile
c---

      write (6,*)
      write (6,901) flow_rate,umean
      write (6,*)

c-----
c Done
c-----

 900  Format (1x,i3,4(1x,f10.5))

 901  Format ("Flow rate     = ",f15.10
     +     ,/,"Mean velocity = ",f15.10
     +       )

      Stop
      End
