      program tube_crc_sw_trans_dr

c==========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c==========================================

c==========================================
c Transient swirling flow in a circular cylinder
c due to the sudden rotation of the cylinder
c with constant angular velocity
c==========================================

      Implicit Double Precision (a-h,o-z)

      Parameter (Ncln=64)

c----------
c constants
c----------

      pi  = 3.14159 265358D0
      pi2 = 2.0*pi

      Null   = 0
      None   = 1
      Ntwo   = 2
      Nthree = 3

      zero = 0.0

c------
c input
c------

      open (1,file="tube_crc_sw_trans.dat",status="unknown")

        read (1,*) nprof   ! number of point for a profile
        read (1,*) visc    ! viscosity
        read (1,*) den     ! density
        read (1,*) a       ! tube radius
        read (1,*) Omega   ! cylinder angular velocity

      close (1)

c---------------
c prepare to run
c---------------

      open (2,file="tube_crc_sw_trans.out",status="unknown")

      nprof1 = nprof+1
      step   = a/(nprof1-1.0)

c--------------------------------
cpPrint data to draw the cylinder
c--------------------------------

      Ncln1 = Ncln+1
      dth   = pi2/(Ncln1-1.0)

      write (2,900) Ncln1

      Do i=1,Ncln1

       theta  = dth*(i-1.0)
       xprint = a*cos(theta)
       yprint = a*sin(theta)

       write (2,900) i,xprint,yprint

      End Do

c--------------------------
c Draw the profile base line
c---------------------------

      Uclnd = Omega*a

      write (2,900) Nthree
      write (2,900) None  ,zero,zero
      write (2,900) Ntwo  ,zero,a
      write (2,900) Nthree,Uclnd,a

c---
c menu
c---

      write (6,*)
      write (6,*)  "            MENU"
      write (6,*)
      write (6,*)  " Enter 0 to quit"
      write (6,*)  "       1 for one profile"
      write (6,*)  "------------------------"
      read  (5,*) menu

      if(menu.eq.0) Go to 99

c------------
c One profile
c------------

      if(menu.eq.1) then

 94   Continue

      write (6,*) 
      write (6,*) " Please enter the time"
      write (6,*) "            99 to quit"
      write (6,*) " ---------------------"
      read (5,*) time

      if(time.eq.99) Go to 99

      write (6,*)
      write (6,*) "u, sigma"
      write (6,*)

      write (2,900) nprof1,time

      sigma = 0.0

      Do i=1,nprof1

       call tube_crc_sw_trans
     +
     +   (a
     +   ,visc
     +   ,den
     +   ,Omega
     +   ,time
     +   ,sigma
     +   ,u
     +   )

       write (2,900) i,u,sigma
       write (6,900) i,u,sigma

       sigma = sigma + step

      End Do

c---
c Done with a profile
c---

      Go to 94

      End If

c-----
c Done
c-----

  99  Continue

      write (2,900) Null
      close (2)

 900  Format (1x,i3,4(1x,f10.5))

      Stop
      End
