      subroutine tube_crc_trans
     +
     +  (a
     +  ,visc
     +  ,den
     +  ,pg
     +  ,time
     +  ,s
     +  ,u
     +  )

c=====================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement
c=====================================

c=====================================
c Transient flow in a circular tube
c due to the sudden application of 
c a constant pressure gradient
c
c SYMBOLS:
c -------
c
c a:    tube radius
c den:  fluid density
c visc: fluid viscosity
c s:    distance from axis
c
c aj0:  zeros of Bessel funciton J0
c=====================================

      Implicit Double Precision (a-h,o-z)

      Dimension aj0(10)

c---
c constants
c---

c--------------------------------
c zeros of the Bessel function J0
c
c From Abramowitz & Stegun, p.409
c--------------------------------

      aj0(1) =  2.40482 55577
      aj0(2) =  5.52007 81103
      aj0(3) =  8.65372 79129
      aj0(4) = 11.79153 44391
      aj0(5) = 14.93091 77086
      aj0(6) = 18.07106 39679
      aj0(7) = 21.21163 39679
      aj0(8) = 24.35247 15308
      aj0(9) = 27.49347 91320
      aj0(10)= 30.63460 64684

c---
c confirm the zeros
c---

c     Do j=1,10
c      write (6,*) bess_J0(aj0(j))
c     End Do
c     pause

c---
c prepare 
c---

      rnu = visc/den       ! kinematic viscosity
      as  = a**2
      tr  = time*rnu/as    ! reduced time
      sr  = s/a            ! reduced sigma

c---
c compute the velocity
c---

      sum = 0.0

      Do j=1,5

        tmp0 =   aj0(j)
        tmp1 =   aj0(j)*sr
        tmp2 = - aj0(j)**2 * tr
        sum = sum
     +      + bess_J0(tmp1)/bess_J1(tmp0)*exp(tmp2)/tmp0**3
      End Do

      u = pg*as*(0.25*(1.0-sr**2)-2.0*sum)/visc

c---
c done
c---

      Return
      End
