%---
clear all
close all
%---

%---------------------------------
% Transient flow in a circular tube
% due to the sudden application of 
% a constant pressure gradient
%
% SYMBOLS:
% -------
%
% a:    tube radius
% den:  fluid density
% visc: fluid viscosity
% s:    distance from axis
%
% aj0:  zeros of Bessel funciton J0
%
% Velocity computed using formula (5.2.22)
% of Pozrikidis (1997, p. 199)
%---------------------------------


nprof = 32;    % number of points for a profile
a = 1;         % tube radius
visc = 1.0;    % viscosity
den = 1.0;     % density
pg = 1.0;      % pressure gradient

%---
% prepare
%---

U = 0.25*pg*a^2/visc;
step = a/nprof;

figure(1)
hold on

%---
 for iloop=1:32
%---

     if(iloop==1) time=0.001;
 elseif(iloop==2) time=0.005;
 elseif(iloop==3) time=0.010;
 else
   time=time+0.01;
 end

 sig = 0;

 for i=1:nprof+1

  u = tube_crc_trans  ...
 ...
     (a ...
     ,visc ...
     ,den ...
     ,pg ...
     ,time ...
     ,sig ...
     );

  xplot(i) = u/U;
  yplot(i) = sig/a;

  sig = sig+step;
 end

 sig=0;
 for i=1:nprof+1
  parabolic(i) = 0.25*pg*(a*a-sig*sig)/visc/U;
  sig=sig+step;
 end

 plot(xplot, yplot,'k')
 plot(xplot,-yplot,'k')

%---
 end
%---

 plot(parabolic, yplot,'r--')
 plot(parabolic,-yplot,'r--')

 xmin=-0.05; xmax=1.2;
 ymin= -1.2; ymax=1.2;
 axis([xmin xmax ymin ymax])
 plot([0 0],[-a a],'r','linewidth',1);
 plot([xmin xmax],[a a],'r','linewidth',4);
 plot([xmin xmax],[-a -a],'r','linewidth',4);
 xlabel('u_x/U','fontsize',15)
 ylabel('y/a','fontsize',15)
 set(gca,'fontsize',15)
 box
