      program tube_crc_wom_dr

c=========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c=========================================

c=========================================
c Unidirectional flow in a circular tube 
c due to an oscillating pressure gradient
c
c The pressure gradient is given by:
c
c dpdx = - pgamp * sin(Omega*time)
c
c pgamp: amplitude
c Omega:  angular frequency
c=========================================

      Implicit Double Precision (a-h,o-z)

c---
c constants
c---

      Null = 0
      None = 1
      Ntwo = 2

      pi = 3.14159 265358D0

c-----------
c Read input
c-----------

      open (1,file="tube_crc_wom.dat")

        read (1,*) nprof   ! number of profile points
        read (1,*) visc    ! viscosity
        read (1,*) den     ! density
        read (1,*) pgamp   ! amplitude of the pressure gradient
        read (1,*) Omega   ! angular frequency
        read (1,*) a       ! tube radius

      close (1)

c--------
c prepare 
c--------

      open (2,file="tube_crc_wom.out",status="unknown")

c-----------------------------
c print data to plot tube wall
c-----------------------------

      span = 3.0*a

      write (2,900) Ntwo
      write (2,900) None,-span,a
      write (2,900) Ntwo, span,a

  94  Continue

      write (6,*)
      write (6,*) " Please enter Omega*t in multiples of pi"
      write (6,*) " Omega is the angular frequency"
      write (6,*) " 99 to quit"
      write (6,*) " ----------"
      read (5,*) ot

      if(ot.eq.99) Go to 99

c---
c profiles
c---

      ot = ot*pi
      nprof1 = nprof+1

      step = a/(nprof1-1.0)
      s    = 0.0D0

      write (2,900) nprof1

      write (6,*)
      write (6,*) "sigma, u"
      write (6,*)

      Do i=1,nprof1

       call tube_crc_wom
     +
     +    (a
     +    ,visc
     +    ,den
     +    ,pgamp
     +    ,Omega
     +    ,ot
     +    ,s
     +    ,u
     +    )

       write (2,900) i,u,s
       write (6,900) i,s,u

       s = s + step

      End Do

c---
c done with the profile
c---

      Go to 94

c-----
c done
c-----

  99  Continue

      write (2,900) null
      close (2)

 900  Format (1x,i3,4(1x,f10.5))

      Stop
      End
