      subroutine tube_ell
     +
     +  (a
     +  ,b
     +  ,visc
     +  ,den
     +  ,gac
     +  ,chi
     +  ,beta
     +  ,y,z
     +  ,flow_rate
     +  ,umean
     +  ,umax
     +  ,u
     +  )

c==========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement
c==========================================

c==========================================
c Steady flow through an elliptical tube
c due to a pressure gradient or gravity
c
c SYMBOLS:
c --------
c
c beta:  tube inclination angle
c gac:   magnitude of the acceleration of gravity
c==========================================

      Implicit Double Precision (a-h,o-z)

c---
c constants
c---

      pi  = 3.14159 265358

c---
c prepare
c---

      as = a**2
      bs = b**2
      area = pi*a*b

      pgmod = chi + den*gac*sin(beta)

c---
c velocity at (y,z)
c---

      ys = y**2
      zs = z**2

      u = 0.5*pgmod/visc * as*bs/(as+bs) * (1.0-ys/as-zs/bs)
  
c---
c flow rate, mean and max velocity
c---

      flow_rate = 0.25*pi*pgmod/visc * a*b*as*bs/(as+bs)
      umean     = flow_rate/area
      umax      = 0.5*pgmod/visc *as*bs/(as+bs)

c-----
c Done
c-----

      Return
      End
