      subroutine tube_rec
     +
     +  (a
     +  ,b
     +  ,visc
     +  ,den
     +  ,gac
     +  ,chi
     +  ,beta
     +  ,maxsm
     +  ,y,z
     +  ,flow_rate
     +  ,u_mean
     +  ,u_max
     +  ,u
     +  )

c================================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c================================================

c================================================
c Steady flow through a tube with rectangular
c cross-section
c due to a pressure gradient or gravity
c
c Origin is set at the tube center
c
c Formulae are given in Pozrikidis (1997, p. 187)
c
c SYMBOLS:
c --------
c
c beta:  tube inclination angle
c gac:   magnitude of the acceleration of gravity
c maxsm: truncation limit for infinite sum
c================================================

      Implicit Double Precision (a-h,o-z)

c----------
c constants
c----------

      pi = 3.14159 265358 D0

c---------------
c prepare to run
c---------------

      as  = a**2
      bs  = b**2
      aob = a/b

      pg_mod = chi + den*gac*sin(beta)  ! modified pressure gradient

      area = 4.0D0*a*b            ! tube cross-sectional area

c--------------------------
c compute:
c
c (a) velocity at (y,z)
c
c (b) Flow rate from formula (5.1.22)
c     of Pozrikidis (1997, p. 187)
c
c (c) max velocity
c     from formula (5.1.21) of Pozrikidis (1997,p. 187)
c--------------------------

      zs = z**2

      sumv = 0.0     ! velocity
      summ = 0.0     ! max velocity
      sumf = 0.0     ! flow rate

      sign = -1.0

      Do n=1,maxsm

       alpha = (n-0.5)*pi

       arg1 = alpha*y/b
       arg2 = alpha*aob
       arg3 = alpha*z/b
       deno  = cosh(arg2)*alpha**3
       sumv = sumv + sign*cosh(arg1)*cos(arg3)/deno
       summ = summ + sign/deno
       sumf = sumf + tanh(arg2)/alpha**5

       sign = -sign

      End Do

c---
c velocity
c---

      u = 0.5*pg_mod*(bs-zs + 4.0*bs*sumv)/visc

c---
c flow rate
c---

      F = 1.0-6.0*sumf/aob

      flow_rate = 4.0D0/3.0D0 *pg_mod*a*b**3*F/visc

      u_mean  = flow_rate/area

c---
c max velocity
c---

      u_max = 0.5*pg_mod/visc * (bs+4.0*bs*summ)

c-----
c Done
c-----

      Return
      End
