      subroutine films_pde(v)

c==========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c==========================================

c==========================================
c Compute the rate of change of y position
c of the interfaces:
c
c  v(i) = dy(i)/dt
c
c for i=1,...,NLR
c==========================================

      Implicit Double Precision (a-h,o-z)

      Dimension X (0:200,0:05),Y (0:200,0:05)
      Dimension Y1(0:200,0:05),Y2(0:200,0:05)
      Dimension Y3(0:200,0:05),Y4(0:200,0:05)
      Dimension  V(0:200,1:05)

c---
c internal variables
c---

      Dimension A(200,5),B(200,5),G(200,5)
      Dimension Q(0:200,5)
      Dimension dQdx(200,5)

      Dimension vs(5),vsr(5),rho(5),tension(5)

c---
c common blocks
c---

      common/XXYY/X,Y,Y1,Y2,Y3,Y4

      common/Int1/NLR,NLR0,NLR1,NSG,NSGa,NSG1,NSG2
      common/Int2/ICU

      common/REAL1/RL,gx,gy
      common/REAL2/vs,vsr,rho,tension

      common/piii/pi,pi2

c------------------------------------
c  Evaluation the derivatives DmY/Dxm 
c  at all interfaces
c  by central differences
c--------------------------------------

      Do j=1,NLR 

c---
c first derivative
c---

       Do i=1,NSG
        Y1(i,j) = (Y(i+1,j)-Y(i-1,j))/(X(i+1,j)-X(i-1,j))
       End Do

c---
c wrap
c---

          Y1(0,j) = Y1(NSG,j)
       Y1(NSG1,j) =   Y1(1,j)
       Y1(NSG2,j) =   Y1(2,j)

c---
c second derivative
c---

       Do i=1,NSG 
        Y2(i,j)=(Y1(i+1,j)-Y1(i-1,j))/(X(i+1,j)-X(i-1,j))
       End Do

c---
c wrap
c---

       Y2(0,   j) = Y2(NSG,j)
       Y2(NSG1,j) = Y2(1,  j)
       Y2(NSG2,j) = Y2(2,  j)

c---
c third derivative
c---

       Do i=1,NSG  ! third
         Y3(i,j) = (Y2(i+1,j)-Y2(i-1,j))/(X(i+1,j)-X(i-1,j))
       End Do

c---
c wrap
c---
 
       Y3(0,   j) = Y3(NSG,j) 
       Y3(NSG1,j) = Y3(1,  j)
       Y3(NSG2,j) = Y3(2,  j)

      End Do

c--------------------------------------
c Compute the effective pressure drop G
c--------------------------------------

      Do i=1,NLR    ! over interfaces
       Do k=1,NSG   ! over points

        dpdxm = 0.0D0

        Do j=i,NLR
         dpdxm= dpdxm + tension(j)*Y3(k,j)
     +        + gy*(rho(j)-rho(j+1))*Y1(k,j)
        End Do

        G(k,i) = 0.50D0*(dpdxm + rho(i)*gx)/vs(i)

       End Do
      End Do
       
c----------------------------
c Compute the coefficients B 
c----------------------------

c---
c free surface
c---

       Do k=1,NSG
        B(k,NLR) = 2.0D0*G(k,NLR)*y(k,NLR)
       End Do

c---
c recursive for interfaces
c---

       Do i=NLR-1,1,-1
        i1 = i+1
        Do k=1,NSG
         B(k,i) = 2.0D0*G(k,i)*y(k,i)
     +          +vsr(i)*(B(k,i1)-2.0D0*G(k,i1)*y(k,i))
        End Do
       End Do

c----------------------------
c Compute the coefficients A 
c----------------------------

       Do k=1,NSG
        A(k,1) = y(k,0)*(y(k,0)*G(k,1)-B(k,1))
       End Do

       Do i=1,NLR-1
        i1 = i+1
        Do k=1,NSG
        A(k,i1) = A(k,i )
     +           +B(k,i )*y(k,i)
     +           -G(k,i )*y(k,i)**2
     +           -B(k,i1)*y(k,i)
     +           +G(k,i1)*y(k,i)**2
        End Do
       End Do

c--------------------------
c Compute the flow rates Q 
c--------------------------

      Do i=1,NLR
       ia = i-1
       Do k=1,NSG
       q(k,i)    = A(k,i)*(y(k,i)   -y(k,ia))
     +      +0.5D0*B(k,i)*(y(k,i)**2-y(k,ia)**2)
     +      -    G(k,i)*(y(k,i)**3-y(k,ia)**3)/3.0D0
       End Do
      End Do

c---
c wrap around
c---

      Do i=1,NLR
       Q(0   ,i) = Q(NSG,i)
       Q(NSG1,i) = Q(1  ,i)
       Q(NSG2,i) = Q(2  ,i)
      End Do

c---
c compute dQ/dx
c
c by central differences (ICU =  1)
c or upwind  differences (ICU ne 1)
c---

c---
      If(ICU.eq.1) then     ! central differences
c---

       Do k=1,NSG
        ka = k-1
        k1 = k+1
        Do i=1,NLR
          dqdx(k,i)=(Q(k1,i)-Q(ka,i))/(x(k1,i)-x(ka,i)) 
        End Do
       End Do

c---
      Else               ! upwind differences
c---

       Do k = 1,NSG
        ka = k-1
        Do i=1,NLR
         dQdx(k,i)=(Q(k,i)-Q(ka,i))/(x(k,i)-x(ka,i)) 
        End Do
       End Do

c---
      End If
c---

c---
c finally compute v according to equation (13)
c of Pozrikidis (1998)
c---

      Do i=1,NLR

        Do k=1,NSG
          sum = 0.0D0
          Do j=1,i
            sum = sum-dqdx(k,j)
          End Do
          v(k,i) = sum
        End Do

c-----
c wrap 
c-----

           v(0,i) = v(NSG,i)
        v(NSG1,i) =   v(1,i)
        v(NSG2,i) =   v(2,i)

      End Do

c-----------------
c print velocities
c-----------------

c     Do i=1,NLR
c       write (6,100) i,(v(k,i),k=1,NSG)
c     End Do

c-----
c Done
c-----

  100 Format (1x,i3,10(1x,f10.6))
  101 Format (1x,i3,1x,i3,10(1x,f10.6))

      Return
      End 
