%---
close all
clear all
%---

%===============================
% draw a NACA four-digit airfoil
%===============================

chord = 1.0;
xcam = 0.4;  % x position of the camber
cam = 0.05;  % camber
thick =0.1; % thickness
n = 32;  % number of nodes around the airfoil

%---
% prepare
%---

dpsi = 2*pi/n;

%---
% prepare to plot
%---

figure(1)
hold on
%axis([-0.1 1.1 -0.2 0.2])
axis equal
axis([-0.1 1.1 -0.3 0.4])
xlabel('x','fontsize',13)
ylabel('y','fontsize',13)
set(gca,'fontsize',13)
box on

%---

for i=1:n

  psi = (i-1.0)*dpsi;
  x = 0.5*(1.0+cos(psi));

%---
% camber line
%---

 if(x<xcam)
  yc(i) = cam*(2.0*xcam*x-x*x)/xcam^2;
 else
  yc(i) = cam/(1.0-xcam)^2*((1.0-2.0*xcam)+2.0*xcam*x-x*x);
 end

%---
% thickness
%---

 yt = 5.0*thick ...
     *(0.2969*sqrt(x)...
      -0.1260*x...
      -0.3516*x*x...
      +0.2843*x*x*x...
      -0.1036*x*x*x*x...
      );

  if(i<(n/2+1)) 
   ya(i) = (yc(i)-yt)*chord;
  else
   ya(i) = (yc(i)+yt)*chord;
  end

  xa(i) = x*chord;

end

xa(n+1) = chord;   % trailing edge
ya(n+1) = 0.0;     % trailing edge
yc(n+1) = 0.0;     % trailing edge

%---
% plotting
%---

plot(xa,ya,'ko-')
plot(xa,yc,'k--')
plot(xa(1),ya(1),'ro')
plot(xa(2),ya(2),'co')
