function [ep]=naca(chord,thick,Npl)

%----
% generate the NACA 23012 airfoil contour
%
% ep are the panel end-points
%----

dpsi = 2*pi/Npl;

ep(1,1) = chord;
ep(1,2) = 0.0;

for i=2:Npl

  psi = (i-1.0)*dpsi;
  x   = 0.5*(1.0+cos(psi));

%---
% camber line:
%---

  if(x<0.2025)
    yc = 2.6595*x*(0.1147+x*(-0.6075+x));
  else
   yc = 0.02208*(1.0-x);
  end

%---
% thickness:
%---

  yt = 5.0*thick*(0.2969*sqrt(x) ...
      -0.1260*x ...
      -0.3516*x*x ...
      +0.2843*x*x*x ...
      -0.1036*x*x*x*x ...
      );
  if(i<=(Npl/2+1))
   y = yc-yt;
  else
   y = yc+yt;
  end
   ep(i,1) = x*chord;
   ep(i,2) = y*chord;
end

ep(Npl+1,1) = chord;    % trailing edge
ep(Npl+1,2) = 0.0;      % trailing edge

%---
% done
%---

return




