      subroutine film0
     +
     +  (method
     +  ,h
     +  ,RL
     +  ,visc
     +  ,rho
     +  ,gamma
     +  ,gac
     +  ,th0
     +  ,eta
     +  ,cr,ci
     +  ,si
     +  )

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c----------------------------------------
c Linear stability of a viscous film resting 
c on a horizontal
c plane or flowing down an inclined plane 
c under gravity
c in the presence of surface tension
c
c Inertial effects are neglected
c
c The growth rate of computed so that the determinant of 
c the matrix in eq. (9.9.41) (Pozrikidis 1997, p. 472)
c is equal to zero.
c
c A closed form expression is given by Yih (1963)
c
c LEGEND:
c ------
c
c AR, AI: Real and imaginary parts of a matrix
c--------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Double Precision LR,LI    ! capital Lamda, real and imaginary

      Dimension AR(3,3),AI(3,3)

      Parameter (cons = 2.0D0)

c----------
c constants
c----------

      pi = 3.14159 265358 D0
      pi2 = 2.0D0*pi

c--------
c prepare
c--------

      a   = pi2*h/RL      ! wave number
      as  = a*a
      a2  = 2.0D0*a
      as2 = 2.0D0*as
      hs  = h*h

      sth0 = Dsin(th0)
      tth0 = Dtan(th0)

      umax = 0.50D0*rho*gac*sth0*hs/visc
      p    = 2.0D0/tth0 + gamma*as/(visc*umax)
      eta  = 0.50D0*p

      If(method.eq.2) Go to 98

c----------------------------------------------
c method described by Pozrikidis (1997, p. 474)
c----------------------------------------------

      rm = exp(-2.0D0*a)

      rm1 = 1.0D0-rm

c----
c first row of the matrix
c---

      AR(1,1) = a2
      AI(1,1) = 0.0
      AR(1,2) = 1.0
      AI(1,2) = 0.0
      AR(1,3) = 1.0
      AI(1,3) = 0.0

c---
c Will transform the complex equation Det(A) = 0
c into the real system
c
c    B11 * LR + B12 * LI + C1 = 0
c    B21 * LR + B22 * LI + C2 = 0
c
c where LR, LI are the real and imaginary
c parts of Lamda defined in eqs. (9.9.42)
c  
c linear system will arise by the method of impulses
c---

c---
c first, set LR = 0, LI = 0.
c and compute C1 and C2
c---

      LR = 0.0
      LI = 0.0

      AR(2,1) =  rm1*(as-LR)
      AI(2,1) = -rm1*    LI
      AR(2,2) =  as+a-LR
      AI(2,2) =      -LI
      AR(2,3) = (as-a-LR)*rm
      AI(2,3) =      -LI *rm

      AR(3,1) =  as2*(1.0D0+rm)-LI*p*rm1
      AI(3,1) =                 LR*p*rm1
      AR(3,2) =  as2-LI*p
      AI(3,2) =      LR*p
      AR(3,3) =  (-as2-LI*p)*rm
      AI(3,3) =        LR*p *rm

      call det_33c (AR,AI,DTR,DTI)

      C1 = DTR
      C2 = DTI

c---
c second, set LR = cons, LI = 0
c and compute B11 and B21
c---

      LR = cons
      LI = 0.0

      AR(2,1) =  rm1*(as-LR)
      AI(2,1) = -rm1*    LI
      AR(2,2) =  as+a-LR
      AI(2,2) =      -LI
      AR(2,3) = (as-a-LR)*rm
      AI(2,3) =      -LI *rm

      AR(3,1) =  as2*(1.0+rm)-LI*p*rm1
      AI(3,1) =              LR*p*rm1
      AR(3,2) =  as2-LI*p
      AI(3,2) =      LR*p
      AR(3,3) =  (-as2-LI*p)*rm
      AI(3,3) =        LR*p *rm

      call det_33c (AR,AI,DTR,DTI)

      B11 = (DTR-C1)/cons
      B21 = (DTI-C2)/cons

c---
c third, set LR = 0, LI = cons
c and compute B11 and B21
c---

      LR = 0.0
      LI = cons

      AR(2,1) =  rm1*(as-LR)
      AI(2,1) = -rm1*    LI
      AR(2,2) =  as+a-LR
      AI(2,2) =      -LI
      AR(2,3) = (as-a-LR)*rm
      AI(2,3) =      -LI *rm

      AR(3,1) =  as2*(1.0+rm)-LI*p*rm1
      AI(3,1) =              LR*p*rm1
      AR(3,2) =  as2-LI*p
      AI(3,2) =      LR*p
      AR(3,3) =  (-as2-LI*p)*rm
      AI(3,3) =        LR*p *rm

      call det_33c (AR,AI,DTR,DTI)

      B12 = (DTR-C1)/cons
      B22 = (DTI-C2)/cons

c---
c Solve the linear system
c---

      C1 = -C1
      C2 = -C2

      Cramer = B11*B22-B12*B21
      LR = (C1*B22-C2*B12)/Cramer
      LI = (C2*B11-C1*B21)/Cramer

c---
c Recover cr and ci
c---

      Par =  LR**2 + LI**2
      cr  =  LR/Par+1
      ci  = -LI/Par

c     cr = cr * umax
c     ci = ci * umax

      si  = ci*a

      cr1 = cr
      ci1 = ci
      si1 = si

c--------------------------
c Yih's analytical solution
c--------------------------

  98  Continue

      par = 1.0D0+cosh(a2)+as2
      cr = 1.0D0+2.0D0/par
      ci = -(sinh(a2)-a2)/as * eta/par

c     cr = cr * umax
c     ci = ci * umax

      si  = ci*a

c------
c print
c------

      if(method.eq.1) then
        write (6,100) cr,cr1,ci,ci1,si,si1
      else If(method.eq.2) then
c       write (6,100) cr,ci,si
      end If

c-----
c Done
c-----

  100 Format (5(3x,f9.6,1x,f9.6))

      Return 
      End
