      subroutine fourier
     +
     +   (RL      ! period
     +   ,Npts    ! number of data points
     +   ,x,y     ! data
     +   ,M       ! number of Fouriercoefficients
     +   ,af,bf   ! Fourier coefficients
     +   )

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c------------------------------------------
c Computation of the complex Fourier coefficients
c of a periodic function.
c
c Origin is shifted to the first point.
c
c Integrals are computed by the Simpson rule
c applied to overlapping parabolas
c
c SYMBOLS:
c -------
c
c L:    period
c wn:   wave number = 2 pi /L
c npts: number of data points
c x,y:	point coordinates
c
c M:             number of Fourier coefficients
c af(i), bf(i):  cosine and sine Fourier coefficients
c                i=0, ..., M
c
c------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Integer p

      Dimension x(200),y(200)
      Dimension af(0:200),bf(0:200)

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi2 = 2.0D0*pi

c--------
c prepare
c--------

      wn = pi2/RL  ! wave number

c-------------------------------
c Even/Odd number of intervals ?
c-------------------------------

      If(mod(npts,2).eq.0) then
c       write (6,*) "fourier: ",npts," intervals: Even"
        M  = npts/2
        fc = 0.5D0
      Else
c       write (6,*) "fourier: ",npts," intervals: Odd"
        M  = (npts-1)/2
        fc = 1.0D0
      End If

c-------
c extend
c-------

      x(npts+1) = x(1)+RL
      y(npts+1) = y(1)
      x(npts+2) = x(2)+RL
      y(npts+2) = y(2)

c-------
c launch
c-------

      Do p=0,M

       af(p) = 0.0D0
       bf(p) = 0.0D0

       Do i=1,npts

c--------------------
c complex transform
c by the Simpson rule
c--------------------

        i1 = i+1
        i2 = i+2

        xhat = x(i)-x(1)
        arg  = p*wn*xhat
        cs   = Dcos(arg)
        sn   = Dsin(arg)
        ff0  = cs*y(i)
        gg0  = sn*y(i)
        xx0  = xhat

        xhat = x(i1)-x(1)
        arg  = p*wn*xhat
        cs   = Dcos(arg)
        sn   = Dsin(arg)
        ff1  = cs*y(i1)
        gg1  = sn*y(i1)
        xx1  = xhat

        xhat = x(i2)-x(1)
        arg  = p*wn*xhat
        cs   = Dcos(arg)
        sn   = Dsin(arg)
        ff2  = cs*y(i2)
        gg2  = sn*y(i2)
        xx2  = xhat

        tm2 = (ff2-ff1)/(xx2-xx1)
        tm1 = (ff0-ff1)/(xx0-xx1)
        bbb = (tm2-tm1)/(xx2-xx0)
        ccc =  tm2-bbb*(xx2-xx1)

        dx2 = xx2-xx1
        dx0 = xx0-xx1

        af(p) = af(p) + ff1*(dx2   -dx0)
     +                + bbb*(dx2**3-dx0**3)/3.0D0
     +                + ccc*(dx2**2-dx0**2)/2.0D0

        tm2 = (gg2-gg1)/(xx2-xx1)
        tm1 = (gg0-gg1)/(xx0-xx1)
        bbb = (tm2-tm1)/(xx2-xx0)
        ccc =  tm2-bbb*(xx2-xx1)

        bf(p) = bf(p) + gg1*(dx2   -dx0)
     +                + bbb*(dx2**3-dx0**3)/3.0D0
     +                + ccc*(dx2**2-dx0**2)/2.0D0

       End Do

       af(p) = 0.5D0*af(p)       ! to account for overlap
       bf(p) = 0.5D0*bf(p)       ! to account for overlap

       af(p) = 2.0D0*af(p)/RL
       bf(p) = 2.0D0*bf(p)/RL

      End Do

      af(m) = fc*af(m)

c-----
c Done
c-----

      Return
      End
