      program vl

c========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c========================================

c-------------------------------------------
c Stability of an infinite vortex layer with
c uniform vorticity
c
c LEGEND:
c ------
c
c bcrit:  critical kb; if kb < bcrit, flow is unstable
c           
c cr: real part of c
c ci: imaginary part of c
c gr: growth rate
c pv: phase velocity

c a11m: ratio of amplitudes of the two vortex boundaries
c phi:	phase shift between vortex boundaries 
c phig:	phase shift between lower boundary and circulation 
c--------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      parameter (bcrit = 0.639)

c---
c constants
c---

      pi   = 3.14159 265358D0
      null = 0

c---
c input
c---

      write (6,*)
      write (6,*) " The velocity above and below the layer"
      write (6,*) " is equal to +(-)U0"
      write (6,*)
      write (6,*) " Please enter U0"
      write (6,*) " ---------------"
c     read  (5,*) U0
      U0 = 1

      write (6,*)
      write (6,*) " The thickness of the layer is 2b"
      write (6,*)
      write (6,*) " Will produce a stability graph"
      write (6,*) " of properties versus kb"
      write (6,*) " where k is the wave number"
      write (6,*)
      write (6,*) " Please enter the maximum kb"
      write (6,*) " ---------------------------"
c     read  (5,*) bmax
      bmax = 4.0

      write (6,*)
      write (6,*) " Please enter the number of graph-points"
      write (6,*) " ---------------------------------------"
c     read  (5,*) np
      np = 2*2*128

      write (6,*)
      write (6,*) " Results will be put in file vl.out"
      write (6,*) " Also show the screen?"
      write (6,*)
      write (6,*) " Enter 1 for YES, 0 for NO"
      write (6,*) " --------------------------"
c     read  (5,*) IO
      IO = 0

c---
c prepare
c---

      open (2,file="vl.out")

      np1 = np+1
      write (2,100) np1

c----

      if(IO.eq.1) then
       write (6,*) 'kb, ci, growth rate, cr, ampl rat, Dphi, Dphig'
       write (6,*) 
      end if

c----

      db = bmax/(np1-1.0)
      b  = 0.000001

c---
c run over wave numbers kb
c---
      Do i=1,np1
c---

      b2    = 2.0*b
      b4    = 4.0*b
      beta  = exp(b2)
      betas = beta*beta

      radical = (1.0-b2)**2-exp(-b4)

c------------------------
      if(b.lt.bcrit) then
c------------------------

       radical = -radical

       c        = sqrt(radical)/b2
       ci       = c
       gr       = b*c
       ph_vel   = 0.0

       ! numerator

       rnr  = betas*(1.0-b2)-1.0
       rni  = b2*betas*c
       rnm  = sqrt(rnr**2+rni**2)
       an   = acos(rnr/rnm)
       if(rni.le.0) an = - an

       ! denominator

       rdr  = beta*b2
       rdi  = rdr*c
       rdm  = sqrt(rdr**2+rdi**2)
       ad   = acos(rdr/rdm)
       if(rdi.le.0) ad = - ad

       ! fraction

       a11m = rnm/rdm
       phi  = (an-ad)/pi

       ! circulation

       rnr = b2*beta*(1.0+beta)-betas+1.0
       rni = b2*beta*c*(1.0-beta)
       rnm = sqrt(rnr**2+rni**2)
       an  = acos(rnr/rnm)
       if(rni.le.0) an = - an

       phig  = -(an-ad)/pi
       
c---------
      else
c---------

       c      = sqrt(radical)/b2
       ci     = 0.0
       gr     = 0.0
       ph_vel = c*U0
       a11m   = -(betas*(1.0-b2)-1.0+b2*betas*c)/(beta*b2*(1.0+c))
       phi    = 1.0
       phig   = 0.0

c-----------
      end if
c-----------

      if(IO.eq.1) 
     +write (6,100) i,b,ci,gr,ph_vel,a11m,phi,phig
      write (2,100) i,b,ci,gr,ph_vel,a11m,phi,phig

      b = b+db

      End Do

c-----
c Done
c-----

      write (2,100) null
      close(2)

 100  Format (1x,i3,10(1x,f8.5))

      stop
      end
