      program pvm_pr

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c-----------------------------------
c Motion of a periodic row of point vortices
c
c SYMBOLS:
c -------
c
c x, y:   coordinates of the point vortices
c circ:   circulation of the point vortices
c---------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension x(-10:100),y(-10:100),c(-10:100)
      Dimension u(-10:100),v(-10:100)

      Dimension xsave(-10:100),ysave(-10:100),csave(-10:100)
      Dimension usave(-10:100),vsave(-10:100)

      Dimension xsm(-10:100),ysm(-10:100)

c--------------
c common blocks
c--------------

      common/xxyy/x,y,c
      common/uuvv/u,v

      common/row/RL

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi2 = 2.0D0*pi

c---
c set parameters
c---

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 to read parameters from the data file"
      write (6,*) " 2 to type in parameters"
      write (6,*) " 0 to quit"
      write (6,*) '----------'
      read  (5,*) Ienrd

      if(Ienrd.eq.0) Go to 99

c------------------------
      if(Ienrd.eq.2) then
c------------------------

      write (6,*) 
      write (6,*) 'How many point vortices?'
      write (6,*) '------------------------'
      read  (5,*) Npv

      write (6,*)
      write (6,*) 'Enter the period'
      write (6,*) '----------------'
      read  (5,*) RL
      write (6,*)

      write (6,*)
      write (6,*) 'Enter the velocity difference'
      write (6,*) '      across the periodic row'
      write (6,*) '-----------------------------'
      read  (5,*) DU

      write (6,*)
      write (6,*) 'The point vortices will be displaced in the'
      write (6,*) 'x and y directions to form a wave'
      write (6,*)
      write (6,*) 'Enter the displacement amplitudes epsx and epsy'
      write (6,*) '-----------------------------------------------'
      read  (5,*) epsx,epsy

      write (6,*)
      write (6,*) 'Enter the number of smoothing passes'
      write (6,*) '      0 for none'
      write (6,*) '----------------'
      read  (5,*) Nsmooth

      write (6,*)
      write (6,*) 'Enter the time step'
      write (6,*) '-------------------'
      read  (5,*) Dt

      write (6,*)
      write (6,*) 'After how many steps print the position?'
      write (6,*) '----------------------------------------'
      read  (5,*) Nprint

      write (6,*)
      write (6,*) 'Enter number of time steps till pause'
      write (6,*) '-------------------------------------'
      read  (5,*) Nstep

c---------
      else
c---------

       open (3,file="pvm_pr.dat")

        read (3,*) Dt
        read (3,*) Nprint
        read (3,*) Nstep
        read (3,*)
        read (3,*) RL
        read (3,*) DU
        read (3,*) epsx,epsy
        read (3,*) Nsmooth
        read (3,*)
        read (3,*) Npv

       close (3)

c-----------
      end if
c-----------

c----------------------------------
c set initial position and strength
c----------------------------------

       wn = pi2/RL   ! wave number
       Dx = RL/Npv

       Do i=1,Npv
        xunp = (i-0.5D0)*Dx
        x(i) = xunp + epsx * sin(wn*xunp)
        y(i) =        epsy * sin(wn*xunp)
        x(i) = x(i)-RL/2.0
        c(i) = DU/(RL*Npv)
       End Do

c----------------
c prepare to run
c----------------

      open (4,file="pvm_pr.out")

      write (4,*) RL

      time   = 0.0D0
      Iprint = Nprint
      Jprint = 0

      Kstep = 1      ! Global counter
      Istep = 1      ! Batch  counter

c-.-.-.-.-.-.-.-.-.-.-.-
c SIMULATION BEGINS HERE
c-.-.-.-.-.-.-.-.-.-.-.-

  98  Continue

c---------------------
c compute the velocity
c---------------------

      call pvm_pr_vel
     +
     +     (Npv
     +     ,x,y,c
     +     ,RL
     +     ,u,v
     +     )

      x(Npv+1) = x(1)+RL
      y(Npv+1) = y(1)
      u(Npv+1) = u(1)
      v(Npv+1) = v(1)

c---
c displaying
c---

      write (6,*) "time = ",time

c     Do i=1,Npv
c       write (6,100) Istep,x(i),y(i),u(i),v(i)
c     End Do

c---
c  printing session
c---

      If(Iprint.eq.Nprint) then
         write (4,100) Npv+1,time
         Do i=1,Npv+1
          write (4,100) i,x(i),y(i)
c    +                   ,u(i),v(i)
c         write (6,100) i,x(i),y(i)
c    +                   ,u(i),v(i)
         End Do
         Iprint = 0
      End If

c---
c time integration by RK2
c---

      Do i=1,Npv
       xsave(i) = x(i)
       ysave(i) = y(i)
       usave(i) = u(i)
       vsave(i) = v(i)
       csave(i) = c(i)
       x(i) = x(i) + u(i)*Dt
       y(i) = y(i) + v(i)*Dt
      End Do

      call pvm_pr_vel
     +
     +     (Npv
     +     ,x,y,c
     +     ,RL
     +     ,u,v
     +     )

      Dth = 0.5D0*Dt

      Do i=1,Npv
       x(i) = xsave(i) + (usave(i)+u(i))*Dth
       y(i) = ysave(i) + (vsave(i)+v(i))*Dth
      End Do

      x(Npv+1) = x(1)+RL
      y(Npv+1) = y(1)
      u(Npv+1) = u(1)
      v(Npv+1) = v(1)

c-----------------------
c wrap around and smooth
c a periodic array
c-----------------------

      Do Ismooth=1,Nsmooth

       x( 0) = x(Npv  )-RL
       y( 0) = y(Npv  )
       x(-1) = x(Npv-1)-RL
       y(-1) = y(Npv-1)
       x(-2) = x(Npv-2)-RL
       y(-2) = y(Npv-2)

       x(Npv+1) = x(1)+RL
       y(Npv+1) = y(1)
       x(Npv+2) = x(2)+RL
       y(Npv+2) = y(2)

       Do i=1,Npv
        xsm(i) = (-x(i-2)+4.0D0*x(i-1)+10.0*x(i)
     +                   +4.0D0*x(i+1)-x(i+2))/16.0D0
        ysm(i) = (-y(i-2)+4.0D0*y(i-1)+10.0*y(i)
     +                   +4.0D0*y(i+1)-y(i+2))/16.0D0
       End Do

       Do i=1,Npv
         x(i) = xsm(i)
         y(i) = ysm(i)
       End Do

c---
      End Do
c---

c---
c reset counters and time
c---

      Kstep  = Kstep+1
      Istep  = Istep+1
      Iprint = Iprint+1
      time   = time+Dt

c---
c batch inquiry
c---

      If(Istep.eq.Nstep) then

       write (6,*) 
       write (6,*) " How many steps till pause?"
       write (6,*) " 0 to quit"
       write (6,*) " ---------"

       read  (5,*) Nstep

       If(Nstep.eq.0) Go to 99

       Istep = 1

       Go to 98

      End If
  
      Go to 98

c-----------
c pack it in
c-----------

  99  Continue

      write (4,100) null

      close (4)

c-----
c Done
c-----

 100  Format (1x,i4,10(1x,f10.5))

      Stop
      End
