      program vp_ax

c-----------------------------------------
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licencing agreement.
c----------------------------------------

c--------------------------------------------
c Dynamical simulation of the motion of a collection of
c axisymmetric vortex patches whose vorticity
c is a linear function of the distance for the axis,
c simulate by the method of contour dynamics
c
c Point advancemene by the second-order Runge-Kutta
c method
c
c Variables ending with g are global
c
c For example:
c
c  xg(i,l): x-position of ith point on lth contour
c  yg(i,l): y-position of ith point on lth contour
c
c----------------------------------

      Implicit Double Precision (a-h,o-z)

c---
c global variables
c---

      Dimension xg(0:166,25),yg(0:166,25),sg(0:166,25)
      Dimension ug(0:166,25),vg(0:166,25)

      Dimension XCg  (166,25),YCg (166,25),Rg  (166,25)
      Dimension TH1g (166,25),TH2g(166,25),TH3g(166,25)
      Dimension orntg(166,25)

c--- related to second-order Runge-Kutta (RK2)
c--- sv stands for save

      Dimension Xsv(166,25),Ysv(166,25)
      Dimension Usv(166,25),Vsv(166,25)

c--- contour and patch properties

      Dimension NSG  (25),NSG1(25),NSG2(25)
      Dimension omega(25)

      Dimension xdc(25)
      Dimension aob(25),req(25),awob(25),bwob(25)
      Dimension peri0(25),epif0(25),ogos0(25)

c---
c local variables
c---

      Dimension X(0:200),Y(0:200)
      Dimension U(0:200),V(0:200)
      Dimension unused1(0:200),unused2(0:200)
      Dimension unused3(0:200),unused4(0:200)
      Dimension unused5(0:200)

      Dimension XC (200),YC (200),R  (200),S   (200)
      Dimension TH1(200),TH2(200),TH3(200),ORNT(200)

c---
c  related to time stepping
c---

      Dimension time(0:2000)

      Dimension peri(0:2000,25),epif(0:2000,25),ogos(0:2000,25)
      Dimension xcen(0:2000,25),ycen(0:2000,25)

c---
c  various
c---

      Dimension ZZ(20),WW(20)      ! Gauss-Legendre weights

c----------------------------------
c common blocks for global variables
c----------------------------------

      common/XXYYg/Xg,Yg
      common/ARCCg/XCg,YCg,Rg,Sg,TH1g,TH2g,TH3g,ORNTg
      common/UUVVg/Ug,Vg

c---
c common blocks for local variables
c---

      common/XXYY/X,Y
      common/ARCC/XC,YC,R,S,TH1,TH2,TH3,ORNT
      common/UUVV/U,V

c---
c various common blocks
c---

      common/ancI1/Ncntr,NSG,NSG1,NSG2
      common/ancI3/NGL
      common/ancR1/omega

      common/ZZWW/ZZ,WW
      common/piii/pi,pih,piq,pi2,pi4,pi6,pi8,srpi

c---
c Constants
c---

      pi   = 3.14159 265358 979 32384
      pih  = 0.5 * pi
      piq  = 0.25* pi
      pi2  = 2.0 * pi
      pi4  = 4.0 * pi
      pi6  = 6.0 * pi
      pi8  = 8.0 * pi
      srpi = sqrt(pi)

      Null = 0
      None = 1
      Ntwo = 2

      zero = 0.0

      oot = 1.0/3.0
      tot = 2.0/3.0

c----------------------
c Simulation parameters
c----------------------

      write (6,*)
      write (6,*) ' Type 0 to enter type in parameters'
      write (6,*) '      1 to read them'
      write (6,*) '        from input file: vp_2d.dat'
      write (6,*)
      write (6,*) '  Warning: if you type 0 will spend a lot'
      write (6,*) '           of time typing'
      write (6,*) ' ---------------------------------------'
      read  (5,*)   Ienrd

c------------------------
      If(Ienrd.eq.0) then
c------------------------

      write (6,*)
      write (6,*) ' Enter the number of vortices'
      write (6,*) ' ----------------------------'
      read  (5,*) Ncntr

      write (6,*) ' Enter the vorticity coefficient of the patches'
      write (6,*) ' one by one'
      write (6,*) ' ----------------------------------------------'
      read  (5,*)  (omega(i),i=1,Ncntr)

      write (6,*)
      write (6,*) ' Enter the number of Gauss integration points'
      write (6,*) ' over each arc'
      write (6,*)
      write (6,*) ' Choose from 2,6,12,20; default is 20'
      write (6,*) ' -------------------------------------------'
      read  (5,*)   NGL

      write (6,*)
      write (6,*) ' Enter the amximum angle subtended by an arc'
      write (6,*) '                          in multiples of pi'
      write (6,*) ' -------------------------------------------'
      read  (5,*)   thmax

      write (6,*)
      write (6,*) ' Enter the minimum and maximum marker point'
      write (6,*) '       separation'
      write (6,*) ' ------------------------------------------'
      read  (5,*)   SPmin,SPmax

      write (6,*)
      write (6,*) ' Normalize patch areas after each step ?'
      write (6,*)
      write (6,*) ' Enter 0 for no, 1 for yes'
      write (6,*) ' ----------------------'
      read  (5,*)   Norm

c------------------
c Initial condition
c------------------

      write (6,*)
      write (6,*) ' Choose the initial shapes'
      write (6,*)
      write (6,*) ' Enter 1 for circular or elliptical'
      write (6,*) '       2 for wobbly'
      write (6,*) ' ---------------------------------------'
      read  (5,*)   Ishape

      If(Ishape.eq.1) then

       write (6,*) ' --------------------------------'
       write (6,*) ' Circular or elliptical shapes'
       write (6,*)
       write (6,*) ' Enter the axis ratio a/b'
       write (6,*) ' one by one'
       write (6,*) ' --------------------------------'
       read  (5,*)  (aob(i),i=1,Ncntr)

       write (6,*)
       write (6,*) ' Enter the equivalent patch radii'
       write (6,*) ' one by one'
       write (6,*) ' --------------------------------'
       read  (5,*)  (req(i),i=1,Ncntr)

      Else If(Ishape.eq.2) then

       write (6,*) ' ------------------------------'
       write (6,*) ' Wobbly shapes'
       write (6,*) ' Enter the coefficients a and b'
       write (6,*) ' one by one'
       write (6,*) ' ------------------------------'

       Do i = 1,Ncntr
        read (5,*) awob(i),bwob(i)
       End Do

      End If

      write (6,*)
      write (6,*) ' Enter the x positions of the patch centers'
      write (6,*) ' one by one'
      write (6,*) ' -----------------------------------------'

      Do i = 1,Ncntr
       read (5,*) xdc(i)
      End Do

      write (6,*)
      write (6,*) ' Enter the number of points around each contour'
      write (6,*) '                                Maximum is 90'
      write (6,*) ' one by one'
      write (6,*) ' ----------------------------------------------'

      Do i = 1,Ncntr
       read (5,*) NSG(i)
      End Do

c----
c Related to time stepping
c---

      write (6,*)
      write (6,*) ' After how many steps of Printout ? '
      write (6,*) ' -----------------------------------'
      read  (5,*)   Nprint

      write (6,*)
      write (6,*) ' Enter the time step Dt'
      write (6,*) ' ----------------------'
      read  (5,*)   Dt

c-----------------------------
      Else If(Ienrd.eq.1) then
c-----------------------------

      open (3,file='vp_ax.dat')

        read (3,*) NGL
        read (3,*)
        read (3,*) THmax
        read (3,*) SPmin
        read (3,*) SPmax
        read (3,*)
        read (3,*) Iread
        read (3,*)
        read (3,*) Ishape
        read (3,*)
        read (3,*) Norm
        read (3,*) Nprint
        read (3,*)
        read (3,*) Dt
        read (3,*)
        read (3,*) Ncntr
        read (3,*)

        Do i=1,Ncntr
         read (3,*) ipatch,xdc(i) ! patch centers
        End Do

c--
c If index =0,
c properties are uniform
c--

        read (3,*) 
        read (3,*) Index,omega_ref

        Do i =1,Ncntr
          omega(i) = omega_ref
          If(Index.eq.1) read (3,*) idle,omega(i)
        End Do

        read (3,*) Index,nsg_ref   ! points around contours

        Do i =1,Ncntr
          nsg(i) = nsg_ref
          If(Index.eq.1) read (3,*) idle,nsg(i)
        End Do

        read (3,*) Index,aob_ref      ! aspect ratio

        Do i =1,Ncntr
          aob(i) = aob_ref
          If(Index.eq.1) read (3,*) idle,aob(i)
        End Do

        read (3,*) Index,req_ref     ! equivalent radius

        Do i =1,Ncntr
          req(i) = req_ref
          If(Index.eq.1) read (3,*) index,req(i)
        End Do

        read (3,*) Index,awob_ref     ! first shape parameter

        Do i =1,Ncntr
          awob(i) = awob_ref
          If(Index.eq.1) read (3,*) idle,awob(i)
        End Do
        read (3,*) Index,bwob_ref     ! second shape parameter

        Do i =1,Ncntr
          bwob(i) = bwob_ref
          If(Index.eq.1) read (3,*) idle,bwob(i)
        End Do

c-----------
      End If    ! end of reading properties
c-----------

c---
c printing
c---

      write (6,*)
      write (6,108)
      write (6,*)

      Do j=1,Ncntr
       write (6,107) j,xdc(j),aob(j),req(j),awob(j),bwob(j)
      End Do

c---
c prepare
c---

      thmax = thmax*pi

      call gauss_leg (NGL,ZZ,WW)

c--
c initial patch perimeters and areas
c---

      Do i=1,Ncntr
        If(Ishape.eq.1) then
         peri0(i) = pi      * req(i)
         epif0(i) = pi4     * req(i)**2
         ogos0(i) = pi4/3.0 * req(i)**3
        Else
         peri0(i) = pi2
         epif0(i) = pi
         ogos0(i) = pi
        End If
      End Do

c------------------------
c Generate initial shapes
c------------------------

      If(Iread.eq.0) then

       time(1) = 0.0

       Do j = 1,Ncntr     !--------- over contours

        write (6,*)
        write (6,*) " Generating contour:",j
        write (6,*)
c---
      If(Ishape.eq.1) then
c---
        NSG1(j) = NSG(j)+1
        NSG2(j) = NSG(j)+2
        step    = pi/(NSG1(j)-1.0)
        rrx     = req(j) * aob(j)**tot
        rry     = rrx/aob(j)

        Do i = 0,NSG2(j)
          tt = (i-1.0)*step
          cs = cos(tt)
          sn = sin(tt)
          xg(i,j) = xdc(j) + rrx * cs
          yg(i,j) =          rry * sn
        End Do
c---
      Else If(Ishape.eq.2) then
c---
        NSG1(j) = NSG(j)+1
        NSG2(j) = NSG(j)+2
        step    = pi/(NSG1(j)-1.0)

        Do i = 0,NSG2(j)
          TT = (i-1.0)*step
          TH = TT  + awob(j)*sin(8.0*TT)
          RR = 1.0 + bwob(j)*cos(4.0*TT)
          xg(i,j) = xdc(j) + RR*cos(TH)
          yg(i,j) =          RR*sin(TH)
        End Do
c---
      End If
c---

      End Do

c----------------------------------
c Read position of marker points
c      from file: vp_ax.inp
c-----------------------------------

      Else If(Iread.eq.1) then

        open (2,file='vp_ax.inp')

        Do j = 1,Ncntr             !------ loop over contours
          read (2,*) NSG1(j),icntr,time(1)
          Write (6,*) "Reading contour ",j
            Do i = 1,NSG1(j)
              read(2,*) idle,xg(i,j),yg(i,j)
            End Do
          NSG (j) = NSG1(j)-1
          NSG2(j) = NSG (j)+2
        End Do             

        close (2)

c---
      End If   !------------ End of Iread

c-------
c extend
c-------

      Do j=1,Ncntr
           Xg(0,j)    =  Xg(2,j)
           Yg(0,j)    = -Yg(2,j)
        Xg(NSG2(j),j) =  Xg(NSG(j),j)
        Yg(NSG2(j),j) = -Yg(NSG(j),j)
      End Do

c----------------------
c Display initial shape
c on the screen
c----------------------

       write (6,*)
       write (6,*)  ! initial profiles
       write (6,*)

       Do j = 1,Ncntr
        write (6,*) j
        Do i = 0,NSG2(j)
          write (6,100) j,i,xg(i,j),yg(i,j)
        End Do
       End Do

c---
c Prepare output files
c---

      open (1,file='vp_ax.xy')     ! for profiles
      open (4,file='vp_ax.diag')   ! for diagnostics

c--
c Prepare to run
c---

      read (3,*)

      Kstep = 1     ! Gloval step counter

c===========================
c  TIME STEPPING BEGINS HERE
c===========================

  90  Continue

      If(Ienrd.eq.0) then
        write (6,*)
        write (6,*) 'Enter number of steps before pausing'
        write (6,*) '                           0 to quit'
        write (6,*) '------------------------------------'
        read  (5,*)  Nstep
      Else
        Read  (3,*)   Nstep
        Write (6,102) Nstep
      End If

      If(Nstep.eq.0) Go to 99
      write (4,102) Nstep

c---
c initialize
c---

      Istep  = 1
      Iprint = Nprint

  97  Continue

      write (6,111) Istep,Nstep

c-------------------------------
c Check the point distribution
c Compute properties of arcs
c-------------------------------

      Do j = 1,Ncntr    !----- over contours

      write (6,*) "Checking contour:",j

c---
c transfer into local variables
c---

      Do i = 0,NSG2(j)
       x(i) =  xg(i,j)
       y(i) =  yg(i,j)
      End Do

      ICH1 = 1
      ICH2 = 1
      ICH3 = 1

        call prd_ax
     +
     +           (NSG(j)
     +           ,ICH1,THMAX
     +           ,ICH2,SPMAX
     +           ,ICH3,SPMIN
     +           ,peri(Kstep,j),epif(Kstep,j),ogos(Kstep,j)
     +           ,xcen(Kstep,j),ycen(Kstep,j)
     +           ,unused1,unused2,unused3,unused4,unused5
     +           ,Istop
     +           )

        If(Istop.eq.1) Go to 99

        NSG1(j) = NSG(j)+1
        NSG2(j) = NSG(j)+2

c---------------------
c Normalize patch area
c---------------------

      If(Norm.eq.1) then

        write (6,*) " Normalize volume of each vortex : ",j

        epf = epif(Kstep,j)
        ogo = ogos(Kstep,j)
        xcn = xcen(Kstep,j)
        ycn = ycen(Kstep,j)

        scale = 1.0/(ogo/ogos0(j))**oot

        Do i  = 0,NSG2(j)
         X(i) = xcn +(x(i)-xcn)*scale
         Y(i) = ycn +(y(i)-ycn)*scale**2
        End Do

        ICH1 = 0
        ICH2 = 0
        ICH3 = 0

        call prd_ax
     +
     +           (NSG(j)
     +           ,ICH1,THMAX
     +           ,ICH2,SPMAX
     +           ,ICH3,SPMIN
     +           ,peri(Kstep,j),epif(Kstep,j),ogos(Kstep,j)
     +           ,xcen(Kstep,j),ycen(Kstep,j)
     +           ,unused1,unused2,unused3,unused4,unused5
     +           ,Istop
     +           )

        If(Istop.eq.1) Go to 99

      End If

c----
c transfer into global variables
c---

      Do i =1,NSG1(j)
            xg(i,j) = x   (i)
            yg(i,j) = y   (i)
           xcg(i,j) =  xc (i)
           ycg(i,j) =  yc (i)
            rg(i,j) =   r (i)
            sg(i,j) =   s (i)
          th1g(i,j) = th1 (i)
          th2g(i,j) = th2 (i)
          th3g(i,j) = th3 (i)
         orntg(i,j) = ornt(i)
      End Do

c---
c extend
c and prevent drifting of points 
c that lie on the x axis
c
c compute the position of point 1 and NSG1
c by extrapolation to avoid instabilities
c---

      xg(0,j) =  xg(2,j)
      yg(0,j) = -yg(2,j)
      sg(0,j) = -sg(2,j)

      Xg(1,j) = (Xg(2,j)*Yg(3,j)**2-Xg(3,j)*Yg(2,j)**2)/
     +          (        Yg(3,j)**2-        Yg(2,j)**2)
      Yg(1,j) = 0.0
      Xg(NSG1(j),j) = (Xg(NSG(j)  ,j)*Yg(NSG(j)-1 ,j)**2
     +                -Xg(NSG(j)-1,j)*Yg(NSG(j)  ,j)**2)/
     +                (Yg(NSG(j)-1,j)**2-Yg(NSG(j),j)**2)
      Yg(NSG1(j),j) = 0.0

      xg(NSG2(j),j) =  xg(NSG(j),j)
      yg(NSG2(j),j) = -yg(NSG(j),j)
      sg(NSG2(j),j) =  2.0*sg(NSG1(j),j)-sg(NSG(j),j)

c---
c Reduce drop arc length, area, volume
c---

      peri(Kstep,j) = peri(Kstep,j)/peri0(j)
      epif(Kstep,j) = epif(Kstep,j)/epif0(j)
      ogos(Kstep,j) = ogos(Kstep,j)/ogos0(j)


      End Do                !------ loop over drops

c---
c Printing session
c---

      write (6,*)
      write (6,113) time(Kstep),Kstep
      write (6,*)
      write (6,305) (peri(Kstep,j),j=1,Ncntr)
      write (6,306) (epif(Kstep,j),j=1,Ncntr)
      write (6,309) (ogos(Kstep,j),j=1,Ncntr)
      write (6,307) (xcen(Kstep,j),j=1,Ncntr)
      write (6,308) (ycen(Kstep,j),j=1,Ncntr)

      write (4,113) time(Kstep),Kstep
      Do j=1,Ncntr
       write (4,104) j,xcen(Kstep,j),ycen(Kstep,j)
     +                ,peri(Kstep,j),epif(Kstep,j)
     +                ,ogos(Kstep,j)
      End Do

c-----------------
c compute velocity
c-----------------

      call vp_ax_vel
     +              (Istop
     +              )

      If(Istop.eq.1) Go to 99

c---
c printing session
c---

      If(Iprint.eq.Nprint) then

       Do j=1,Ncntr
        write (1,103) NSG1(j),j,time(Kstep),DT
          Do i = 1,NSG1(j)
            write (1,120) i,Xg(i,j),Yg(i,j),Ug(i,j),Vg(i,j)
          End Do
       End Do

       Iprint = 0

      End If

c---
c save
c---

      Do j=1,Ncntr
        Do i = 1,NSG1(j)
         Xsv(i,j) = Xg(i,j)
         Ysv(i,j) = Yg(i,j)
         Usv(i,j) = Ug(i,j)
         Vsv(i,j) = Vg(i,j)
          Xg(i,j) = Xg(i,j) + Dt*Ug(i,j)
          Yg(i,j) = Yg(i,j) + Dt*Vg(i,j)
        End Do
      End Do

c---
c second step in RK2
c---

      ICH1 = 0
      ICH2 = 0
      ICH3 = 0

      Do j=1,Ncntr      ! loop over contours

c---
c extend
c and prevent drifting
c compute the position of point 1 and NSG1
c by extrapolation
c---

      Xg(0,j) =  Xg(2,j)
      Yg(0,j) = -Yg(2,j)

      Xg(1,j) = (Xg(2,j)*Yg(3,j)**2-Xg(3,j)*Yg(2,j)**2)/
     +          (        Yg(3,j)**2-        Yg(2,j)**2)
      Yg(1,j) = 0.0
      Xg(NSG1(j),j) = (Xg(NSG(j)  ,j)*Yg(NSG(j)-1,j)**2
     +                -Xg(NSG(j)-1,j)*Yg(NSG(j)  ,j)**2)/
     +                (Yg(NSG(j)-1,j)**2-Yg(NSG(j),j)**2)
      Yg(NSG1(j),j) = 0.0

      Xg(NSG2(j),j) =  Xg(NSG(j),j)
      Yg(NSG2(j),j) = -Yg(NSG(j),j)

c---
c Transfer into prd
c---

        Do i=0,NSG2(j)
         x(i) = xg(i,j)
         y(i) = yg(i,j)
        End Do

c---------------
c Geometry at intermediate step
c--------------

        call prd_ax
     +
     +           (NSG(j)
     +           ,ICH1,THMAX
     +           ,ICH2,SPMAX
     +           ,ICH3,SPMIN
     +           ,peri,epif,volume,xcen,ycen
     +           ,unused1,unused2,unused3,unused4,unused5
     +           ,Istop
     +           )

      If(Istop.eq.1) Go to 99

c---
c transfer from prd
c---

        Do i =1,NSG1(j)
            xg(i,j) =    x(i)
            yg(i,j) =    y(i)
           xcg(i,j) =   xc(i)
           ycg(i,j) =   yc(i)
            rg(i,j) =    r(i)
            sg(i,j) =    s(i)
          th1g(i,j) =  th1(i)
          th2g(i,j) =  th2(i)
          th3g(i,j) =  th3(i)
         orntg(i,j) = ornt(i)
      End Do

c---
      End Do                    ! End of loop over drops
c---

c---------------------------
c Second velocity evaluation
c---------------------------

      call vp_ax_vel
     +              (Istop
     +              )

      If(Istop.eq.1) Go to 99

c---
c second step in RK2
c---

      Dth = 0.5*Dt

      Do j=1,Ncntr

        Do i=1,NSG(j)
         Xg(i,j) = Xsv(i,j)+Dth*(Usv(i,j)+Ug(i,j))
         Yg(i,j) = Ysv(i,j)+Dth*(Vsv(i,j)+Vg(i,j))
        End Do

c---
c extend
c and prevent drifting
c compute the position of point 1 and NSG1
c by extrapolation
c---

      Xg(0,j) =  Xg(2,j)
      Yg(0,j) = -Yg(2,j)

      Xg(1,j) = (Xg(2,j)*Yg(3,j)**2-Xg(3,j)*Yg(2,j)**2)/
     +          (        Yg(3,j)**2        -Yg(2,j)**2)
      Yg(1,j) = 0.0
      Xg(NSG1(j),j) = (Xg(NSG(j)  ,j)*Yg(NSG(j)-1,j)**2
     +                -Xg(NSG(j)-1,j)*Yg(NSG(j)  ,j)**2)/
     +                (Yg(NSG(j)-1,j)**2-Yg(NSG(j),j)**2)
      Yg(NSG1(j),j) = 0.0

       Xg(NSG2(j),j) =  Xg(NSG(j),j)
       Yg(NSG2(j),j) = -Yg(NSG(j),j)

      End Do

c---
c reset counters and time
c---

      Istep  = Istep+1
      Kstep  = Kstep+1
      Iprint = Iprint+1

      time(Kstep) = time(Kstep-1) + DT

      IF(Istep.le.Nstep) Go to 97

      Go to 90


c-.-.-.-.-.-.-.-.-.-.-.-.
c SIMULATION HAS ENDED
c-.-.-.-.-.-.-.-.-.-.-.-.

c---
c Finish up be printing final state
c---

  99  Continue

c---
c print marker points
c---

       open (9,file="vp_ax.rst")   ! restart file

       Do j=1,Ncntr
        write (1,103) NSG1(j),j,time(Kstep)
        write (9,103) NSG1(j),j,time(Kstep)
          Do i = 1,NSG1(j)
            write (1,120) i,Xg(i,j),Yg(i,j)
     +                     ,Ug(i,j),Vg(i,j)
            write (9,120) i,Xg(i,j),Yg(i,j)
     +                     ,Ug(i,j),Vg(i,j)
          End Do
       End Do

       write (1,112) Null
       write (9,112) Null

      close (9)

c---
c Print run parameters
c---

      thmax = thmax/pi    ! unscale

      write (1,105)  Ncntr
     +              ,NGL
     +              ,THmax,SPmax,SPmin
     +              ,Iread
     +              ,Ishape
     +              ,Norm
     +              ,Nprint
     +              ,DT

      write (1,*)
      write (1,106)
      write (1,*)
      Do j=1,Ncntr
       write (1,107) j,omega(j)
      End Do
      write (1,*)
      write (1,108)
      write (1,*)
      Do j=1,Ncntr
       write (1,107) j,xdc(j),req(j),awob(j),bwob(j)
      End Do

c---
c close down
c---

      write (1,110)
      write (4,102) Null

      close (1)
      If(Ienrd.eq.1) close(3)

 100  Format (1X,I2,1X,I2,10(1X,F10.5))
 101  Format (1X,'U = ',F15.10,'  V = ',F15.10)
 102  Format (1X,I4,5(1X,F12.8))
 120  Format (1X,I4,2(1X,F12.8),2(1X,F8.4),2(1X,F12.8))
 103  Format (1X,I5,1X,I2,3(1X,F15.5))
 104  Format (1X,I4,20(1X,F9.5))
 105  Format (/
     +       ,' Ncntr  = ',I2,/,/
     +       ,' NGL    = ',I1,/
     +       ,' THMAX  = ',F6.3,/
     +       ,' SPMAX  = ',F6.3,/
     +       ,' SPMIN  = ',F6.3,/
     +       ,' Iread  = ',I1,/
     +       ,' Ishape = ',I1,/
     +       ,' Norm   = ',I1,/
     +       ,' Nprint = ',I3,/
     +       ,' Dt     = ',F7.4,/
     +       )

 106  Format ('omega')
 107  Format (1x,i2,10(1x,f5.3))
 108  Format ('xdc, ydc, aob, req, awob, bwob')

 109  Format (' T=',F7.3,' S=',F10.7,' A=',F10.7,
     +        ' X=',F8.5,' Y=',F8.5)
 110  Format ( " PROGRAM VP_AX",/,/)
 111  Format ( " Executing step ",I4, " Total steps: ",I4)
 112  Format (1X,I4,1X,I4)
 113  Format (1X,F8.5,1X,I6)
 115  Format (10(1X,f10.5))

 140  Format (1X,I4,1x,f10.6,20(1X,F8.5))
 145  Format (1X,I4,1x,f15.6,2(1X,F8.5),1X,f15.5)

 169  Format (1X,F8.4,100(1x,f7.5))

 305  Format (' ARC LENGTHS   ',10(/,10(1x,F6.3)))
 306  Format (' SURFACE AREAS ',10(/,10(1x,F6.3)))
 307  Format (' X-CENTERS     ',10(/,10(1x,F6.3)))
 308  Format (' Y-CENTERS     ',10(/,10(1x,F6.3)))
 309  Format (' VOLUMES       ',10(/,10(1x,F6.3)))

      Stop
      End
