      program channel_vv_ftcs

c=============================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c=============================================

c--------------------------------------------
c Unidirectional flow in a channel
c Velocity vorticity formulation 
c for channel flow discussed by Pozrikidis 
c "Fluid Dynamics: Theory, Computation,
c                  and Numerical Simulation"
c Springer
c
c SYMBOLS:
c -------
c
c dody:   d(omega)\dy
c--------------------------------------------

      Implicit Double Precision(a-h,o-z)

      Dimension dody(128),dody_new(128),y(128)
      Dimension a(200),b(200),c(200),s(200),u(200)
      Dimension x(200)

c---
c constants
c---

      Null = 0

c---
c input
c---

      h = 1.0
      rmu = 1.0   ! viscosity
      rnu = 1.0   ! kinematic
      dpdx = -0.5

      write (6,*) " Enter the number of finite-difference divisions"
      write (6,*) " -----------------------------------------------"
      read  (5,*) N

      write (6,*) " Enter the numerical diffusion number: alpha"
      write (6,*) " (less than 0.5 to ensure numerical stability)"
      write (6,*) " -------------------------------------------"
      read  (5,*) alpha


      write (6,*) " Enter the number of steps"
      write (6,*) " -------------------------"
      read  (5,*) nstep

c---
c prepare
c---

      open (8,file="channel_vv_ftcs.out")

      N1 = N+1
      Dy = h/N

      Dt = alpha*Dy*Dy/rnu

c---
c grid points
c---

      Do i=1,N1
       y(i) = (i-1.0)*Dy
      End Do

c---
c initialize d omaga/d y
c---

      Do i=1,N1
       dody(i) = 0.0
      End Do

      time = 0.0

c---------
      Do istep=1,nstep
c---------

      U1    = 0.2*(1.0 - exp(-time))     ! wall is moving gradually
      dU1dt = 0.2*exp(-time)

      U2    = 0.1*(1.0 - exp(-2.0*time)) ! wall is moving gradually
      dU2dt = 0.1*(2.0*exp(-2.0*time))

      dody(1)  = -dU1dt/rnu - dpdx/rmu
      dody(N1) = -dU2dt/rnu - dpdx/rmu

      Do i=2,N
      dody_new(i) = dody(i)
     +        +alpha * (dody(i+1)-2.0*dody(i)+dody(i-1))
      End Do

      Do i=2,N
       dody(i) = dody_new(i)
      End Do

c------
c recover the velocity by solving a tridiagonal system
c------

      u(1)  = U1
      u(N1) = U2

      Do i=2,n

       ia = i-1
       s(ia) = -Dy**2 * dody(i)
       a(ia) = -2
       b(ia) =  1 
       c(ia) =  1
     
      End Do

      s(1)   = s(1)   - u(1)
      s(N-1) = s(N-1) - u(N1)

      call thomas (n-1,a,b,c,s,x)

      Do i=1,n-1
       u(i+1) = x(i)
      End Do

      !---
      ! record:
      !---

      write (8,*) N1
      write (6,*) N1

      Do i=1,N1
       write (8,100) i,y(i),u(i),dody(i)
       write (6,100) i,y(i),u(i),dody(i)
      End Do

      time = time+Dt

c--------
      End Do
c--------

c---
c done
c---

      write (8,100) Null
      close (8)

  100 Format (1x,i3,10(1x,f15.10))

      stop
      end
