      subroutine cvt_pm_press1
     +
     +  (Lx,Ly
     +  ,Nx,Ny
     +  ,Ulid
     +  ,visc
     +  ,rho
     +  ,Uamp
     +  ,IPBC
     +  ,IseePS
     +  ,Itermax
     +  ,tol
     +  ,P
     +  )

c==========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c==========================================

c--------------------------------------------
c Solve the pressure Poisson equation (PPE)
c with the Neumann boundary condition all around
c
c Boundary conditions:
c --------------------
c
c     df/dx =  pnbcl   at x = ax
c     df/dx = -pnbcr   at x = bx
c     df/dy =  pnbcb   at y = ay
c     df/dy = -pnbct   at y = by
c--------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Double Precision Lx,Ly

      Dimension  X(129,129), Y(129,129)
      Dimension  U(129,129), V(129,129)
      Dimension  Ulid(-1:131)

      Dimension  Divu(129,129)

      Dimension RNLx(0:130,0:130)  ! x-component of the
                                   ! NonLinear convection term
      Dimension RNLy(0:130,0:130)  ! y-component of the
                                   ! NonLinear convection term
      Dimension DivNL(129,129)     ! Div of the NonLinear convection term

      Dimension pnbcl(129),pnbcr(129)  ! left and right poisson
                                       ! Neumann boundary conditions
      Dimension pnbcb(129),pnbct(129)  ! bottom and top poisson
                                       ! Neumann boundary conditions
      Dimension P(0:130,0:130)

      Dimension Blin(16666)

c-------
c common
c-------

      common/XXYY/X,Y      
      common/UUVV/U,V 
      common/DIVE/Divu

      common/pnbc/pnbcl,pnbcr,pnbcb,pnbct  ! Poisson Neumann BC
      common/piii/pi,pi2,pi4

c--------
c prepare
c--------

      Nxb = Nx-2
      Nxa = Nx-1
      Nx1 = Nx+1
      Nx2 = Nx+2

      Nyb = Ny-2
      Nya = Ny-1
      Ny1 = Ny+1
      Ny2 = Ny+2

      Nodes = Nx1*Ny1   ! number of boundary and internal nodes

c----------
c steps etc
c----------

      Dx = Lx/(Nx1-1.0D0)
      Dy = Ly/(Ny1-1.0D0)

      Dx2 = 2.0D0*Dx
      Dy2 = 2.0D0*Dy

      Dxs = Dx**2
      Dys = Dy**2

c------------------------------
c initialize the nonlinear term
c------------------------------

      Do j=0,Ny2
       Do i=0,Nx2
        RNLx(i,j) = 0.0D0
        RNLy(i,j) = 0.0D0
       End Do
      End Do

c---------------------------
c compute the nonlinear term
c---------------------------

! interior nodes

      Do j=2,Ny

       ja = j-1
       j1 = j+1

       Do i=2,Nx

        ia = i-1
        i1 = i+1

c       RNLx(i,j) =  (U(i1,j)*U(i1,j)-U(ia,j)*U(ia,j))/Dx2
c    +            +  (U(i,j1)*V(i,j1)-U(i,ja)*V(i,ja))/Dy2
c       RNLy(i,j) =  (V(i1,j)*U(i1,j)-V(ia,j)*U(ia,j))/Dx2
c    +            +  (V(i,j1)*V(i,j1)-V(i,ja)*V(i,ja))/Dy2

        RNLx(i,j) =  U(i,j) * (U(i1,j)-U(ia,j))/Dx2
     +            +  V(i,j) * (U(i,j1)-U(i,ja))/Dy2
        RNLy(i,j) =  U(i,j) * (V(i1,j)-V(ia,j))/Dx2
     +            +  V(i,j) * (V(i,j1)-V(i,ja))/Dy2
       End Do

      End Do

! top

      Do i=0,Nx2
c       RNLx(i,j) = 0.0D0
        RNLx(i,Ny1) = Ulid(i) * (Ulid(i+1)-Ulid(i-1))/Dx2
      End Do

c--------------------------
c Compute: div(grad(uu))
c--------------------------

! interior nodes

      Do j=2,Ny
       Do i=2,Nx
        DivNL(i,j) = (RNLx(i+1,j)-RNLx(i-1,j))/Dx2
     +             + (RNLy(i,j+1)-RNLy(i,j-1))/Dy2
       End Do
      End Do

! left and right wall

      Do j=1,Ny1
       DDx = (-3.0D0*RNLx(1,j)+4.0D0*RNLx(2,j)-RNLx(3,j))/Dx2
       DDy = 0.0D0
       DivNL(1,j) = DDx+DDy
       DDx = (3.0D0*RNLx(Nx1,j)-4.0D0*RNLx(Nx,j)+RNLx(Nxa,j))/Dx2
       DDy = 0.0D0
       DivNL(Nx1,j) = DDx+DDy
c      write (6,106) j,DivNL(1,j),DivNL(Nx1,j)
      End Do

! bottom and top wall

      Do i=1,Nx1
       DDx        = 0.0D0
       DDy        = (-3.0D0*RNLy(i,1)+4.0D0*RNLy(i,2)-RNLy(i,3))/Dy2
       DivNL(i,1) = DDx+DDy
       DDx        = (RNLx(i+1,Ny1)-RNLx(i-1,Ny1))/Dx2
       DDy        = (3.0D0*RNLy(i,Ny1)-4.0D0*RNLy(i,Ny)+RNLy(i,Nya))/Dy2
       DivNL(i,Ny1) = DDx+DDy
c      write (6,106) i,DivNL(i,1),DivNL(i,Ny1)
      End Do

! upper two corners

      DivNL(  1,Ny1) =  0.5D0*(DivNL(2,Ny1)+DivNL(1,Ny))
      DivNL(Nx1,Ny1) =  0.5D0*(DivNL(Nx,Ny1)+DivNL(Nx1,Ny))

c--------
c display
c--------

      if(IseePS.eq.1) then

       write (6,*)
       write (6,*) " cvt_pm_press1: - rho * Div(NL)"
       write (6,*) " ------------------------------"
       write (6,*)

       fc = -rho

       Do j=1,Ny1
         write (6,100) (fc*DivNL(i,j),i=1,Nx1)
       End Do

       pause

      end if

c----------------------------------
c RHS of Poisson's equation for the
c pressure
c----------------------------------

      Ic = 0

      Do j=1,Ny1
       Do i=1,Nx1
        Ic = Ic+1
        Blin(Ic) = - rho*DivNL(i,j)
     +    - vsc*( (Divu(i+1,j)-2.0D0*Divu(i,j)+Divu(i-1,j))/Dxs
     +           +(Divu(i,j+1)-2.0D0*Divu(i,j)+Divu(i,j-1))/Dys)
        Blin(Ic) = Dxs*Blin(Ic)
       End Do
      End Do

c-----
c pressure Neumann boundary conditions
c left and right
c
c IMPORTANT:
c
c Choice IPBC = 0 is inaccurate
c-----

c     write (6,*) " cvt_pm_press1: PPE left and right BC"

      Do j=1,Ny1

        pnbcl(j) = 0.0D0
        pnbcr(j) = 0.0D0

        if(IPBC.eq.0) then
          pnbcl(j) =  2.0D0*visc*U(2,j)/Dxs
          pnbcr(j) = -2.0D0*visc*U(Nx,j)/Dxs
        else if(IPBC.eq.1) then
          pnbcl(j) =  4.0D0*visc*(U(2, j)-0.125D0*U(3,  j))/Dxs
          pnbcr(j) = -4.0D0*visc*(U(Nx,j)-0.125D0*U(Nxa,j))/Dxs
        else if(IPBC.eq.2) then
          pnbcl(j) =  visc*(U(1,  j)-2.0D0*U(2, j)+U(3,  j))/Dxs
          pnbcr(j) = -visc*(U(Nx1,j)-2.0D0*U(Nx,j)+U(Nxa,j))/Dxs
        Else If(IPBC.eq.3) then
          pnbcl(j) =  visc*(2.0D0*U(1,j)-5.0D0*U(2,j)
     +                     +4.0D0*U(3,j)-U(4,j))/Dxs
          pnbcr(j) = -visc*(2.0D0*U(Nx1,j)-5.0D0*U(Nx,j)
     +                     +4.0D0*U(Nxa,j)-U(Nxb,j))/Dxs
        else
          write (6,*) " cvt_press1: this option is not available"
          stop
        end if

c       write (6,106) j,pnbcl(j),pnbcr(j)

      End Do

c-----
c pressure Neumann boundary conditions
c bottom and top
c-----

c     write (6,*) " cvt_press1: PPE bottom and top BC"

      Do i=1,Nx1

        i1 =i+1
        ia =i-1

c       arg = pi2*X(i,Ny1)/Lx
c       DuDx  = 0.50D0*Uamp*sin(arg) * pi2/Lx
        DuDx = (Ulid(i1)-Ulid(ia))/Dx2
        DuDx = (4.0D0*V(i,Ny)-V(i,Nya))/Dy2

        if(IPBC.eq.0) then
          pnbcb(i) =  2.0D0*visc*V(i,2) /Dys
          pnbct(i) = -2.0D0*visc*(V(i,Ny)-Dy*DuDx)/Dys
        else if(IPBC.eq.1) then
          pnbcb(i) =  4.0D0*visc*(V(i,2)-0.125D0*V(i,3))/Dys
          pnbct(i) = -4.0D0*visc*(V(i,Ny)-0.125D0*V(i,Nya)
     +                           -0.75D0*Dy*DuDx )/Dys
        else if(IPBC.eq.2) then
          pnbcb(i) =  visc*(V(i,  1)-2.0D0*V(i, 2)+V(i,  3))/Dys
          pnbct(i) = -visc*(V(i,Ny1)-2.0D0*V(i,Ny)+V(i,Nya))/Dys
        else if(IPBC.eq.3) then
          pnbcb(i) =  visc*(2.0D0*V(i,1)-5.0D0*V(i,2)
     +                     +4.0D0*V(i,3)-V(i,4) )/Dys
          pnbct(i) = -visc*(2.0D0*V(i,Ny1)-5.0D0*V(i,Ny)
     +                     +4.0D0*V(i,Nya)-V(i,Nyb))/Dys
        else
          write (6,*) " cvt_pm_press1: this option not available"
          stop
        end if

c       write (6,106) i,pnbcb(i),pnbct(i)

      End Do

c----
c RHS of Poisson's equation for the pressure
c incorporating the inhomogeneous
c Neumann boundary conditions
c----

      beta = Dxs/Dys

c bottom

      Ic = 0
      Do i=1,Nx1
        Ic = Ic+1
        Blin(Ic) = Blin(Ic)+beta*Dy2*pnbcb(i)
      End Do

c left and right

      Ic = 0
      Do j=1,Ny1
        Ic = Ic+1
        Blin(Ic) = Blin(Ic)+Dx2*pnbcl(j)
        Ic = Ic+Nx
        Blin(Ic) = Blin(Ic)+Dx2*pnbcr(j)
      End Do

c top

      Ic = Nodes-Nx1
      Do i=1,Nx1
        Ic = Ic+1
        Blin(Ic) = Blin(Ic)+beta*Dy2*pnbct(i)
      End Do

c--------
c Display
c--------

      Go to 333

      write (6,*)
      write (6,*) " cvt_press1: RHS: "
      write (6,*)

      Ic = 0
      Do i=1,Nx1
        Do j=1,Ny1
          Ic = Ic+1
          write (6,118) Ic,Blin(Ic)
         End Do
      End Do
      pause
 333  Continue

c----------------------
c Gauss-Siedel solution
c----------------------

      call pois_gs1
     +
     +  (Nx,Ny
     +  ,Dx,Dy
     +  ,Blin
     +  ,Itermax     ! maximum number of GS iterations
     +  ,tol         ! GS tolerance
     +  ,P           ! solution matrix
     +  )

c--------
c Display
c--------

      Go to 666

      write (6,*)
      write (6,*) " cvt_press1: pressure:"
      write (6,*)

      Psft = 0.0D0

      Do j=1,Ny1
        write (6,110) (P(i,j)-Psft,i=1,Nx1)
      End Do

      pause
 666  Continue

c-----
c Done
c-----

 100  Format (98(1x,f8.4))
 110  Format (30(1x,f8.3))
 118  Format (1x,i4,F12.8)

      return
      end
