      subroutine cvt_pm_press2
     +
     +  (Lx,Ly
     +  ,Nx,Ny
     +  ,Ulid
     +  ,visc
     +  ,rho
     +  ,Uamp
     +  ,IGhia
     +  ,IPBC
     +  ,IseePS
     +  ,Itermax
     +  ,tol
     +  ,P
     +  )

c===========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c===========================================

c--------------------------------------------
c Solve the pressure Poisson equation (PPE)
c
c Boundary conditions:
c --------------------
c
c     df/dx =  pnbcl   at x = ax
c     df/dx = -pnbcr   at x = bx
c     df/dy =  pnbcb   at y = ay
c     df/dy = -pnbct   at y = by

c--------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Double Precision Lx,Ly

      Dimension  X(129,129), Y(129,129)
      Dimension  U(129,129), V(129,129)
      Dimension       Ulid(-1:131)

      Dimension RNLx(0:130,0:130)  ! x-component of the
                                 ! NonLinear convection term
      Dimension RNLy(0:130,0:130)  ! y-component of the
                                 ! NonLinear convection term
      Dimension DivNL(129,129)    ! Div of the NonLinear convection term

      Dimension pnbcl(129),pnbcr(129)  ! left and right poisson
                                     ! Neumann boundary conditions
      Dimension pnbcb(129),pnbct(129)  ! bottom and top poisson
                                     ! Neumann boundary conditions
      Dimension P(0:130,0:130)

      Dimension Blin(16666)

c-------
c common
c-------

      common/XXYY/X,Y      
      common/UUVV/U,V 

      common/pnbc/pnbcl,pnbcr,pnbcb,pnbct  ! Poisson Neumann BC
      common/piii/pi,pi2,pi4

c--------
c prepare
c--------

      Nxb = Nx-2
      Nxa = Nx-1
      Nx1 = Nx+1
      Nx2 = Nx+2

      Nyb = Ny-2
      Nya = Ny-1
      Ny1 = Ny+1
      Ny2 = Ny+2

      Nodes = Nxa*Nya   ! number of boundary and internal nodes

c----------
c steps etc
c----------

      Dx = Lx/(Nx1-1.0D0)
      Dy = Ly/(Ny1-1.0D0)

      Dx2 = 2.0D0*Dx
      Dy2 = 2.0D0*Dy

      Dxs = Dx**2
      Dys = Dy**2

c------------------------------
c initialize the nonlinear term
c------------------------------

      Do j=1,Ny1
       Do i=1,Nx1
        RNLx(i,j) = 0.0D0
        RNLy(i,j) = 0.0D0
       End Do
      End Do

c---------------------------
c compute the nonlinear term
c---------------------------

! interior nodes

      Do j=2,Ny

       ja = j-1
       j1 = j+1

       Do i=2,Nx

        ia = i-1
        i1 = i+1

c       RNLx(i,j) =  (U(i1,j)*U(i1,j)-U(ia,j)*U(ia,j))/Dx2
c    +            +  (U(i,j1)*V(i,j1)-U(i,ja)*V(i,ja))/Dy2
c       RNLy(i,j) =  (V(i1,j)*U(i1,j)-V(ia,j)*U(ia,j))/Dx2
c    +            +  (V(i,j1)*V(i,j1)-V(i,ja)*V(i,ja))/Dy2

        RNLx(i,j) =  U(i,j) * (U(i1,j)-U(ia,j))/Dx2
     +            +  V(i,j) * (U(i,j1)-U(i,ja))/Dy2
        RNLy(i,j) =  U(i,j) * (V(i1,j)-V(ia,j))/Dx2
     +            +  V(i,j) * (V(i,j1)-V(i,ja))/Dy2
       End Do

      End Do

! top

      Do i=2,Nx
c       RNLx(i,j) = 0.0D0
        RNLx(i,Ny1) = Ulid(i) * (Ulid(i+1)-Ulid(i-1))/Dx2
      End Do

c--------------------------
c Compute: div(grad(uu))
c--------------------------

      Do j=2,Ny
       Do i=2,Nx
        DivNL(i,j) = (RNLx(i+1,j)-RNLx(i-1,j))/Dx2
     +             + (RNLy(i,j+1)-RNLy(i,j-1))/Dy2
       End Do
      End Do

c--------
c display
c--------

      If(IseePS.eq.1) then

       write (6,*)
       write (6,*) " - rho * Div(NL)"
       write (6,*) " ---------------"
       write (6,*)

       fc = -rho

       Do j=1,Ny1
         write (6,100) (fc*DivNL(i,j),i=1,Nx1)
       End Do

      End If

c----------------------------------
c RHS of Poisson's equation for the
c pressure
c----------------------------------

      Ic = 0

      Do j=2,Ny
       Do i=2,Nx
        Ic = Ic+1
        Blin(Ic) = - rho*DivNL(i,j)
        Blin(Ic) = Dxs*Blin(Ic)
       End Do
      End Do

c---------------------------------------------
c Set the pressure Neumann boundary conditions
c---------------------------------------------

c-------------------------------
c IGhia = 0:
c
c PBC is is applied at the walls
c-------------------------------

       If(IGhia.eq.0) then

c-----
c left and right
c-----

      Do j=2,Ny

        If(IPBC.eq.0) then
          pnbcl(j) =  2.0D0*visc*U(2 ,j)/Dxs
          pnbcr(j) = -2.0D0*visc*U(Nx,j)/Dxs
        Else If(IPBC.eq.1) then
          pnbcl(j) =  4.0D0*visc*(U(2, j)-0.125D0*U(3,  j))/Dxs
          pnbcr(j) = -4.0D0*visc*(U(Nx,j)-0.125D0*U(Nxa,j))/Dxs
        Else If(IPBC.eq.2) then
          pnbcl(j) =  visc*(U(1,  j)-2.0D0*U(2, j)+U(3,  j))/Dxs
          pnbcr(j) = -visc*(U(Nx1,j)-2.0D0*U(Nx,j)+U(Nxa,j))/Dxs
        Else If(IPBC.eq.3) then
          pnbcl(j) =  visc*(2.0D0*U(1,j)-5.0D0*U(2,j)
     +                     +4.0D0*U(3,j)-U(4,j))/Dxs
          pnbcr(j) = -visc*(2.0D0*U(Nx1,j)-5.0D0*U(Nx,j)
     +                     +4.0D0*U(Nxa,j)-U(Nxb,j))/Dxs
        Else
          write (6,*) " cvt_pm_press2: this option is not available"
          stop
        End If

c       write (6,*) j,pnbcl(j),pnbcr(j)

      End Do

c-----
c bottom and top
c-----

      Do i=2,Nx

        i1 =i+1
        ia =i-1

c       arg = pi2*X(i,Ny1)/Lx
c       DuDx  = 0.50*UUlid*sin(arg) * pi2/Lx
        DuDx = (Ulid(i1)-Ulid(ia))/Dx2
        DuDx = (4.0D0*V(i,Ny)-V(i,Nya))/Dy2

        If(IPBC.eq.0) then
          pnbcb(i) =  2.0D0*visc*V(i,2)/Dys
          pnbct(i) = -2.0D0*visc*(V(i,Ny) - Dy*DuDx )/Dys
        Else If(IPBC.eq.1) then
          pnbcb(i) =  4.0D0*visc*(V(i, 2)-0.125D0*V(i, 3))/Dys
          pnbct(i) = -4.0D0*visc*(V(i,Ny)-0.125D0*V(i,Nya)
     +                           -0.75D0*Dy*DuDx )/Dys
        Else If(IPBC.eq.2) then
          pnbcb(i) =  visc*(V(i,  1)-2.0D0*V(i, 2)+V(i,  3))/Dys
          pnbct(i) = -visc*(V(i,Ny1)-2.0D0*V(i,Ny)+V(i,Nya))/Dys
        Else If(IPBC.eq.3) then
          pnbcb(i) =  visc*(2.0D0*V(i,1)-5.0D0*V(i,2)
     +                     +4.0D0*V(i,3)-V(i,4))/Dys
          pnbct(i) = -visc*(2.0D0*V(i,Ny1)-5.0D0*V(i,Ny)
     +                     +4.0D0*V(i,Nya)-V(i,Nyb))/Dys
        Else
          write (6,*) " cvt_pm_press2: this option not available"
          stop
        End If

c       write (6,*) i,pnbcb(i),pnbct(i)

      End Do

c-------------------------------------
c IGhia = 1:
c
c Ghia's method
c the pnbc is applied midway between
c the wall and adjacent layers
c-------------------------------------

       Else If(IGhia.eq.1) then

c-----
c left and right walls
c-----

c     write (6,*) " cvt_press2: PPE left and right BC"

      Do j=2,Ny

       j1 = j+1
       ja = j-1

        Um = 0.5D0*(U(2,j)+U(1,j))
        Vm = 0.5D0*(V(2,j)+V(1,j))

        pnbcl(j) = - rho* ( Um*(U(2,j)-U(1,j))/Dx
     +    + 0.25D0*Vm* (U(2,j1)+U(1,j1)-U(2,ja)-U(1,ja))/Dy )
c    +    +  visc * 0.50D0 * (U(3,j)-U(1,j))/Dxs
     +    +  visc * (U(2,j)+0.25D0*U(3,j))/Dxs
     +    +  visc * 0.50D0 * (U(2,j1)-2.0D0*U(2,j)+U(2,ja))/Dys

        Um = 0.5D0*(U(Nx1,j)+U(Nx,j))
        Vm = 0.5D0*(V(Nx1,j)+V(Nx,j))

        pnbcr(j) = rho* ( Um*(U(Nx1,j)-U(Nx,j))/Dx
     +    + 0.25D0*Vm* (U(Nx,j1)+U(Nx1,j1)-U(Nx,ja)-U(Nx1,ja))/Dy )
c    +   - visc * 0.50D0 * (U(Nxa,j)-U(Nx1,j))/Dxs
     +   - visc * (U(Nxa,j)+0.25D0*U(Nxb,j))/Dxs
     +   - visc * 0.50D0 * (U(Nx,j1)-2.0D0*U(Nx,j)+U(Nx,ja))/Dys

c       write (6,*) j,pnbcl(j),pnbcr(j)

      End Do

c-----
c bottom and top
c-----

      Do i=2,Nx

        i1 =i+1
        ia =i-1

c       arg = pi2*X(i,Ny1)/Lx
c       DuDx  = 0.50*UUlid*sin(arg) * pi2/Lx
        DuDx = (Ulid(i1)-Ulid(ia))/Dx2

c---
        Um = 0.5D0*(U(i,2)+U(i,1))
        Vm = 0.5D0*(V(i,2)+V(i,1))

        pnbcb(i) = - rho* ( Vm*(V(i,2)-U(i,1))/Dy
     +   + 0.25D0*Um* (V(i1,2)+V(i1,1)-V(ia,2)-V(ia,1))/Dx )
c    +   + visc * 0.50D0 * (V(i,3)-V(i,1))/Dys
     +   + visc * (V(i,2)+0.25D0*V(i,3))/Dys
     +   + visc * 0.50D0 * (V(i1,2)-2.0D0*V(i,2)+V(ia,2))/Dxs

        Um = 0.5D0*(U(i,Ny1)+U(i,Ny))
        Vm = 0.5D0*(V(i,Ny1)+V(i,Ny))

        pnbct(i) = rho* ( Vm*(V(i,Ny1)-V(i,Ny))/Dy
     +    + 0.25D0*Um*(V(i1,Ny1)+V(i1,Ny)-V(ia,Ny1)-V(ia,Ny))/Dx )
c    +   - visc * 0.50D0 * (V(i,Nya)-V(i,Ny1)-2.0D0*Dy*DuDx)/Dys
     +   - visc * (V(i,Nya)+0.25D0*V(i,Nyb) - 1.5D0*Dy*DuDx)/Dys
     +   - visc * 0.50D0 * (V(i1,Ny)-2.0D0*V(i,Ny)+V(ia,Ny))/Dxs

      End Do

c-----------
      End If
c-----------

c-------------------------------------------
c Compute the RHS
c of Poisson's equation for the pressure
c incorporating the inhomogeneous
c Neumann boundary conditions
c--------------------------------------------

      beta = Dxs/Dys

c-------
c bottom
c-------

      Ic = 0

      Do i=2,Nx

        Ic = Ic+1
        If(IGhia.eq.0) then
          Blin(Ic) = Blin(Ic) + beta*Dy2*pnbcb(i)/3.0D0
        Else If(IGhia.eq.1) then
          Blin(Ic) = Blin(Ic) + beta*Dy*pnbcb(i)
        Else
          write (6,*) " cvt_pm_press2: method not implemented"
        End If

      End Do

c --------------
c left and right
c --------------

      Ic = 0

      Do j=2,Ny
        Ic = Ic+1

        If(IGhia.eq.0) then
          Blin(Ic) = Blin(Ic) + Dx2*pnbcl(j)/3.0D0
          Ic = Ic+Nx-2
          Blin(Ic) = Blin(Ic) + Dx2*pnbcr(j)/3.0D0
        Else If(IGhia.eq.1) then
          Blin(Ic) = Blin(Ic) + Dx*pnbcl(j)
          Ic = Ic+Nx-2
          Blin(Ic) = Blin(Ic) + Dx*pnbcr(j)
        Else
          write (6,*) " cvt_pm_press2: method not implemented"
        End If

      End Do

c----
c top
c----

      Ic = Nodes-Nx+1

      Do i=2,Nx    ! loop Nx-1 times

        Ic = Ic+1

        If(IGhia.eq.0) then
          Blin(Ic) = Blin(Ic) + beta*Dy2*pnbct(i)/3.0D0
        Else If(IGhia.eq.1) then
          Blin(Ic) = Blin(Ic) + beta*Dy*pnbct(i)
        Else
          write (6,*) " Method not implemented"
        End If

      End Do

c--------
c Display
c--------

      Go to 333

      write (6,*)
      write (6,*) " cvt_press2: RHS: "
      write (6,*)

      Ic = 0
      Do i=1,Nx1
        Do j=1,Ny1
          Ic = Ic+1
          write (6,118) Ic,Blin(Ic)
         End Do
      End Do

      pause

 333  Continue

c----------------------
c Gauss-Siedel solution
c----------------------

      call pois_gs2
     +
     +  (Nx,Ny
     +  ,Dx,Dy
     +  ,Blin
     +  ,IGhia
     +  ,Itermax     ! maximum number of GS iterations
     +  ,tol         ! GS tolerance
     +  ,P           ! solution matrix
     +  )

c--------
c Display
c--------

      Go to 666

      write (6,*)
      write (6,*) " cvt_press2: pressure:"
      write (6,*)

      Poff = 0.0D0

      Do j=1,Ny1
        write (6,110) (P(i,j)-Poff,i=1,Nx1)
      End Do

      pause
 666  Continue

c-----
c Done
c-----

 100  Format (98(1x,f8.4))
 110  Format (30(1x,f8.3))
 118  Format (1x,i4,F12.8)

      Return
      End
