      program stats

c-----------------------------------------
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licencing agreement
c----------------------------------------

      Implicit Double Precision (a-h, o-z)

      Dimension x(32768),f(32768,2),z(32768)
      Dimension u(32768),v(32768),T(32768)
      Dimension g(0:32768,2)
      Dimension af(0:32768),bf(0:32768),cfm(0:32768)

c----------
c constants
c----------

      pi = 3.14159265358
      pi2 = 2*pi

      null = 0

c--------
c prepare
c--------

      open (1,file="stats.out")

c------------
c preferences
c------------

  98  Continue

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 to compute mean and RMS values"
      write (6,*) "11 to compute the FFT of w"
      write (6,*) "12 to compute the FFT of u"
      write (6,*) "13 to compute the FFT of T"
      write (6,*) " 0 to quit"
      write (6,*) " ---------"
      read (5,*) menu

      if(menu.eq.0) Go to 99

c--------------
c read the data
c---------------

       open (2,file="keller.dat")

       read (2,*) n,Dt

       write (6,*)
       write (6,*) " Number of data points : ",n

       Do i=1,n
         x(i) = (i-1.0)*Dt
         read (2,*) idle,u(i),v(i),T(i)
         f(i,2) = 0.0
       End Do

       close (2)

c---------------
c reset data set
c---------------

  92   Continue

       write (6,*)
       write (6,*) " Analysis will be done on 2^Nexp points"
       write (6,*)
       write (6,*) " Please enter Nexp"
       write (6,*) " -----------------"
       read  (5,*) nexp

       nsave = n
       n = 2**nexp

       if(n.gt.nsave) then
         write (6,*)
         write (6,*) " Please enter a lower nexp"
         write (6,*)
         Go to 92
       end if

c-------------------------
c compute averages and RMS
c--------------------------

      if(menu.eq.1) then

      umean=0.0;
      vmean=0.0;
      Tmean=0.0;

      Do i=1,n
       umean=umean+u(i)
       vmean=vmean+v(i)
       Tmean=Tmean+T(i)
      End Do

      umean=umean/n
      vmean=vmean/n
      Tmean=Tmean/n

      urms=0.0;
      vrms=0.0;
      Trms=0.0;

      Do i=1,n
       urms=urms+(u(i)-umean)**2
       vrms=vrms+(v(i)-vmean)**2
       Trms=Trms+(T(i)-Tmean)**2
      End Do

      urms=sqrt(urms)/n;
      vrms=sqrt(vrms)/n;
      Trms=sqrt(Trms)/n;

      write (6,*) " Mean of ux: ",umean
      write (6,*) " Mean of uy: ",vmean
      write (6,*) " Mean of  T: ",Tmean
      write (6,*) " RMS  of ux: ",urms
      write (6,*) " RMS  of uy: ",vrms
      write (6,*) " RMS  of  T: ",Trms

c-------------
c compute FFTs
c-------------

      else

      Do i=1,n
         if(menu.eq.11) then
          f(i,1) = u(i)
         elseif(menu.eq.12) then
          f(i,1) = v(i)
         else
          f(i,1) = T(i)
         end if
         f(i,2) = 0.0D0
      End Do

c-------
c reduce
c-------

      n1 = n+1
      fc = 1/(n1-1.0D0)

      Do i=1,n
       g(i-1,1) = fc*f(i,1)
       g(i-1,2) = fc*f(i,2)
      End Do

c-----------
c Do the fft
c-----------

      call fft 
     +
     +  (nexp
     +  ,g
     +  ,af,bf
     +  )

c---------
c printing
c---------

      write (6,*)
      write (6,*) " Fourier Coefficients"
      write (6,*)
      write (6,*) "  i, a(i), b(i), |c(i)|"
      write (6,*) " ----------------------"
      write (6,*)

      write (1,101) n-1
      write (6,101) n-1

      Do i=1,n-1

        cfm(i) = sqrt(af(i)*af(i)+bf(i)*bf(i))   ! magnitude of power spectrum

        xprint = pi2*i/Dt
        yprint = cfm(i)*cfm(i)

        write (6,101) i,xprint,af(i),bf(i),yprint
        write (1,101) i,xprint,af(i),bf(i),yprint
      End Do

      write (1,101) null

      close (1)

c----------------------
c Confirm interpolation
c----------------------

      nh = n/2

      write (6,*)
      write (6,*) " Verified values"
      write (6,*)
      write (6,*) " i, x, f, F"
      write (6,*) " ----------"

      wn = pi2/(n*Dt)   ! fundamental wave number

      Do i=1,n
        xh = x(i)-x(1)     ! hat{x}
        z(i) = 0.5*af(0)
        Do ll=1,nh
         arg  = ll*wn*xh
         cs   = cos(arg)
         sn   = sin(arg)
         z(i) = z(i)+af(ll)*cs+bf(ll)*sn
        End Do
        write (6,101) i,x(i),f(i,1),z(i)
      End Do

c-------
c repeat
c-------

      End If      ! loop over menu

      Go to 98

c-----------
c pack it in
c-----------

  99  Continue

 100  Format (1x,f10.5,1x,f10.5,1x,f10.5)
 101  Format (1x,I8,1x,f16.5,10(1x,f10.5))

      Stop
      End
